/*
 * i386/linux/jit-md.h
 * Linux i386 JIT configuration information.
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#ifndef __i386_linux_jit_md_h
#define __i386_linux_jit_md_h

/**/
/* Include common information. */
/**/
#include "i386/jit.h"

/**/
/* Extra exception handling information. */
/**/
#if defined(HAVE_FEATURES_H)
#include <features.h>
#endif
#if defined(HAVE_ASM_SIGNAL_H) && !defined(__GLIBC__)
#include <asm/signal.h>
#endif
#if defined(HAVE_SIGNAL_H)
#include <signal.h>
#endif
#if defined(HAVE_ASM_SIGCONTEXT_H) && !defined(__GLIBC__)
#include <asm/sigcontext.h>
#endif
#if defined(HAVE_SIGCONTEXT_H)
#include <sigcontext.h>
#endif

/*
 * Lets play "Guess what the fucking kernel is going to give a signal
 * handler, cuz its never seems to be the same between any release
 * of linsucks."
 */

#if defined(SA_SIGINFO)

#define __USE_GNU
#include <ucontext.h>

#define EXCEPTIONPROTO int sig, siginfo_t *si, struct ucontext *ctx

/* Get the first exception frame from a signal handler */
#define	EXCEPTIONFRAME(f, c)						\
	(f).retbp = (c)->uc_mcontext.gregs[REG_EBP];	       		\
	(f).retpc = (c)->uc_mcontext.gregs[REG_EIP] + 1

/* Get the stack pointer from a sigcontext */
#define EXCEPTIONSP(c) (c)->uc_mcontext.gregs[REG_ESP]

/* Get the frame pointer from a sigcontext */
#define EXCEPTIONBP(c) (c)->uc_mcontext.gregs[REG_EBP]

/* Change a sigcontext to return to a different function/stack. */
#define EXCEPTION_ALT_FRAME(ctx, new_pc, new_sp) \
do { \
    (ctx)->uc_mcontext.gregs[REG_EIP] = (int)(new_pc); \
    (ctx)->uc_mcontext.gregs[REG_ESP] = (int)(new_sp); \
} while(0)

#else

/* Function prototype for signal handlers */
#if defined(HAVE_STRUCT_SIGCONTEXT_STRUCT) && !defined(__GLIBC__)
/* Linux < 2.1.1 */
#define	EXCEPTIONPROTO							\
	int sig, struct sigcontext_struct ctx

#elif defined(HAVE_STRUCT_SIGCONTEXT) || defined(__GLIBC__)
/* Linux >= 2.1.1  or Linux 2.0.x with glibc2 */
#define	EXCEPTIONPROTO							\
	int sig, struct sigcontext ctx
#else
#error Do not know how to define EXCEPTIONPROTO
#endif

/* Get the first exception frame from a signal handler */
#define	EXCEPTIONFRAME(f, c)						\
	(f).retbp = (c).ebp;						\
	(f).retpc = (c).eip + 1

#endif

/* Initialize an exceptionFrame from a stack frame pointer. */
#define EXCEPTION_TOP_FRAME(f, bp) \
do { \
    (f)->retbp = (uintp)bp; \
    (f)->retpc = 0; \
} while(0)

/* Jthreads uses guard pages. */
#if defined(HAVE_MMAP) && defined(UNIX_JTHREADS2)
#undef STACK_LIMIT
#endif

#endif
