/*
 * fsResource.c
 * Fs resource for teams, wraps the unix-jthreads2 fs object.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _fs_resource_h
#define _fs_resource_h

#include "jthread.h"

#if defined(UNIX_JTHREADS2)

#include "resource.h"

/*
 * The fsResource.
 *
 * link - The standard resource header.
 */
typedef struct _fsResource {
	resource link;
} fsResource;

/*
 * A fsResourceClient holds the fs specific data for a team.
 *
 * client - The standard client header.
 * js - The fs object for this team.
 */
typedef struct _fsResourceClient {
	resourceClient client;
	iLock *lock;
	jsked *js;
	char *rootDirectory;
} fsResourceClient;

/*
 * A fsResourceSpec holds the scheduling specification for a fs.
 *
 * spec - The standard resourceSpec header.
 * jr - The fsReservation specification.  XXX Should be inlined.
 */
typedef struct _fsResourceSpec {
	resourceSpec spec;
	char *rootDirectory;
} fsResourceSpec;

extern fsResource metalFSResource;

#define FS_RESOURCE_TEAM_ARGUMENT \
	"fs.root:string - The root directory for the team.\n"

/*
 * AddModule: fsResourceModule
 * Conditional: defined(UNIX_JTHREADS2)
 *   ModuleDependency: jteamModule.initMetal
 *   ModuleDependency: memResourceModule.initMetal
 *   ModuleDependency: jskedResourceModule.initMetal
 * Transition: initMetal
 *   ModuleDependency: threadModule.jthread
 * Transition: reinitMetal
 *   ModuleDependency: resourceModule.reinstall
 */

#endif

#endif
