/*
 * jskedResource.c
 * JSked resource for teams, wraps the unix-jthreads2 jsked object.
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "config.h"

#include <sys/types.h>
#include <netinet/in.h>

#include "assert_pp.h"
#include "tags.h"
#include "memResource.h"
#include "jskedResource.h"
#include "jteam.h"
#include "module.h"
#include "moduleList.h"
#include "soft.h"
#include "jsyscall.h"
#include "logResource.h"
#include "nativeClass.h"
#include "nativeClass-inline.h"

#if defined(UNIX_JTHREADS2)
#include "jthread.h"
#include "jthreadState.h"
#include "jthreadScheduler.h"
#include "jsked.h"

#include "jskedPriority.h"
#include "jskedStride.h"

#include "edu_utah_janosvm_resources_CPUSpec.h"
#include "edu_utah_janosvm_resources_StrideCPUSpec.h"
#include "edu_utah_janosvm_resources_CPUResource.h"

#include "edu_utah_janosvm_resources_IOSpec.h"
#include "edu_utah_janosvm_resources_IOResource.h"
#include "edu_utah_janosvm_resources_DescriptorIOSpec.h"

#define JAVA_CPU_SPEC "edu/utah/janosvm/resources/CPUSpec"

#define JAVA_DIO_SPEC "edu/utah/janosvm/resources/DescriptorIOSpec"

#if defined(JSKED_STRIDE)
jskedStrideReservation jskedKernelReservation = {
    edu_utah_janosvm_resources_StrideCPUSpec_KERNEL_TICKETS,
};
#endif
#if defined(JSKED_PRIORITY)
jskedPriorityReservation jskedKernelReservation = {
    128
};
#endif

/** Team fd property **/
jsiPropertyInfo fdRootProperties[] = {
	{ "fds", (1L << JSI_LIST_PROPERTIES), "",
	  "List the file descriptors allocated to this team.\n" },
	{ "fd", (1L << JSI_GET_PROPERTY), "index:int",
	  "Get information about the given file descriptor.\n" },
	{ 0 }
};

static
char *handleFDMessage(jsiHandlerContext *hc,
		      jsiHandler *jh,
		      jsiMessage *jm);
static
int getFDSuites(jsiHandler *jh, jsiMessage *jm);

static jsiHandlerFunctions fdFunctions;
static jsiHandlerFunctions fdsFunctions;
static jsiNamedHandler fdHandler;
static jsiNamedHandler fdsHandler;

/** Module declarations **/
static int initJskedResourceModule(int stage);
static char *configJskedResourceModule(void);

module_t jskedResourceModule = {
	"jsked",
	0,
	initJskedResourceModule,
	0,
	0,
	0,
	configJskedResourceModule,
};

/** Resource declarations **/
static struct _resourceSpec *jskedCreateJSISpec(struct _resource *res,
						jsiMessage *jm);
static void jskedDeleteJSISpec(struct _resource *res,
			       struct _resourceSpec *rs);
static int jskedSubscribe(struct _resource *res,
			  struct _resourceClient *rc,
			  struct _resourceSpec *rs);
static int jskedEnableResource(struct _resource *res,
			       struct _resourceClient *rc);
static void jskedUnsubscribe(struct _resource *res,
			     struct _resourceClient *rc);
static char *jskedHandleJSIQuery(struct _resourceClient *rc,
				 jsiMessage *jm);

static resourceFunctions jskedResourceFunctions = {
	0,
	0,
	0,
	0,
	jskedCreateJSISpec,
	jskedDeleteJSISpec,
	0,
	jskedSubscribe,
	jskedEnableResource,
	jskedUnsubscribe,
	jskedHandleJSIQuery,
};

/** jskedResource variables **/

jskedResource metalJskedResource;

jskedResource metalIOResource;

jskedResourceClient metalJskedResourceClient = {
	{
		0,
		0,
		&metalJskedResource.link,
		0
	},
	0
};

jioResourceClient metalIOResourceClient = {
	{
		0,
		0,
		&metalIOResource.link,
		0
	},
};

int initJskedResourceModule(int stage)
{
	int retval = 0;
	
	switch( stage )
	{
	case SID_JSKEDRESOURCEMODULE_INITMETAL:
		installResource(&metalJskedResource.link,
				RA_Name, "cpu",
				RA_Flags, RF_INTERNAL | RF_KERNEL_CLIENT,
				RA_Functions, &jskedResourceFunctions,
				RA_ClientSize, sizeof(jskedResourceClient),
				RA_SpecSize, sizeof(jskedResourceSpec),
				RA_TAG_DONE);
		metalTeam->internalResources[metalJskedResource.link.
					    internalIndex] =
			&metalJskedResourceClient.client;
		metalJskedResourceClient.client.owner = metalTeam;
		
		installResource(&metalIOResource.link,
				RA_Name, "io",
				RA_Flags, RF_INTERNAL | RF_KERNEL_CLIENT,
				RA_Functions, &jskedResourceFunctions,
				RA_ClientSize, sizeof(jioResourceClient),
				RA_SpecSize, sizeof(jioResourceSpec),
				RA_TAG_DONE);
		metalTeam->internalResources[metalIOResource.link.
					    internalIndex] =
			&metalIOResourceClient.client;
		metalIOResourceClient.client.owner = metalTeam;
		retval = 1;
		break;
	case SID_JSKEDRESOURCEMODULE_REINITMETAL:
		metalJskedResourceClient.js = jskedCurrent();
		jthread_set_context_jsked(jthread_current(), jskedCurrent());
		retval = 1;
		break;
	default:
		installResource(&metalJskedResource.link,
				RA_SpecNativeClass, JAVA_CPU_SPEC,
				RA_TAG_DONE);
		installResource(&metalIOResource.link,
				RA_SpecNativeClass, JAVA_DIO_SPEC,
				RA_TAG_DONE);
		
		jsiInitNamedHandler(&fdHandler,
				    JNA_Name, "fd",
				    JNA_Functions, &fdFunctions,
				    JNA_HandleMessage, handleFDMessage,
				    JNA_GetSuites, getFDSuites,
				    JNA_TAG_DONE);
		jsiInstallHandler(&teamHandlerList, &fdHandler);
		
		jsiInitNamedHandler(&fdsHandler,
				    JNA_Name, "fds",
				    JNA_Functions, &fdsFunctions,
				    JNA_HandleMessage, handleFDMessage,
				    JNA_TAG_DONE);
		jsiInstallHandler(&teamHandlerList, &fdsHandler);
		
		retval = 1;
		break;
	}
	return( retval );
}

char *configJskedResourceModule(void)
{
	char *retval;

	retval = ""
#if defined(JSKED_STRIDE)
		" (JSKED_STRIDE)"
#elif defined(JSKED_PRIORITY)
		" (JSKED_PRIORITY)"
#else
		" (Unknown jsked scheduler?)"
#endif
		;
	return( retval );
}

resourceSpec *jskedCreateJSISpec(struct _resource *res, jsiMessage *jm)
{
	resourceSpec *retval = 0;
	int count = 0;

	if( res == &metalJskedResource.link )
	{
#if defined(JSKED_STRIDE)
		count = jsiMessageIdentifierCount(jm, "cpu.tickets");
#endif
		if( count && (retval = createResourceSpec(res)) )
		{
			jskedResourceSpec *jrs;
			
			jrs = (jskedResourceSpec *)retval;
#if defined(JSKED_STRIDE)
			if( (jrs->jr =
			     gc_malloc(sizeof(jskedStrideReservation),
						 GC_ALLOC_TEAM_SCHEDULE)) )
			{
				jskedStrideReservation *jsr;
				
				jsr = (jskedStrideReservation *)jrs->jr;
				jsr->jsr_Tickets = 100;
				jsiMessageGet(jm,
					      JMA_Integer, "cpu.tickets", 0,
					      &jsr->jsr_Tickets,
					      JMA_TAG_DONE);
			}
			else
			{
				gc_free(retval);
				retval = 0;
			}
#endif
		}
	}
	else
	{
		count = jsiMessageIdentifierCount(jm, "fd.max");
		if( count && (retval = createResourceSpec(res)) )
		{
			jioResourceSpec *jios;
			
			jios = (jioResourceSpec *)retval;
			if( jsiMessageGet(
				jm,
				JMA_Integer, "fd.max", 0, &jios->fdmax,
				JMA_TAG_DONE) )
			{
				gc_free(retval);
				retval = 0;
			}
		}
	}
	return( retval );
}

void jskedDeleteJSISpec(struct _resource *res, struct _resourceSpec *rs)
{
	if( res == &metalJskedResource.link )
	{
		jskedResourceSpec *jrs;
		
		jrs = (jskedResourceSpec *)rs;
		gc_free(jrs->jr);
	}
	deleteResourceSpec(rs);
}

char *jskedHandleJSIQuery(struct _resourceClient *rc, jsiMessage *jm)
{
	jskedResourceClient *jrc = (jskedResourceClient *)rc;
	char *spec_name, *spec_value, *retval = 0;
	jioAccount *ipc, *disk, *net, *net_packet;
	struct _jskedReservation jr;
	jsked *js = jrc->js;

	if( rc->handler == &metalIOResource.link )
	{
		return( retval );
	}
	switch( jm->what )
	{
	case JSI_REPLY:
		jskedGetReservation(js, &jr);
		ipc = &js->js_IOAccount[JSKED_IOACCOUNT_IPC];
		disk = &js->js_IOAccount[JSKED_IOACCOUNT_DISK];
		net = &js->js_IOAccount[JSKED_IOACCOUNT_NETWORK];
		net_packet =
			&js->js_IOAccount[JSKED_IOACCOUNT_NETWORK_PACKET];
		jsiMessageAdd(jm,
#if defined(JSKED_STRIDE)
			      JMA_Integer, "cpu.tickets", jr.jsr_Tickets,
#endif
			      JMA_LongLongInteger,
			        "cpu.used",
			        js->js_CPUAccount.jca_TotalUsed,
			      JMA_Integer,
			        "fd.open",
			        js->js_IORequestPort.jrp_FDs.Count,
			      JMA_Integer,
			        "fd.max",
			        js->js_IORequestPort.jrp_FDs.Max,
			      JMA_Integer,
			        "fd.peak",
			        js->js_IORequestPort.jrp_FDs.Peak,
			      JMA_LongLongInteger,
			        "ipc.in",
			        ipc->jia_Accumulator[JIO_ACCOUNT_IN],
			      JMA_LongLongInteger,
			        "ipc.out",
			        ipc->jia_Accumulator[JIO_ACCOUNT_OUT],
			      JMA_LongLongInteger,
			        "disk.in",
			        disk->jia_Accumulator[JIO_ACCOUNT_IN],
			      JMA_LongLongInteger,
			        "disk.out",
			        disk->jia_Accumulator[JIO_ACCOUNT_OUT],
			      JMA_LongLongInteger,
			        "net.in",
			        net->jia_Accumulator[JIO_ACCOUNT_IN],
			      JMA_LongLongInteger,
			        "net.out",
			        net->jia_Accumulator[JIO_ACCOUNT_OUT],
			      JMA_LongLongInteger,
			        "net_packet.in",
			        net_packet->jia_Accumulator[JIO_ACCOUNT_IN],
			      JMA_LongLongInteger,
			        "net_packet.out",
			        net_packet->jia_Accumulator[JIO_ACCOUNT_OUT],
			      JMA_TAG_DONE);
		break;
	case JSI_SET_PROPERTY:
		if( jsiPopMessageSpecifier(jm, &spec_name, &spec_value) )
		{
			if( !strcmp(spec_name, "fd.max") )
			{
				int count;
				
				if( !jsiMessageGet(
					jm,
					JMA_Integer, "data", 0, &count,
					JMA_TAG_DONE) )
				{
					jioRequestPort *jrp;
					int rc;

					jrp = &jskedContext()->
						js_IORequestPort;
					jsiClearMessage(jm);
					rc = jioSetFDMax(jrp, count);
					if( rc == 0 )
					{
						jsiMessageAdd(
							jm,
							JMA_What, JSI_REPLY,
							JMA_String, "result", "ok",
							JMA_TAG_DONE);
					}
					else
					{
						retval = strerror(rc);
					}
				}
				else
				{
					retval = "No data to set";
				}
			}
			else
#if defined(JSKED_STRIDE)
			if( !strcmp(spec_name, "tickets") )
			{
				int tickets;
				
				if( !jsiMessageGet(
					jm,
					JMA_Integer, "data", 0, &tickets,
					JMA_TAG_DONE) )
				{
					jr.jsr_Tickets = tickets;
					if( tickets <= 0 )
						return("Invalid ticket value");
					jskedChangeReservation(jskedContext(),
							       &jr);
					jsiClearMessage(jm);
					jsiMessageAdd(jm,
						      JMA_What, JSI_REPLY,
						      JMA_String,
						      "result", "ok",
						      JMA_TAG_DONE);
				}
				else
				{
					retval = "No data to set";
				}
			}
			else
#endif
			{
				retval = "Unknown property specifier";
			}
		}
		else
		{
			retval = "Missing jsked property specifier";
		}
		break;
	default:
		retval = "Unsupported operation";
		break;
	}
	return( retval );
}

int jskedSubscribe(struct _resource *res,
		   struct _resourceClient *rc,
		   struct _resourceSpec *rs)
{
	int retval = 0;
	
	if( res == &metalJskedResource.link )
	{
		jskedResourceClient *jrc = (jskedResourceClient *)rc;
		jskedResourceSpec *jrs = (jskedResourceSpec *)rs;
		jthreadState *jts;
		int pri = 0;
		jsked *js;
		
		jrc->jsc.jsc_Flags = JSCF_NO_EXIT;
		if( (jts = jsthreadSchedulerCreateThread(
			0,
			(char *)&pri,
			sizeof(struct _jthread),
			JTSA_StackSize, JTHREAD_STACK_DEFAULT_SIZE,
			JTSA_Function, jsthreadScheduler,
			JTSA_Argument, &jrc->jsc,
			JTSA_TAG_DONE)) )
		{
			if( (js = jskedCreate(jrs ? jrs->jr : 0,
					      jts,
					      jsthreadSchedulerCreateThread,
					      jsthreadSchedulerDeleteThread)) )
			{
				if( jrs )
				{
					js->js_CPULimit = jrs->limit;
				}
				js->js_UserData = jrc->client.owner;
				jrc->js = js;
				retval = 1;
			}
			else
			{
				jsthreadSchedulerDeleteThread(0, jts);
			}
		}
		else
		{
			teamLogEntry(
				"cpu",
				LEA_String,
				"Error", "Could not create scheduler thread.", -1,
				LEA_TAG_DONE);
		}
	}
	else if( res == &metalIOResource.link )
	{
		jioResourceSpec *jios = (jioResourceSpec *)rs;
		jskedResourceClient *jrc;
		unsigned int fdmax;

		jrc = (jskedResourceClient *)
			getTeamInternalResource(rc->owner,
						&metalJskedResource.link);
		if( jios )
		{
			fdmax = jios->fdmax;
		}
		else
		{
			fdmax = JIO_DEFAULT_FD_MAX;
		}
		if( jioSetFDMax(&jrc->js->js_IORequestPort, fdmax) == 0 )
		{
			retval = 1;
		}
	}
	return( retval );
}

int jskedEnableResource(struct _resource *res, struct _resourceClient *rc)
{
	int retval = 1;
	
	if( res == &metalJskedResource.link )
	{
		jskedResourceClient *jrc = (jskedResourceClient *)rc;
		
		retval = jskedAdd(jrc->js);
	}
	return( retval );
}

void jskedUnsubscribe(struct _resource *res, struct _resourceClient *rc)
{
	if( res == &metalJskedResource.link )
	{
		jskedResourceClient *jrc = (jskedResourceClient *)rc;
		
		if( jrc->js )
		{
			jsked *js = jrc->js;
			
			if( jrc->js->js_State != JSKED_STATE_INACTIVE )
			{
				jskedRem(js);
			}
			jsthreadSchedulerDeleteThread(js,
						      js->js_SchedulerThread);
			jskedDelete(js);
		}
		jrc->js = 0;
	}
}

static
char *fileToMessage(jioSharedFD *jsfd, jsiMessage *jm, int fd)
{
	char *retval = 0;
	struct stat st;
	int rc;
	
	if( (rc = KFSTAT(fd, &st)) == 0 )
	{
		jioAccount *jia;
		
		jia = &jsfd->jsfd_Account;
		jsiMessageAdd(jm,
			      JMA_What, JSI_REPLY,
			      JMA_String, "type", "file",
			      sizeof(st.st_dev) == 4 ?
			      JMA_Integer : JMA_LongLongUnsignedInteger,
					"device", st.st_dev,
			      sizeof(st.st_ino) == 4 ?
			      JMA_Integer : JMA_LongLongUnsignedInteger,
					"inode", st.st_ino,
			      JMA_Integer, "mode", st.st_mode,
			      JMA_Integer, "uid", st.st_uid,
			      JMA_Integer, "gid", st.st_gid,
			      sizeof(st.st_rdev) == 4 ?
			      JMA_Integer : JMA_LongLongUnsignedInteger,
					"rdev", st.st_rdev,
			      sizeof(st.st_size) == 4 ?
			      JMA_Integer : JMA_LongLongUnsignedInteger,
					"size", st.st_size,
			      JMA_LongLongUnsignedInteger, "bytes.in",
					jia->jia_Accumulator[JIO_ACCOUNT_IN],
			      JMA_LongLongUnsignedInteger, "bytes.out",
					jia->jia_Accumulator[JIO_ACCOUNT_OUT],
			      JMA_TAG_DONE);
	}
	else
	{
		retval = strerror(rc);
	}
	return( retval );
}

static
char *sockToMessage(jioSharedFD *jsfd, jsiMessage *jm, int fd)
{
	struct sockaddr local, remote;
	socklen_t lnamelen, rnamelen;
	char *retval = 0;
	int rc;

	lnamelen = sizeof(local);
	rnamelen = sizeof(remote);
	memset(&remote, 0, sizeof(remote));
	if( ((rc = KGETSOCKNAME(fd, &local, &lnamelen)) == 0) )
	{
		struct sockaddr_in *lsin, *rsin;
		jioAccount *jia;
		
		rc = KGETPEERNAME(fd, &remote, &rnamelen);
		jia = &jsfd->jsfd_Account;
		switch( local.sa_family )
		{
		case AF_INET:
			lsin = (struct sockaddr_in *)&local;
			rsin = (struct sockaddr_in *)&remote;
			jsiMessageAdd(
				jm,
				JMA_What, JSI_REPLY,
				JMA_String, "type", "inet",
				JMA_IPv4Address, "local.ip", lsin->sin_addr,
				JMA_Integer, "local.port",
					ntohs(lsin->sin_port),
				JMA_IPv4Address, "remote.ip", rsin->sin_addr,
				JMA_Integer, "remote.port",
					ntohs(rsin->sin_port),
				JMA_LongLongUnsignedInteger, "bytes.in",
					jia->jia_Accumulator[JIO_ACCOUNT_IN],
				JMA_LongLongUnsignedInteger, "bytes.out",
					jia->jia_Accumulator[JIO_ACCOUNT_OUT],
				JMA_TAG_DONE);
			break;
		default:
			jsiMessageAdd(jm,
				      JMA_What, JSI_REPLY,
				      JMA_String, "type", "socket",
				      JMA_Integer, "family", local.sa_family,
				      JMA_TAG_DONE);
			break;
		}
	}
	else
	{
		retval = strerror(rc);
	}
	return( retval );
}

char *handleFDMessage(jsiHandlerContext *hc,
		      jsiHandler *jh,
		      jsiMessage *jm)
{
	char *spec_name, *spec_value, *retval = 0;
	jioRequestPort *jrp;
	jioSharedFD jsfd;
	int lpc, fd;
	jsked *js;

	js = jskedContext();
	jrp = &js->js_IORequestPort;
	switch( jm->what )
	{
	case JSI_LIST_PROPERTIES:
		jsiClearMessage(jm);
		jm->what = JSI_REPLY;
		for( lpc = 0; lpc < FD_SETSIZE; lpc++ )
		{
			int valid;
			
			STOPWORLD();
			valid = (jioFindFD(jrp, lpc) != 0);
			RESUMEWORLD();
			if( valid )
			{
				jsiMessageAdd(jm,
					      JMA_Integer, "result", lpc,
					      JMA_TAG_DONE);
			}
		}
		break;
	case JSI_GET_PROPERTY:
		jsiRestoreMessageSpecifier(jm);
		jsiPopMessageSpecifier(jm, &spec_name, &spec_value);
		if( spec_value &&
		    (sscanf(spec_value, "%d", &fd) == 1) )
		{
			int valid = 0;
			jioFD *jfd;
			
			STOPWORLD();
			if( (jfd = jioFindFD(jrp, fd)) )
			{
				jsfd = *jfd->jfd_FD;
				valid = 1;
			}
			RESUMEWORLD();
			if( valid )
			{
				jsiClearMessage(jm);
				switch( jsfd.jsfd_Type )
				{
				case JIO_FD_TYPE_FILE:
				case JIO_FD_TYPE_STDIO:
					retval = fileToMessage(&jsfd, jm, fd);
					break;
				case JIO_FD_TYPE_SOCKET:
					retval = sockToMessage(&jsfd, jm, fd);
					break;
				default:
					retval = "Unknown file descriptor";
					break;
				}
			}
			else
			{
				retval = "Unknown file descriptor";
			}
		}
		else
		{
			retval = "Unknown file descriptor";
		}
		break;
	default:
		retval = "Unsupported fd operation";
		break;
	}
	return( retval );
}

int getFDSuites(jsiHandler *jh, jsiMessage *jm)
{
	int retval;

	retval = jsiMessageAdd(
		jm,
		JMA_PropertyList, "result", fdRootProperties,
		JMA_TAG_DONE);
	return( retval );
}

void edu_utah_janosvm_resources_CPUSpec_nativeConstructor(
	struct Hedu_utah_janosvm_resources_CPUSpec *jjrs,
	jlong limit)
{
	jskedResourceSpec *jrs = OBJECT2NATIVE(&metalJskedResource.link.
					       specNativeClass,
					       (Hjava_lang_Object *)jjrs);

	require(limit >= 0);

	jrs->spec.handler = &metalJskedResource.link;
	jrs->limit = limit;
}

jlong edu_utah_janosvm_resources_CPUSpec_getLimit(
	struct Hedu_utah_janosvm_resources_CPUSpec *jjrs)
{
	jskedResourceSpec *jrs = OBJECT2NATIVE(&metalJskedResource.link.
					       specNativeClass,
					       (Hjava_lang_Object *)jjrs);

	return( jrs->limit );
}

#if defined(JSKED_STRIDE)
void edu_utah_janosvm_resources_StrideCPUSpec_nativeConstructor(
	struct Hedu_utah_janosvm_resources_StrideCPUSpec *jjrs,
	jint tickets)
{
	jskedResourceSpec *jrs = OBJECT2NATIVE(&metalJskedResource.link.
					       specNativeClass,
					       (Hjava_lang_Object *)jjrs);
	jskedStrideReservation *jsr;

	require(tickets > 0);

	jrs->spec.handler = &metalJskedResource.link;
	jsr = checkPtr(gc_malloc(sizeof(jskedStrideReservation),
				 GC_ALLOC_TEAM_SCHEDULE));
	jsr->jsr_Tickets = tickets;
	jrs->jr = jsr;
}

jint edu_utah_janosvm_resources_StrideCPUSpec_getTickets(
	struct Hedu_utah_janosvm_resources_StrideCPUSpec *jjrs)
{
	jskedResourceSpec *jrs = OBJECT2NATIVE(&metalJskedResource.link.
					       specNativeClass,
					       (Hjava_lang_Object *)jjrs);

	return( ((jskedStrideReservation *)jrs->jr)->jsr_Tickets );
}

void edu_utah_janosvm_resources_StrideCPUSpec_nativeFinalizer(
	struct Hedu_utah_janosvm_resources_StrideCPUSpec *jjrs)
{
	jskedResourceSpec *jrs = OBJECT2NATIVE(&metalJskedResource.link.
					       specNativeClass,
					       (Hjava_lang_Object *)jjrs);

	if( !jjrs->found )
	{
		gc_free(jrs->jr);
		jrs->jr = 0;
	}
}

Hjava_lang_Object *
edu_utah_janosvm_resources_StrideCPUSpec_clone(
	struct Hedu_utah_janosvm_resources_StrideCPUSpec *jjrs)
{
	struct Hedu_utah_janosvm_resources_StrideCPUSpec *retval = 0;
	jskedResourceSpec *jrs, *retval_jrs;
	
	retval = (struct Hedu_utah_janosvm_resources_StrideCPUSpec *)
		newObject(jjrs->base.dtable->class);
	retval_jrs = OBJECT2NATIVE(&metalJskedResource.link.specNativeClass,
				   (Hjava_lang_Object *)retval);
	jrs = OBJECT2NATIVE(&metalJskedResource.link.specNativeClass,
			    (Hjava_lang_Object *)jjrs);
	retval_jrs->spec.handler = &metalJskedResource.link;
	retval_jrs->jr = checkPtr(gc_malloc(sizeof(jskedStrideReservation),
					    GC_ALLOC_TEAM_SCHEDULE));
	*retval_jrs->jr = *jrs->jr;
	return( (Hjava_lang_Object *)retval );
}

struct Hedu_utah_janosvm_resources_CPUSpec *
edu_utah_janosvm_resources_CPUSpec_current(Hjava_lang_Class *cl)
{
	struct Hedu_utah_janosvm_resources_CPUSpec *retval = 0;
	jskedResourceSpec *jrs;
	jsked *js;

	js = jskedContext();
	retval = (struct Hedu_utah_janosvm_resources_CPUSpec *)
		newObject(cl);
	jrs = OBJECT2NATIVE(&metalJskedResource.link.specNativeClass,
			    (Hjava_lang_Object *)retval);
	jrs->spec.handler = &metalJskedResource.link;
	jrs->jr = checkPtr(gc_malloc(sizeof(jskedStrideReservation),
				     GC_ALLOC_TEAM_SCHEDULE));
	jrs->limit = js->js_CPULimit;
	jskedGetReservation(js, jrs->jr);
	return( retval );
}

void edu_utah_janosvm_resources_CPUResource_change(
	struct Hedu_utah_janosvm_resources_CPUSpec *jjrs)
{
	errorInfo einfo;
	int error = 0;
	
	if( jjrs )
	{
		jskedResourceSpec *jrs;

		jrs = OBJECT2NATIVE(&metalJskedResource.link.specNativeClass,
				    (Hjava_lang_Object *)jjrs);
		if( !jskedChangeReservation(jskedContext(), jrs->jr) )
		{
			postException(&einfo, RESOURCE_SPEC_EXCEPTION_S);
		}
	}
	else
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(NullPointerException),
				     "Null spec");
		error = 1;
	}
	if( error )
	{
		throwError(&einfo);
	}
}

#endif

jlong edu_utah_janosvm_resources_CPUResource_getUsage(void)
{
	jlong retval = 0;
	jsked *js;

	js = jskedContext();
	retval = js->js_CPUAccount.jca_TotalUsed;
	return( retval );
}

jlong edu_utah_janosvm_resources_CPUResource_getIdle(void)
{
	return( jsked_state.jss_CPUAccount.jca_TotalUsed );
}

jlong edu_utah_janosvm_resources_IOResource_getIn(int account)
{
	errorInfo einfo;
	jlong retval;
	jsked *js;

	if( (account < 0) || (account >= JSKED_IOACCOUNT_MAX) )
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(IllegalArgumentException),
				     "%d is not a valid account",
				     account);
		throwError(&einfo);
	}

	js = jskedContext();
	retval = js->js_IOAccount[account].jia_Accumulator[JIO_ACCOUNT_IN];
	return( retval );
}

jlong edu_utah_janosvm_resources_IOResource_getOut(int account)
{
	errorInfo einfo;
	jlong retval;
	jsked *js;

	if( (account < 0) || (account >= JSKED_IOACCOUNT_MAX) )
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(IllegalArgumentException),
				     "%d is not a valid account",
				     account);
		throwError(&einfo);
	}

	js = jskedContext();
	retval = js->js_IOAccount[account].jia_Accumulator[JIO_ACCOUNT_OUT];
	return( retval );
}

void edu_utah_janosvm_resources_DescriptorIOSpec_nativeConstructor(
	struct Hedu_utah_janosvm_resources_DescriptorIOSpec *jjios,
	jint count)
{
	jioResourceSpec *jios = OBJECT2NATIVE(&metalIOResource.link.
					      specNativeClass,
					      (Hjava_lang_Object *)jjios);

	require(count >= 0);

	jios->spec.handler = &metalIOResource.link;
	jios->fdmax = count;
}

jint edu_utah_janosvm_resources_DescriptorIOSpec_getCount(
	struct Hedu_utah_janosvm_resources_DescriptorIOSpec *jjrs)
{
	jioResourceSpec *jios = OBJECT2NATIVE(&metalIOResource.link.
					      specNativeClass,
					      (Hjava_lang_Object *)jjrs);

	return( jios->fdmax );
}

struct Hedu_utah_janosvm_resources_DescriptorIOSpec *
edu_utah_janosvm_resources_DescriptorIOSpec_current(void)
{
	struct Hedu_utah_janosvm_resources_DescriptorIOSpec *retval = 0;
	jioResourceSpec *jios;
	
	retval = (struct Hedu_utah_janosvm_resources_DescriptorIOSpec *)
		newObject(metalIOResource.link.specNativeClass.peer);
	jios = OBJECT2NATIVE(&metalIOResource.link.specNativeClass,
			    (Hjava_lang_Object *)retval);
	jios->spec.handler = &metalIOResource.link;
	jios->fdmax = jskedContext()->js_IORequestPort.jrp_FDs.Max;
	return( retval );
}

/** Jsked call backs **/

void jskedCPUOverdraw(jsked *js, jcpu_accum_t amount)
{
	extern struct Hedu_utah_janosvm_sys_EventQueue *KernelEventQueue;
	extern struct Hjava_lang_Thread *KernelEventThread;
	extern Method *AddTailMethod;
	
	struct Hedu_utah_janosvm_sys_TeamBackEnd *be;
	jteam *jt = (jteam *)js->js_UserData;

	require(jcpuDisabled());
	
	be = (struct Hedu_utah_janosvm_sys_TeamBackEnd *)
		NATIVE2OBJECT(&jteamNativeClass, jt);
	if( be->cpuOverdrawEvent )
	{
		struct Hedu_utah_janosvm_sys_TeamEvent *te;

		te = be->cpuOverdrawEvent;
		be->cpuOverdrawEvent = 0;
		do_execute_java_method(KernelEventQueue,
				       0, 0,
				       AddTailMethod,
				       0,
				       te);
		jthread_interrupt((jthread_t)
				  unhand(KernelEventThread)->PrivateInfo);
	}
	else
	{
		/* Its already in progress, don't send another one. */
	}
}

#endif
