/*
 * jskedResource.c
 * JSked resource for teams, wraps the unix-jthreads2 jsked object.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jsked_resource_h
#define _jsked_resource_h

#include "jthread.h"

#if defined(UNIX_JTHREADS2)

#include "resource.h"

#include "jthreadScheduler.h"

/*
 * The jskedResource.
 *
 * link - The standard resource header.
 */
typedef struct _jskedResource {
	resource link;
} jskedResource;

/*
 * A jskedResourceClient holds the jsked specific data for a team.
 *
 * client - The standard client header.
 * js - The jsked object for this team.
 * jsc - The jsthreadConfig scheduler configuration for this team.
 */
typedef struct _jskedResourceClient {
	resourceClient client;
	jsked *js;
	jsthreadConfig jsc;
} jskedResourceClient;

/*
 * A jioResourceClient holds the jsked I/O specific data for a team.
 *
 * client - The standard client header.
 */
typedef struct _jioResourceClient {
	resourceClient client;
} jioResourceClient;

/*
 * A jskedResourceSpec holds the scheduling specification for a jsked.
 *
 * spec - The standard resourceSpec header.
 * limit - The CPU usage limit.
 * jr - The jskedReservation specification.  XXX Should be inlined.
 */
typedef struct _jskedResourceSpec {
	resourceSpec spec;
	jcpu_accum_t limit;
	struct _jskedReservation *jr;
} jskedResourceSpec;

/*
 * A jioResourceSpec holds the I/O specification for a jsked.
 *
 * spec - The standard resourceSpec header.
 * fdmax - The limit on the number of open file descriptors.
 */
typedef struct _jioResourceSpec {
	resourceSpec spec;
	int fdmax;
} jioResourceSpec;

extern jskedResource metalJskedResource;

#if defined(JSKED_STRIDE)
#define JSKED_RESOURCE_TEAM_ARGUMENT \
	"cpu.tickets:int - The number of scheduling tickets for the team.\n" \
	"fd.max:int - The limit on the number of open file descriptors.\n"
#else
#define JSKED_RESOURCE_TEAM_ARGUMENT \
	"fd.max:int - The limit on the number of open file descriptors.\n"
#endif

/*
 * AddModule: jskedResourceModule
 * Conditional: defined(UNIX_JTHREADS2)
 *   ModuleDependency: jteamModule.initMetal
 *   ModuleDependency: memResourceModule.initMetal
 * Transition: initMetal
 *   ModuleDependency: threadModule.jthread
 * Transition: reinitMetal
 *   ModuleDependency: resourceModule.reinstall
 */

#endif

#endif
