/*
 * Copyright (c) 1999, 2000, 2002 The University of Utah.  All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file.
 *
 * Contributed by the Flux Research Group at the University of Utah.
 * Authors: Godmar Back, Leigh Stoller, Patrick Tullmann
 */

/*
 * Inline jthread interruption routines.
 */

#ifndef anjthread_intr_inline_h
#define anjthread_intr_inline_h

#include "anjthread_types_priv.h"
#include "anjthread_current.h"

#if 0

ANJTHREAD_INLINE void
jthread_disable_stop(void)
{
	an_thread_blockintr();
}

ANJTHREAD_INLINE void 
jthread_enable_stop(void)
{
	an_thread_unblockintr();
}

#else

ANJTHREAD_INLINE void
jthread_disable_stop(void)
{
	jthread_t tid = jthread_current();

	require(tid->stopCounter < 10);
	
	tid->flags |= JTHREAD_FLAG_DONT_STOP;
	tid->stopCounter += 1;
}

ANJTHREAD_INLINE void 
jthread_enable_stop(void)
{
	jthread_t tid = jthread_current();

	require(tid->stopCounter >= 1);
	require(tid->stopCounter < 10);

	tid->stopCounter -= 1;
	if( tid->stopCounter == 0 )
	{
		tid->flags &= ~JTHREAD_FLAG_DONT_STOP;
		if( tid->flags & JTHREAD_FLAG_INTR_MASK )
		{
			jthread_slaughter(tid);
		}
	}
}

#endif

#endif /* anjthread_intr_inline_h */
