/*
 * jactivation.c
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "jthread.h"
#include "jactivation.h"

/*
 * Metal jactivation, so it can be dereferenced before the proper
 * initialization sequence has occurred.
 */
jactivation metalJActivation = {
	0,
	JAS_RUNNING,
	&metalJThreadState.jt_State /* XXX */
};

jactivationState jactivation_state = {
	&metalJActivation,
	&metalJThreadState.jt_State /* XXX */
};

/*
 * Regular strings used when printing out the jactivation_state_t values.
 */
static
char *jactivationStateStrings[JAS_MAX] = {
	0,
	"unused",
	"running",
	"stalled",
};

jactivation *jactivationAlloc(jactivationList *jal, jthreadState *js)
{
	jactivation *retval;

	require(jal != 0);
	require(jal->jal_Unused != 0);
	require(js != 0);

	retval = jal->jal_Unused;
	jal->jal_Unused = retval->ja_Next;
	retval->ja_Next = 0;
	retval->ja_State = JAS_STALLED;
	retval->ja_Thread = js;
	return( retval );
}

void jactivationPrint(FILE *file, jactivation *ja)
{
	require((sizeof(jactivationStateStrings) / sizeof(char *)) == JAS_MAX);
	
	require(file != 0);
	require(ja != 0);
	require(ja->ja_State > JAS_MIN);
	require(ja->ja_State < JAS_MAX);

	fprintf(file,
		"jactivation(%p)[ja_Next=%p; ja_State=%s; ja_Thread={ ",
		ja,
		ja->ja_Next,
		jactivationStateStrings[ja->ja_State]);
	if( ja->ja_Thread )
		jthreadStatePrint(file, ja->ja_Thread);
	else
		fprintf(file, "(null)");
	fprintf(file, " }]");
}

void jactivationListPrint(FILE *file, jactivationList *jal)
{
	jactivation *ja;
	
	require(file != 0);
	require(jal != 0);

	fprintf(file, "jactivationList(%p)[jal_Unused={ ", jal);
	ja = jal->jal_Unused;
	while( ja )
	{
		jactivationPrint(file, ja);
		if( ja->ja_Next )
			fprintf(file, ", ");
		ja = ja->ja_Next;
	}
	fprintf(file, " }]");
}

void jactivationStatePrint(FILE *file)
{
	jactivation *ja;
	
	require(file != 0);

	jcpuDisable();
	{
		fprintf(file, "jactivation_state[jas_Current={");
		if( (ja = jactivationCurrent()) )
		{
			jactivationPrint(file, ja);
		}
		fprintf(file, "}]");
	}
	jcpuEnable();
}

/** jcpuContext call backs **/

void jcpuContextAsyncSignals(jsignal_set_t recvmask)
{
	jthreadState *jts;

	require(recvmask != 0);

	jts = jactivationCurrentThread();
	if( jts->jts_AsyncSignalHandler )
	{
		jts->jts_AsyncSignalHandler(jts, recvmask);
	}
}
