/*
 * jactivationEvent.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <assert.h>

#include "jactivationEvent.h"

#include "assert_pp.h"

/*
 * The event type strings.
 */
static
char *jactivationEventTypeStrings[JACTIVATION_EVENT_MAX] = {
	0,
	"unused",
	"spawn",
	"kill",
	"block",
	"resume",
	"preempt",
};

void jactivationEventPrint(FILE *file, jactivationEvent *jae)
{
	require((sizeof(jactivationEventTypeStrings) / sizeof(char *)) ==
		JACTIVATION_EVENT_MAX);
	
	require(file != 0);
	require(jae != 0);
	require(jae->jae_Type > JACTIVATION_EVENT_MIN);
	require(jae->jae_Type < JACTIVATION_EVENT_MAX);

	fprintf(file,
		"jactivationEvent(%p)[jae_Next=%p; "
		"jae_Type=%s(%d); "
		"jae_Subject=%p; "
		"jae_UserData=%p",
		jae,
		jae->jae_Next,
		(jae->jae_Type >= JACTIVATION_EVENT_MAX) ? "trap" :
		jactivationEventTypeStrings[jae->jae_Type],
		jae->jae_Type,
		jae->jae_Subject.activation,
		jae->jae_UserData);
	switch( jae->jae_Type )
	{
	case JACTIVATION_EVENT_UNUSED:
	case JACTIVATION_EVENT_SPAWN:
	case JACTIVATION_EVENT_DEATH:
	case JACTIVATION_EVENT_BLOCK:
	case JACTIVATION_EVENT_RESUME:
	case JACTIVATION_EVENT_PREEMPT:
		break;
	default:
		{
			jactivationTrapEvent *jte =
				(jactivationTrapEvent *)jae;
			fprintf(file,
				"; jte_Function=%p",
				jte->jte_Function);
		}
		break;
	}
	fprintf(file, "]");
}
