/*
 * jactivationEvent.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jactivation_event_h
#define _jactivation_event_h

#include <stdio.h>

#include "listNode.h"

struct _jthread;
struct _jactivation;
struct _jthreadState;

/*
 * Standard event types:
 *   unused - Free for use.
 *   spawn - A new thread for the scheduler.
 *   death - The thread has died.
 *   block - The thread has blocked waiting for some event.
 *   resume - The thread has been resumed.
 *   preempt - The thread has been preempted.
 */
typedef enum {
	JACTIVATION_EVENT_MIN,
	JACTIVATION_EVENT_UNUSED,
	JACTIVATION_EVENT_SPAWN,
	JACTIVATION_EVENT_DEATH,
	JACTIVATION_EVENT_BLOCK,
	JACTIVATION_EVENT_RESUME,
	JACTIVATION_EVENT_PREEMPT,
	JACTIVATION_EVENT_MAX
} jactivation_event_type_t;

/*
 * A jactivationEvent is used to communicate events to the thread scheduler.
 *
 * jae_Next - Link to the next event in the list.
 * jae_Type - The type of event.
 * jae_Subject.activation - The activation that the event concerns.
 * jae_Subject.thread - The thread that the event concerns.
 * jae_UserData - XXX
 */
typedef struct _jactivationEvent {
	struct _jactivationEvent *jae_Next;
	jactivation_event_type_t jae_Type;
	union {
		struct _jactivation *activation;
		struct _jthreadState *thread;
	} jae_Subject;
	void *jae_UserData;
} jactivationEvent;

/*
 * A jactivationTrapEvent is used to execute "user" code in the context of
 * scheduler thread.  For example, walking the list of threads.
 *
 * jte_Event - The standard event header.
 * jte_Function - The function to execute within the context of the scheduler
 *   thread.
 */
typedef struct _jactivationTrapEvent {
	jactivationEvent jte_Event;
	void (*jte_Function)(struct _jactivationTrapEvent *jte, ...);
} jactivationTrapEvent;

/*
 * Print a formatted version of the given jactivationEvent.
 *
 * file - The file to send the output too.
 * jae - The jactivationEvent to print.
 */
void jactivationEventPrint(FILE *file, jactivationEvent *jae);

#endif
