/*
 * jcpuContext.c
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>

#include "assert_pp.h"

#include "jcpu.h"
#include "jcpuContext.h"

jcpuContextState jcpu_context_state;

void jcpuContextInit(jcpuContext *jcc, void *newStack)
{
	require(jcc != 0);
	require(newStack != 0);

	jcc->jcc_Flags = JCF_ROOT_FRAME; /* Mostly for debugging. */
	jcc->jcc_ErrorNumber = 0;
	if( JCPU_SETJMP(jcc->jcc_Env) == JCPU_RESUME_CODE )
	{
		/* This is the new context! */
		jcpuContextRootFrame();
		
		ensure(0);
	}
	else
	{
		void *oldStack, **newBP;

		/* The regular context... */
#if defined(SAVE_FP)
		SAVE_FP(jcc->jcc_FPState);
#endif
		/* Copy part of the current stack into the new stack. */
		oldStack = GET_SP(jcc->jcc_Env);
#if defined(STACK_GROWS_UP)
#error "Implement me"
#else
		newStack -= JTHREAD_STACK_COPY;
		jcc->jcc_StackTop = (void *)newStack;
		memcpy((void *)newStack, oldStack, JTHREAD_STACK_COPY);
		newStack = 0;
#endif
#if defined(NEED_STACK_ALIGN)
		jcc->jcc_StackTop = (void *)STACK_ALIGN(jcc->jcc_StackTop);
#endif
		SET_SP(jcc->jcc_Env, jcc->jcc_StackTop);
#if defined(SET_BP)
		/*
		 * Clear the base pointer in the new thread's stack.
		 * Nice for debugging, but not strictly necessary.
		 */
		newBP = (GET_BP(jcc->jcc_Env) - oldStack) + jcc->jcc_StackTop;
		SET_BP(jcc->jcc_Env, newBP);
		*newBP = 0;
#endif
#if defined(FP_OFFSET)
		/* needed for: IRIX */
		SET_FP(jcc->jcc_Env,
		       jcc->jcc_StackTop +
		       ((void *)GET_FP(jcc->jcc_Env) - oldStack));
#endif
	}
}

void jcpuContextRestore(jcpuContext *jcc)
{
	require(jcpuDisabled());
	
	require(jcc != 0);
	require(!(jcc->jcc_Flags & JCF_ACTIVE));

	jcpu_context_state.jccs_AsyncSignals = &jcc->jcc_AsyncSignals;
	errno = jcc->jcc_ErrorNumber;
	jcc->jcc_Flags |= JCF_ACTIVE;
	JCPU_LONGJMP(jcc->jcc_Env, JCPU_RESUME_CODE);

	ensure(0);
}

void jcpuContextPrint(FILE *file, jcpuContext *jcc)
{
	require(file != 0);
	require(jcc != 0);
	
	fprintf(file,
		"jcpuContext(%p)[jcc_Flags=%lx; "
		"jcc_StackTop=%p; "
		"jcc_ErrorNumber=%d; ",
		jcc,
		jcc->jcc_Flags,
		jcc->jcc_StackTop,
		jcc->jcc_ErrorNumber);
	/* XXX  ^ Use strerror() ^ ? */
	jsignalPrint(file, &jcc->jcc_AsyncSignals);
	fprintf(file, "]");
}
