/*
 * jcpuEvent.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jcpu_event_h
#define _jcpu_event_h

/*
 * Standard event types, a loose mapping to unix signals:
 *   kernel trap - XXX
 *   debug trap - XXX
 *   reset - SIGINT/SIGTERM
 *   null pointer - SIGSEGV/SIGBUS
 *   arithmetic - SIGFP
 *   timer - SIGALRM
 *   preempt - SIGVTALRM
 *   io - SIGIO
 *   child - SIGCHLD
 */
typedef enum {
	JCPU_EVENT_MIN,
	JCPU_EVENT_KERNEL_TRAP,
	JCPU_EVENT_DEBUG_TRAP,
	JCPU_EVENT_RESET,
	JCPU_EVENT_NULL_POINTER,
	JCPU_EVENT_ARITHMETIC,
	JCPU_EVENT_TIMER,
	JCPU_EVENT_PREEMPT,
	JCPU_EVENT_IO,
	JCPU_EVENT_CHILD,
	JCPU_EVENT_MAX
} jcpu_event_type_t;

/*
 * A jcpuEvent is a loose wrapper for unix signals.
 *
 * XXX A bit excessive(?)...
 *
 * jce_Next - The next event in the list.
 * jce_Type - The type of event.
 */
typedef struct _jcpuEvent {
	struct _jcpuEvent *jce_Next;
	jcpu_event_type_t jce_Type;
} jcpuEvent;

/*
 * Print the value of the jcpuEvent.
 *
 * file - The file to print to.
 * jce - The event to print.
 */
void jcpuEventPrint(FILE *file, jcpuEvent *jce);

#endif
