/*
 * jcpuSignal.c
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>

#include "config.h"
#include "config-signal.h"
#include "jthread.h" // XXX remove me
#include "md.h"
#include "assert_pp.h"

#include "jcpuSignal.h"

#include "jcpu.h"

jcpuSignalState jcpu_signal_state;

int jcpuSignalAsync[JCPU_SIGNAL_ASYNC_COUNT] = {
	SIGIO,
	SIGALRM,
	SIGCHLD,
	SIGINT,
	SIGTERM,
#if defined(SIGVTALRM)
	SIGVTALRM,
#endif
};

int jcpuSignalSync[JCPU_SIGNAL_SYNC_COUNT] = {
	SIGFPE,
	SIGSEGV,
	SIGBUS,
	SIGABRT,
};

int jcpuSignalAltStack[JCPU_SIGNAL_ALT_STACK_COUNT] = {
	SIGSEGV,
	SIGBUS,
};

int jcpuSignalIgnored[JCPU_SIGNAL_IGNORED_COUNT] = {
	SIGPIPE,
};

#define ALT_STACK_SIZE MINSIGSTKSZ + 4096

int jcpuSignalInit(void)
{
	struct sigaltstack ss;
	int lpc;

	if( (ss.ss_sp = jcpuMalloc(ALT_STACK_SIZE,
				   ALLOC_TYPE_SIG_STACK)) == 0 )
	{
		return( 0 );
	}
	ss.ss_size = ALT_STACK_SIZE;
	ss.ss_flags = 0;

	if( sigaltstack(&ss, 0) == -1 )
	{
		jcpuFree(ss.ss_sp);
		perror("Cannot setup alternate signal stack");
		return( 0 );
	}
	
	sigemptyset(&jcpu_signal_state.jss_QueuedSignals);
	
	sigemptyset(&jcpu_signal_state.jss_SignalSets[JCPU_SYNC_SIGNALS]);
	for( lpc = 0; lpc < (sizeof(jcpuSignalSync) / sizeof(int)); lpc++ )
	{
		sigaddset(&jcpu_signal_state.jss_SignalSets[JCPU_ALL_SIGNALS],
			  jcpuSignalSync[lpc]);
		sigaddset(&jcpu_signal_state.jss_SignalSets[JCPU_SYNC_SIGNALS],
			  jcpuSignalSync[lpc]);
	}
	
	sigemptyset(&jcpu_signal_state.jss_SignalSets[JCPU_ALT_STACK_SIGNALS]);
	for( lpc = 0; lpc < (sizeof(jcpuSignalAltStack) / sizeof(int)); lpc++ )
	{
		sigaddset(&jcpu_signal_state.
			  jss_SignalSets[JCPU_ALT_STACK_SIGNALS],
			  jcpuSignalAltStack[lpc]);
	}
	
	sigemptyset(&jcpu_signal_state.jss_SignalSets[JCPU_ASYNC_SIGNALS]);
	for( lpc = 0; lpc < (sizeof(jcpuSignalAsync) / sizeof(int)); lpc++ )
	{
		sigaddset(&jcpu_signal_state.jss_SignalSets[JCPU_ALL_SIGNALS],
			  jcpuSignalAsync[lpc]);
		sigaddset(&jcpu_signal_state.
			  jss_SignalSets[JCPU_ASYNC_SIGNALS],
			  jcpuSignalAsync[lpc]);
	}

	for( lpc = 0; lpc < (sizeof(jcpuSignalIgnored) / sizeof(int)); lpc++ )
	{
		if( signal(jcpuSignalIgnored[lpc], SIG_IGN) == SIG_ERR )
		{
			perror("Error setting signal");

			return( 0 );
		}
	}
	return( 1 );
}

void jcpuSignalRegister(int sig, void *handler)
{
	struct sigaction sa;

	require(handler != 0);
	
	sa.sa_handler = handler;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);

	if( sigismember(&jcpu_signal_state.jss_SignalSets[JCPU_ASYNC_SIGNALS],
			sig) )
	{
		/* Need to mask out the asynchronous signals. */
		sa.sa_mask = jcpu_signal_state.
			jss_SignalSets[JCPU_ASYNC_SIGNALS];
	}
	else if( sigismember(&jcpu_signal_state.
			     jss_SignalSets[JCPU_ALT_STACK_SIGNALS],
			     sig) )
	{
		sa.sa_flags |= SA_ONSTACK;
		/*
		 * Need to mask out all the signals, we do not want to handle
		 * recursive signals on this stack.
		 */
		sa.sa_mask = jcpu_signal_state.
			jss_SignalSets[JCPU_ALL_SIGNALS];
	}

#if defined(SA_SIGINFO)
	sa.sa_handler = 0;
	sa.sa_sigaction = handler;
	sa.sa_flags |= SA_SIGINFO;
#endif
#if defined(SA_RESTART)
	sa.sa_flags |= SA_RESTART;
#endif
	if( sigaction(sig, &sa, NULL) == -1 )
	{
		perror("jcpuSignalRegister/sigaction");
		
		ensure(0);
	}
}
