/*
 * jio_file.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jio_file_h
#define _jio_file_h

#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>

/* jthread versions of the standard posix API's */

int jioRead(int fd, void *buf, size_t len, ssize_t *out);
int jioReadAndWait(int fd, void *buf, size_t len, int sample_time,
		   ssize_t *out);
int jioTimedRead(int fd, void *buf, size_t len, int timeout, ssize_t *out);
int jioRecvfrom(int fd, void *buf, size_t len, int flags,
		struct sockaddr *saddr, int *slen, int timeout, ssize_t *out);
int jioWrite(int fd, const void *buf, size_t len, ssize_t *out);
int jioTimedWrite(int fd, const void *buf, size_t len, int timeout,
		  ssize_t *out);
int jioSendto(int fd, const void* buf, size_t len, int flags,
	      const struct sockaddr *addr, int addrlen, ssize_t *out);
int jioTimedConnect(int fd,
		    struct sockaddr *name,
		    int namelen,
		    int timeout);
int jioAccept(int fd,
	      struct sockaddr *name,
	      int *len,
	      int timeout,
	      int *out);
int jioSocket(int af, int type, int proto, int *out);
int jioOpen(const char *path, int flags, int mode, int *out);
int jioPipe(int *files);
int jioDup(int fd, int *out_fd);
int jioClose(int fd);

#endif
