/*
 * jmutex.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _mutex_h
#define _mutex_h

#include <stdio.h>

struct _jthread;

/*
 * A jmutex is used to track the current holder of a lock and the list of
 * threads waiting to take the lock.
 *
 * jm_Holder - NULL or the thread that currently hold's the lock.
 * jm_Waiters - A singly linked list of threads waiting for the lock.
 */
typedef struct _jmutex {
	struct _jthread *jm_Holder;
	struct _jthread *jm_Waiters;
} jmutex;

/*
 * A jcondvar is used to track the list of threads waiting for a signal.
 *
 * jcv_Waiters - A singly linked list of threads waiting for a signal.
 */
typedef struct _jcondvar {
	struct _jthread *jcv_Waiters;
} jcondvar;

/*
 * Macros used when initializing a static jmutex/jcondvar.
 */
#define JMUTEX_STATIC_INITIALIZER { NULL, NULL }
#define JCONDVAR_STATIC_INITIALIZER { NULL }

void jmutex_initialise(jmutex *jm);
void jmutex_destroy(jmutex *jm);
extern void jmutex_lock(jmutex *jm);
extern void jmutex_unlock(jmutex *jm);

void jcondvar_initialise(jcondvar *jcv);
void jcondvar_destroy(jcondvar *jcv);
extern int jcondvar_wait(jcondvar *jcv,
			 jmutex *jm,
			 unsigned long long timeout);
extern void jcondvar_signal(jcondvar *jcv, jmutex *jm);
extern void jcondvar_broadcast(jcondvar *jcv, jmutex *jm);

extern void jmutex_print(FILE *file, jmutex *jm);
extern void jcondvar_print(FILE *file, jcondvar *jcv);

#endif
