/*
 * jsignalSet.c
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>

#include "assert_pp.h"

#include "jsignalSet.h"

int jsignalAlloc(jsignalSet *jss, int signalNumber)
{
	int retval = -1;
	
	require(jss != 0);
	require((signalNumber == -1) ||
		((signalNumber >= MIN_JSIGNAL_NUMBER) &&
		 (signalNumber <= MAX_JSIGNAL_NUMBER)));
	
	if( signalNumber != -1 )
	{
		jsignal_set_t bitmask = (1L << signalNumber);

		/* They want a specific signal number. */
		if( !(jss->jss_AllocMask & bitmask) )
		{
			jss->jss_AllocMask |= bitmask;
			retval = signalNumber;
		}
	}
	else
	{
		int lpc, bitmask;

		/*
		 * Any old signal will do...  Start at the end of the bit mask
		 * so "systemy" stuff can preallocate the least significant
		 * bits.
		 */
		for( lpc = MAX_JSIGNAL_NUMBER;
		     (lpc >= 0) && (retval == -1);
		     lpc-- )
		{
			bitmask = (1L << lpc);
			if( !(jss->jss_AllocMask & bitmask) )
			{
				jss->jss_AllocMask |= bitmask;
				retval = lpc;
			}
		}
	}
	return( retval );
}

void jsignalFree(jsignalSet *jss, int signalNumber)
{
	require(jss != 0);

	if( signalNumber != -1 )
	{
		require(signalNumber >= MIN_JSIGNAL_NUMBER);
		require(signalNumber <= MAX_JSIGNAL_NUMBER);
		require(jss->jss_AllocMask & (1L << signalNumber));
		
		jss->jss_AllocMask &= ~(1L << signalNumber);
		jss->jss_WaitMask &= ~(1L << signalNumber);
		jss->jss_RecvMask &= ~(1L << signalNumber);
	}
}

void jsignalPrint(FILE *file, jsignalSet *jss)
{
	require(file != 0);
	require(jss != 0);

	fprintf(file,
		"jsignalSet(%p)[jss_AllocMask=%08lx; "
		"jss_WaitMask=%08lx; "
		"jss_RecvMask=%08lx]",
		jss,
		jss->jss_AllocMask,
		jss->jss_WaitMask,
		jss->jss_RecvMask);
}
