/*
 * jthreadScheduler.h
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#ifndef _jthread_scheduler_h
#define _jthread_scheduler_h

#include "jsked.h"
#include "jthread.h"

/*
 * jsthreadConfig flags:
 *   JSCF_NO_EXIT - Don't exit if there are only daemon threads left.
 */
enum {
	JSCB_NO_EXIT,
};

enum {
	JSCF_NO_EXIT = (1L << JSCB_NO_EXIT),
};

/*
 * A jsthreadConfig is used to configure a new jsthreadScheduler.  It should
 * be passed as the argument to the scheduler thread.
 *
 * jsc_Flags - Holds the JSCF flags.
 */
typedef struct _jsthreadConfig {
	unsigned long jsc_Flags;
} jsthreadConfig;

/*
 * The main scheduler function.
 *
 * arg - ignored.
 */
void jsthreadScheduler(void *arg);

/*
 * return - An initialized jthreadState that can be used as the root scheduler
 *   thread.
 */
jthreadState *jsthreadSchedulerRoot(void *arg);

/*
 * Create a new thread inside the given jsked.
 *
 * js - The jsked the thread will be managed by.
 * spec - The resource specification.
 * tag - The start of the tag list for initializing the jthreadState.
 * return - A new jthreadState object or NULL if something failed.
 */
jthreadState *jsthreadSchedulerCreateThread(jsked *js,
					    char *spec,
					    size_t size,
					    int tag,
					    ...);

/*
 * Delete a previously allocated jthreadState object.
 *
 * js - The jsked containing the jthread.
 * jts - The jthreadState to delete.
 */
void jsthreadSchedulerDeleteThread(jsked *js, jthreadState *jts);

/*
 * Types of threads that need to be initialized.
 */
typedef enum {
	JTHREAD_TYPE_MIN,
	JTHREAD_TYPE_ROOT,	/* The process thread scheduler. */
	JTHREAD_TYPE_SCHEDULER,	/* A regular thread scheduler. */
	JTHREAD_TYPE_NORMAL,	/* A normal thread. */
	JTHREAD_TYPE_MAX,
} jthread_type_t;

/*
 * Callback used to initialize the outer jthread structure (i.e. whatever is
 * containing the jthreadState).
 *
 * jt - The jthread object to initialize.
 * type - A JTHREAD_TYPE value indicating the type of thread being initialized.
 * return - True if the initialization was successful.
 */
extern int jthreadInit(jthread_t jt, jthread_type_t type);

/*
 * Callback used when the jsked is disabled.
 *
 * jt - The thread to disable.
 */
extern void jthreadDisable(jthread_t jt);

/*
 * Callback used when the jsked is enabled.
 *
 * jt - The thread to enable.
 */
extern void jthreadEnable(jthread_t jt);

#endif
