/*
 * kpath.c
 *
 * Copyright (c) 2003 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#include "config.h"

#include "kpath.h"

#include "gc.h"
#include "system.h"

struct addPath {
	parsedString entry;
	struct lnMinList *list;
};

static
int addParsedPathEntry(struct addPath *ap)
{
	int retval = 1;

	if( ap->entry.len == 0 )
	{
		/* Empty path, ignore... */
	}
	else if( kpFindParsedPath(ap->list, &ap->entry) == 0 )
	{
		pathEntry *pe;

		if( (pe = malloc(sizeof(pathEntry) + ap->entry.len + 1)) )
		{
			memset(pe, 0, sizeof(pathEntry) + ap->entry.len + 1);
			pe->pe_Path = (char *)(pe + 1);
			strncpy(pe->pe_Path, ap->entry.data, ap->entry.len);
			lnAddTail(ap->list, &pe->pe_Link);
		}
		else
		{
			retval = 0;
		}
	}
	else
	{
		/* Duplicate, ignore... */
	}
	return( retval );
}

int kpAddParsedPath(struct lnMinList *list, parsedString *paths)
{
	struct addPath ap;
	int retval = 0;

	ap.list = list;
	if( (list == 0) || (paths->len == 0) )
	{
		retval = 1;
	}
	else if( parseSubString(paths,
				SPO_Do,
				  SPO_String, &ap.entry,
				  SPO_While, path_separator, "",
				  SPO_Handle, addParsedPathEntry, &ap,
				  SPO_End,
				SPO_End) )
	{
		retval = 1;
	}
	return( retval );
}

int kpAddPath(struct lnMinList *list, char *paths)
{
	parsedString ps;

	ps.data = paths;
	ps.len = paths ? strlen(paths) : 0;
	return( kpAddParsedPath(list, &ps) );
}

pathEntry *kpFindParsedPath(struct lnMinList *list, parsedString *path)
{
	pathEntry *curr, *retval = 0;

	if( list )
	{
		curr = (pathEntry *)list->lh_Head;
		while( curr->pe_Link.ln_Succ && !retval )
		{
			if( cmpPStrStr(path, curr->pe_Path) == 0 )
			{
				retval = curr;
			}
			curr = (pathEntry *)curr->pe_Link.ln_Succ;
		}
	}
	return( retval );
}

pathEntry *kpFindPath(struct lnMinList *list, char *path)
{
	parsedString ps;

	ps.data = path;
	ps.len = path ? strlen(path) : 0;
	return( kpFindParsedPath(list, &ps) );
}

const char *file_part(const char *filename)
{
	char *retval = 0;
	parsedString ps;
	
	if( parseString((char *)filename,
			SPO_NotEmpty,
			  SPO_Do,
			    SPO_String, &ps,
			    SPO_While, file_separator, "",
			    SPO_End,
			  SPO_End,
			SPO_End) )
	{
		retval = ps.data;
	}
	return( retval );
}

parsedString dir_part(const char *filename)
{
	parsedString ps, retval = { 0, 0 };

	parseString((char *)filename,
		    SPO_NotEmpty,
		      SPO_String, &retval,
		      SPO_Do,
			SPO_String, &ps,
			SPO_While, file_separator, "",
		        SPO_End,
		      SPO_End,
		    SPO_End);
	return( retval );
}

int absolute_path(const char *filename)
{
	return( !strncmp(filename, file_separator, strlen(file_separator)) );
}
