/*
 * java.io.FileDescriptor.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-io.h"
#include "files.h"
#include "java_io_FileDescriptor.h"
#include "jsyscall.h"
#include "../../../kaffe/kaffevm/support.h"
#include "../../../kaffe/kaffevm/debug.h"
#include <jni.h>
#include <assert.h>

/*
 * Initialise a file descriptor to the given file nr.
 */
struct Hjava_io_FileDescriptor*
java_io_FileDescriptor_initSystemFD(struct Hjava_io_FileDescriptor* this, jint i)
{
	unhand(this)->fd = i;
	return (this);
}

/*
 * Is this file descriptor valid ?
 */
jbool
java_io_FileDescriptor_valid(struct Hjava_io_FileDescriptor* this)
{
#if 0
	if (unhand(this)->fd >= 0) {
		return (1);
	}
	else {
		return (0);
	}
#else
	const int fd = unhand(this)->fd;
	if (fd >= 0)
	{
		const int rc = fcntl(fd, F_GETFL);
		if (rc < 0)
		{
			dprintf("FileDescriptor.valid(%d) -> FAILED\n", fd);
			return 0;
		}
		return 1;
	}
	return 0;
#endif
}

/*
 * Synchronise this file descriptor with the real file system.
 */
void
java_io_FileDescriptor_sync(struct Hjava_io_FileDescriptor* this)
{
#if defined(HAVE_FSYNC)
	if (unhand(this)->fd >= 0) {
		/* XXX make part of jsyscall --- errno is unprotected */
		int r = fsync(unhand(this)->fd);
		if (r < 0) {
			SignalError("java.io.SyncFailedException", SYS_ERROR(errno));
		}
	}
#elif defined(HAVE_SYNC)
	/* Fallback on a full sync */
	/* XXX make part of jsyscall */
	sync();
#else
	/* Well we can't do anything can we? */
	SignalError("java.io.SyncFailedException", "no sync supported");
#endif
}


jint
java_io_FileDescriptor_dup0(jint fd)
{
	int newfd;
	int rc;
	
	if (fd < 0)
		SignalError("java.io.IOException", "invalid file descriptor");
	rc = KDUP(fd, &newfd);
	if (rc != 0)
		SignalError("java.io.IOException", SYS_ERROR(rc));
	return (jint)newfd;
}
