/*
 * StdinInputStream.c
 * Native functions for edu/utah/janosvm/io/StdinInputStream.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * XXX move this code into kaffe/janosvm/<something>
 */

#include "config.h"
#include "config-std.h"
#include "config-io.h"
#include "config-mem.h"
#include "files.h"
#include "defs.h"
#include "jsyscall.h"
#include "itypes.h"
#include "edu_utah_janosvm_kit_comm_jsi_JSIConnectionHandler.h"
#include "edu_utah_janosvm_kit_comm_jsi_NativePlexChannel.h"
#include "edu_utah_janosvm_kit_comm_jsi_NativePlexProvider.h"
#include "../../../kaffe/kaffevm/support.h"
#include "../../../kaffe/kaffevm/stringSupport.h"
#include "soft.h"

#include <janosvm/nativeClass-inline.h>

typedef struct _jsiNativePlexChannel {
	jsiChannel *jc;
} jsiNativePlexChannel;

static nativeClass npcNativeClass;

typedef struct _jsiNativePlexProvider {
	jsiConnectionHandler jch;
	jsiProvider *jp;
	struct Hedu_utah_janosvm_kit_comm_jsi_JSIConnectionHandler *jJch;
} jsiNativePlexProvider;

static nativeClass nppNativeClass;

void
edu_utah_janosvm_kit_comm_jsi_NativePlexProvider_bindNativeClass(void)
{
	static int ok = 0;
	errorInfo einfo;

	if( ok )
		return;
	if( reserveTeam(kernelTeam) == TEAM_ERROR_NONE )
	{
		struct Hjava_lang_Class *cl;

		cl = lookupClass("edu/utah/janosvm/kit/comm/jsi/"
				 "NativePlexProvider",
				 NULL, &einfo);
		assert(cl != 0);
		nppNativeClass.size = sizeof(jsiNativePlexProvider);
		if( bindNativeClass(&nppNativeClass, cl, &einfo) )
		{
			ok = 1;
		}
		unreserveTeam(kernelTeam);
	}
	else
	{
		postException(&einfo, DEAD_TEAM_EXCEPTION_S);
	}
	if( !ok )
		throwError(&einfo);
}

static
int jsiNativePlexProviderHandler(jsiProvider *jp, jsiChannel *jc)
{
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexProvider *jNpp;
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc;
	jsiNativePlexProvider *npp;
	errorInfo einfo;
	int retval = 0;

	npp = (jsiNativePlexProvider *)jp->handleConnection;
	jNpp = (struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexProvider *)
		NATIVE2OBJECT(&nppNativeClass, npp);
	if( (jNpc = (struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *)
	     newObjectChecked(npcNativeClass.peer, &einfo)) )
	{
		jsiNativePlexChannel *npc;
		
		npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
		npc->jc = jc;
		do_execute_java_method(
			npp->jJch,
			"handle",
			"(Ledu/utah/janosvm/kit/comm/jsi/JSIProvider;"
			"Ledu/utah/janosvm/kit/comm/jsi/JSIChannel;)V",
			0,
			0,
			jNpp,
			jNpc);
	}
	else
	{
		retval = 1;
	}
	return( retval );
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexProvider_init(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexProvider *jNpp,
	struct Hjava_lang_String *jProtocol)
{
	jsiNativePlexProvider *npp;
	errorInfo einfo;
	char *protocol;
	int ok = 0;

	if( gc_add_ref(jNpp) )
	{
		if( (protocol = stringJava2C(jProtocol)) )
		{
			npp = OBJECT2NATIVE(&nppNativeClass, &jNpp->base);
			npp->jch.func = jsiNativePlexProviderHandler;
			if( (npp->jp = jsiAttachProviderTo(protocol,
							   0,
							   &npp->jch)) )
			{
				ok = 1;
			}
			else
			{
				postException(&einfo, JAVA_IO(IOException));
			}
			gc_free(protocol);
		}
		else
		{
			postOutOfMemory(&einfo);
		}
	}
	else
	{
		postOutOfMemory(&einfo);
	}
	
	if( !ok )
		throwError(&einfo);
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexProvider_handle(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexProvider *jNpp,
	struct Hedu_utah_janosvm_kit_comm_jsi_JSIConnectionHandler *jJch)
{
	jsiNativePlexProvider *npp;
	
	npp = OBJECT2NATIVE(&nppNativeClass, &jNpp->base);
	if( npp->jp )
	{
		errorInfo einfo;
		
		if( gc_add_ref(jJch) )
		{
			npp->jJch = jJch;
			jsiHandleProvider(npp->jp);
		}
		else
		{
			postOutOfMemory(&einfo);
			throwError(&einfo);
		}
	}
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexProvider_detach(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexProvider *jNpp)
{
	jsiNativePlexProvider *npp;

	gc_rm_ref(jNpp);
	
	npp = OBJECT2NATIVE(&nppNativeClass, &jNpp->base);
	if( npp->jp )
	{
		jsiDeleteProvider(npp->jp);
		npp->jp = 0;
		gc_rm_ref(npp->jJch);
	}
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_bindNativeClass(void)
{
	static int ok = 0;
	errorInfo einfo;
	
	if( ok )
		return;
	if( reserveTeam(kernelTeam) == TEAM_ERROR_NONE )
	{
		struct Hjava_lang_Class *cl;

		cl = lookupClass("edu/utah/janosvm/kit/comm/jsi/"
				 "NativePlexChannel",
				 NULL, &einfo);
		assert(cl != 0);
		npcNativeClass.size = sizeof(jsiNativePlexChannel);
		if( bindNativeClass(&npcNativeClass, cl, &einfo) )
		{
			ok = 1;
		}
		unreserveTeam(kernelTeam);
	}
	else
	{
		postException(&einfo, DEAD_TEAM_EXCEPTION_S);
	}
	if( !ok )
		throwError(&einfo);
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_init(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc,
	struct Hjava_lang_String *jProtocol)
{
	jsiNativePlexChannel *npc;
	errorInfo einfo;
	char *protocol;
	int ok = 0;
	
	protocol = checkPtr(stringJava2C(jProtocol));
	npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
	if( (npc->jc = jsiConnectChannelTo(protocol, 0)) )
	{
		ok = 1;
	}
	else
	{
		postException(&einfo, JAVA_IO(IOException));
	}
	gc_free(protocol);
	if( !ok )
		throwError(&einfo);
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_sendMessage(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc,
	HArrayOfByte *msg)
{
	jsiNativePlexChannel *npc;
	
	npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
	if( npc->jc )
	{
		jsiSendMessage(npc->jc,
			       unhand_array(msg)->body,
			       obj_length(msg));
	}
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_replyMessage(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc,
	HArrayOfByte *msg)
{
	jsiNativePlexChannel *npc;
	
	npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
	if( npc->jc )
	{
		jsiReplyMessage(npc->jc,
				unhand_array(msg)->body,
				obj_length(msg));
	}
}

HArrayOfByte *
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_recvMessage(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc)
{
	jsiNativePlexChannel *npc;
	HArrayOfByte *retval = 0;
	
	npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
	if( npc->jc )
	{
		char *buf;
		int size;
		
		if( (buf = jsiRecvMessage(npc->jc, &size)) )
		{
			retval = (HArrayOfByte *)
				newArray(TYPE_CLASS(TYPE_Byte), size);
			memcpy(unhand_array(retval)->body, buf, size);
		}
	}
	return( retval );
}

void
edu_utah_janosvm_kit_comm_jsi_NativePlexChannel_disconnect(
	struct Hedu_utah_janosvm_kit_comm_jsi_NativePlexChannel *jNpc)
{
	jsiNativePlexChannel *npc;
	
	npc = OBJECT2NATIVE(&npcNativeClass, &jNpc->base);
	if( npc->jc )
	{
		jsiDeleteChannel(npc->jc);
		npc->jc = 0;
	}
}
