/*
 * java.lang.ClassLoader.c
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "config-std.h"
#include "config-mem.h"
#include "jmalloc.h"
#include "../../../kaffe/kaffevm/gtypes.h"
#include "../../../kaffe/kaffevm/file.h"
#include "../../../kaffe/kaffevm/readClass.h"
#include "../../../kaffe/kaffevm/constants.h"
#include "../../../kaffe/kaffevm/access.h"
#include "../../../kaffe/kaffevm/classMethod.h"
#include "../../../kaffe/kaffevm/object.h"
#include "../../../kaffe/kaffevm/locks.h"
#include "../../../kaffe/kaffevm/itypes.h"
#include "../../../kaffe/kaffevm/support.h"
#include "../../../kaffe/kaffevm/stringSupport.h"
#include "../../../kaffe/kaffevm/baseClasses.h"
#include "../../../kaffe/kaffevm/exception.h"
#include "../../../kaffe/kaffevm/thread.h"
#include "../../../kaffe/kaffevm/gc.h"
#include "../../../kaffe/janosvm/classPath.h"
#include "../../../kaffe/janosvm/nameSpace.h"
#include <native.h>
#include "java_lang_ClassLoader.h"
#include "defs.h"

void
java_lang_ClassLoader_nativeInit(struct Hjava_lang_ClassLoader* this)
{
	nameSpaceClient *nsc;
	
	nsc = (nameSpaceClient *)
		getTeamInternalResource(0, &metalNameSpaceResource.link);
	this->classGroup = (struct Hkaffe_util_Ptr *)
		createClassGroup(0, nsc->cGroup);
}

/*
 * Translate an array of bytes into a class.
 */
struct Hjava_lang_Class*
java_lang_ClassLoader_defineClass0(struct Hjava_lang_ClassLoader* this, struct Hjava_lang_String* name, HArrayOfByte* data, jint offset, jint length)
{
	Hjava_lang_Class *clazz, *dup_clazz;
	nameMapping *nm;
	classFile hand;
	errorInfo info;

	classFileInit(&hand, &unhand_array(data)->body[offset], length, CP_BYTEARRAY);

	clazz = newClass();
	if (clazz == 0) {
		postOutOfMemory(&info);
		throwError(&info);
	}

	/* 
	 * If a name was given, but the name we found in the class file
	 * was a different one, complain.
	 */
	if (name != NULL) {
		Utf8Const *utf;

		utf = checkPtr(stringJava2Utf8ConstReplace(name, '.', '/'));
		utf8ConstAssign(clazz->primaryView.name, utf);
	}
	clazz = readClass(clazz, &hand, this, &info);
	if (clazz == 0) {
		throwError(&info);
	}

	/*
	 * See if an entry for that name and class loader already exists
	 * create one if not.
	 */
	nm = getClassMapping(clazz->primaryView.name, this, &info);
	if (nm == 0) {
		throwError(&info);
	}

	/*
	 * see if somebody loaded that class already
	 */
	if( classMappingLoad(nm, &dup_clazz, &info) )
	{
		if( dup_clazz )
		{
			postExceptionMessage(&info,
					     JAVA_LANG(ClassFormatError),
					     "Duplicate name: %s",
					     nm->name->data);
			throwError(&info);
		}
		/*
		 * While it is not necessary that one be able to actually *use*
		 * the returned class object at this point, it is mandatory
		 * that the returned clazz object is a functional Class object.
		 *
		 * The following call will make sure that the returned class
		 * object has its dispatch table set.  The transition
		 * PRELOADED->PREPARED in processClass sets class->head.dtable.
		 *
		 * Presumably, it shouldn't be necessary here, but is at the
		 * moment - XXX
		 */
		else if( processClass(clazz,
				      CSTATE_PREPARED,
				      &info) == false )
		{
			throwError(&info);
		}
	}
	else
	{
		throwError(&info);
	}
	return( clazz );
}

/*
 * Resolve classes reference by this class.
 */
void
java_lang_ClassLoader_resolveClass0(struct Hjava_lang_ClassLoader* this,
				    struct Hjava_lang_Class* class)
{
	errorInfo info;
	if (processClass(class, CSTATE_COMPLETE, &info) == false) {
		throwError(&info);
	}
}

/*
 * Find a loaded class.
 *
 * Note that this means to find a class for which we were the defining
 * classloader.  We must not report classes for which we were merely the
 * initiating loader.
 */
struct Hjava_lang_Class*
java_lang_ClassLoader_findLoadedClass0(Hjava_lang_ClassLoader* this, Hjava_lang_String* str)
{
	Hjava_lang_Class *retval;
        int len = STRING_SIZE(str);
        Utf8Const* c;
        char* name;
        char buffer[100];
	errorInfo info;

        if (len <= sizeof(buffer) - 1) {
                name = buffer;
        }
        else {
                name = gc_malloc (len, GC_ALLOC_TEMP_STRING);
        }
	if (!name) {
		postOutOfMemory(&info);
		throwError(&info);
	}
        stringJava2CBuf(str, name, len+1);
        classname2pathname(name, name);

        c = utf8ConstNew(name, len);
        if (name != buffer) {
                gc_free (name);
        }
	if (!c) {
		postOutOfMemory(&info);
		throwError(&info);
	}

	retval = findValidClassMapping(c, this);
        utf8ConstRelease(c);

	return( retval );
}
