/*
 * moabthread.c
 *
 * Copyright (c) 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "config.h"

#include <nodeos/an_domain.h>
#include <nodeos/an_pbuf.h>

#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/thread.h"
#include "../../../../kaffe/kaffevm/systems/nodeos-threads/anjthread_create.h"
#include "../../../../kaffe/kaffevm/systems/nodeos-threads/anjthread_create-inline.h"
#include "../../../../kaffe/kaffevm/systems/nodeos-threads/anjthread_globals.h"

#include "moabglue.h"
#include "moabthread.h"


static void
moabglue_explicitThreadEnterKaffe(void* runnable)
{
	extern JNIEnv Kaffe_JNIEnv;
	JNIEnv *env = &Kaffe_JNIEnv;
	jmethodID runmethod;
	
	/* XXX runnable is a java.lang.Runnable, see moabglue_startExplicitThread */
	runmethod = (*env)->GetMethodID(env,
					(*env)->GetObjectClass(env, runnable),
					"run",
					"()V");
	anjthread_enterkaffe(runnable, runmethod);
}


void
moabglue_startExplicitThread(an_domain_t* domain,
			     struct Hjava_lang_Runnable* runnable)
{
	// XXX Increment talive to convince Kaffe that a thread exists.  
	jthread_talive++;

	assert(domain);
	assert(runnable);

	an_domain_startthread(domain,
			      moabglue_explicitThreadEnterKaffe,
			      runnable,
			      NULL);
}

void
moabglue_inChanCallback(an_pbuf_t* buffer, void* arg, int payloadStart)
{
	moabInChannelData *ic = (moabInChannelData *)arg;
	jthread_t tid = jthread_current();
	
	assert(buffer);
	assert(arg);

	/* XXX Increment talive to convince Kaffe that a thread exists. */
        // jthread_talive++;       

	/*
	 * The Java code keeps track of the "owner" of a buffer (system or user).
	 * We have to update the owner field...
	 *
	 * XXX THIS IS GROSS MAGICAL CODE.  THIS IS BAD.
	 */
	*(int*)((void*)buffer + sizeof(an_pbuf_t)) = 2;

	anjthread_reenter_kaffe(tid);

	ic->jreceive(ic->javaCallBack,
		     KAFFE_PBUF(buffer),
		     payloadStart,
		     jthread_stacklimit());
	
	anjthread_leave_kaffe(tid);
}

