/*
 * SensorFeature.c
 * Native implementation of kaffe.sensors.SensorFeature functions.
 *
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#include "config.h"
#include "object.h"
#include <native.h>
#include "lm_error.h"
#include "lm_sensors.h"

#include "kaffe_sensors_Sensor.h"
#include "kaffe_sensors_SensorFeature.h"

jdouble
kaffe_sensors_SensorFeature_read(struct Hkaffe_sensors_SensorFeature *feature)
{
	sensors_feature_data *sfd;
	int success = 0;
	errorInfo einfo;
	jdouble retval;

	sfd = (sensors_feature_data *)feature->internal;
	if( feature->mode & kaffe_sensors_SensorFeature_MODE_R )
	{
		sensors_chip_name *scn;

		scn = (sensors_chip_name *)feature->sensor->internal;
		switch( sensors_get_feature(*scn, sfd->number, &retval) )
		{
		case 0:
			success = 1;
			break;
		case -SENSORS_ERR_NO_ENTRY:
			postExceptionMessage(
				&einfo,
				"java.util.NoSuchElementException",
				"%s",
				sfd->name);
			break;
		case -SENSORS_ERR_WILDCARDS:
		case -SENSORS_ERR_ACCESS:
		case -SENSORS_ERR_DIV_ZERO:
		case -SENSORS_ERR_CHIP_NAME:
		case -SENSORS_ERR_BUS_NAME:
		case -SENSORS_ERR_PARSE:
			assert(0);
			break;
		case -SENSORS_ERR_PROC:
			postExceptionMessage(&einfo,
					     JAVA_IO(IOException),
					     "Couldn't read feature from "
					     "/proc: %s",
					     sfd->name);
			break;
		default:
			postExceptionMessage(&einfo,
					     JAVA_IO(IOException),
					     "Couldn't read feature: %s",
					     sfd->name);
			break;
		}
	}
	else
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(SecurityException),
				     "No read access for feature: %s",
				     sfd->name);
	}
	if( !success )
	{
		throwError(&einfo);
	}
	return( retval );
}

void
kaffe_sensors_SensorFeature_write(struct Hkaffe_sensors_SensorFeature *feature,
				  jdouble v)
{
	sensors_feature_data *sfd;
	int success = 0;
	errorInfo einfo;
	
	sfd = (sensors_feature_data *)feature->internal;
	if( feature->mode & kaffe_sensors_SensorFeature_MODE_W )
	{
		sensors_chip_name *scn;

		scn = (sensors_chip_name *)feature->sensor->internal;
		if( sensors_set_feature(*scn, sfd->number, v) == 0 )
		{
			success = 1;
		}
		else
		{
			postExceptionMessage(&einfo,
					     JAVA_IO(IOException),
					     "Couldn't write feature: %s",
					     sfd->name);
		}
	}
	else
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(SecurityException),
				     "No write access for feature: %s",
				     sfd->name);
	}
	if( !success )
	{
		throwError(&einfo);
	}
}
