/*
 * Sensor.c
 * Native implementation of kaffe.sensors.Sensor functions.
 *
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#include "config.h"
#include "object.h"
#include <native.h>
#include "files.h"
#include "support.h"

#include "mbmon.h"
#include "xmb_sensors.h"

#include "kaffe_sensors_Sensor.h"
#include "kaffe_sensors_SensorFeature.h"

void
kaffe_sensors_Sensor_nativeInit(void)
{
	static char *defaultProbeMethod = " ";
	
	char *tyan, *probeMethod;
	int error = 0, rc;
	errorInfo einfo;

	if( (tyan = getProperty(getSystemProperties(),
				"kaffe.sensors.xmb.tyan-tiger",
				0)) )
	{
		if( strcasecmp(tyan, "yes") )
		{
			TyanTigerMP_flag = 1;
		}
		gc_free(tyan);
	}
	
	probeMethod = getProperty(getSystemProperties(),
				  "kaffe.sensors.xmb.method",
				  defaultProbeMethod);
	rc = InitMBInfo(probeMethod[0]);
	switch( rc )
	{
	case 0:
		break;
	case -1:
		switch( errno )
		{
		case ENOMEM:
			postOutOfMemory(&einfo);
			error = 1;
			break;
		case EACCES:
			postExceptionMessage(&einfo,
					     JAVA_LANG(SecurityException),
					     "Cannot access sensors. "
					     "(Use setuid root?)");
			error = 1;
			break;
		default:
			postExceptionMessage(&einfo,
					     JAVA_IO(IOException),
					     "%s",
					     SYS_ERROR(errno));
			error = 1;
			break;
		}
		break;
	default:
		postExceptionMessage(&einfo,
				     JAVA_LANG(InternalError),
				     "Cannot find sensors (use setuid root?)");
		error = 1;
		break;
	}
	if( probeMethod != defaultProbeMethod )
	{
		gc_free(probeMethod);
	}
	if( error )
	{
		throwError(&einfo);
	}
}


struct Hkaffe_sensors_Sensor *
kaffe_sensors_Sensor_nativeDetect(struct Hjava_lang_Class *cls)
{
	struct Hkaffe_sensors_Sensor *retval;

	require(cls != 0);

	retval = (struct Hkaffe_sensors_Sensor *)
		execute_java_constructor(0, 0, cls, "()V");
	retval->next = 0;
	retval->internal = 0;
	retval->name = stringC2Java(getSensorName());
	return( retval );
}

struct Hkaffe_sensors_SensorFeature *
kaffe_sensors_Sensor_detectFeatures(struct Hkaffe_sensors_Sensor *sensor,
				    struct Hjava_lang_Class *cls)
{
	struct Hkaffe_sensors_SensorFeature *feature, *retval = 0;
	int lpc;

	require(sensor != 0);
	require(cls != 0);

	for( lpc = 0; xmb_sensor_features[lpc].name; lpc++ )
	{
		xmb_sensor_feature *xsf;
		
		xsf = &xmb_sensor_features[lpc];
		
		feature = (struct Hkaffe_sensors_SensorFeature *)
		    execute_java_constructor(0, 0, cls, "()V");
		feature->next = retval;
		retval = feature;
		feature->sensor = sensor;
		feature->name = stringC2Java(xsf->name);
		feature->mode = kaffe_sensors_SensorFeature_MODE_R;
		feature->internal = (struct Hkaffe_util_Ptr *)xsf;
	}
	return( retval );
}
