/*
 * SensorFeature.c
 * Native implementation of kaffe.sensors.SensorFeature functions.
 *
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#include "config.h"
#include "object.h"
#include <native.h>

#include "xmb_sensors.h"

#include "kaffe_sensors_Sensor.h"
#include "kaffe_sensors_SensorFeature.h"

/*
 * XXX These are copied over since xmbmon defines some globals in mbmon.h.
 *
 * Sigh.
 */
int getTemp(float *, float *, float *);
int getVolt(float *, float *, float *, float *, float *, float *, float *);
int getFanSp(int *, int *, int *);

jdouble
kaffe_sensors_SensorFeature_read(struct Hkaffe_sensors_SensorFeature *feature)
{
	xmb_sensor_feature *xsf;
	jdouble retval = 0.0;
	errorInfo einfo;
	int success = 0;

	require(feature != 0);

	xsf = (xmb_sensor_feature *)feature->internal;
	if( feature->mode & kaffe_sensors_SensorFeature_MODE_R )
	{
		float fvalues[7];
		int ivalues[3];
		
		switch( xsf->type )
		{
		case SENSOR_TYPE_TEMP:
			if( getTemp(&fvalues[0],
				    &fvalues[1],
				    &fvalues[2]) == 0 )
			{
				retval = (jdouble)fvalues[xsf->index];
				success = true;
			}
			else
			{
				postExceptionMessage(
					&einfo,
					JAVA_IO(IOException),
					"Could not read feature: %s",
					xsf->name);
			}
			break;
		case SENSOR_TYPE_FAN:
			if( getFanSp(&ivalues[0],
				     &ivalues[1],
				     &ivalues[2]) == 0 )
			{
				retval = (jdouble)ivalues[xsf->index];
				success = true;
			}
			else
			{
				postExceptionMessage(
					&einfo,
					JAVA_IO(IOException),
					"Could not read feature: %s",
					xsf->name);
			}
			break;
		case SENSOR_TYPE_VOLT:
			if( getVolt(&fvalues[0],
				    &fvalues[1],
				    &fvalues[2],
				    &fvalues[3],
				    &fvalues[4],
				    &fvalues[5],
				    &fvalues[6]) == 0 )
			{
				retval = (jdouble)fvalues[xsf->index];
				success = true;
			}
			else
			{
				postExceptionMessage(
					&einfo,
					JAVA_IO(IOException),
					"Could not read feature: %s",
					xsf->name);
			}
			break;
		default:
			postExceptionMessage(&einfo,
					     JAVA_LANG(InternalError),
					     "Unknown sensor type: %d for %s",
					     xsf->type,
					     xsf->name);
			break;
		}
	}
	else
	{
		postExceptionMessage(&einfo,
				     JAVA_LANG(SecurityException),
				     "No read access for feature: %s",
				     xsf->name);
	}
	if( !success )
	{
		throwError(&einfo);
	}
	return( retval );
}

void
kaffe_sensors_SensorFeature_write(struct Hkaffe_sensors_SensorFeature *feature,
				  jdouble v)
{
	errorInfo einfo;

	require(feature != 0);
	
	postExceptionMessage(&einfo,
			     JAVA_LANG(UnsupportedOperationException),
			     "Writing to sensors is not supported.");
	throwError(&einfo);
}
