/*
 * xmb_sensors.h
 *
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Contributed by the Flux Research Group, Department of Computer Science,
 * University of Utah, http://www.cs.utah.edu/flux/
 */

#ifndef _xmb_sensors_h
#define _xmb_sensors_h

/*
 * This file essentially emulates the lmsensors "sensor feature" framework.
 */

/*
 * Sensor types:
 *   SENSOR_TYPE_TEMP - Temperature sensor.
 *   SENSOR_TYPE_FAN - Fan speed sensor.
 *   SENSOR_TYPE_VOLT - Voltage sensor.
 */
typedef enum {
	SENSOR_TYPE_INVALID,
	SENSOR_TYPE_TEMP,
	SENSOR_TYPE_FAN,
	SENSOR_TYPE_VOLT,
	SENSOR_TYPE_MAX,
} xmb_sensor_type_t;

/*
 * Encodes a "sensor feature" as provided by xmbmon.
 *
 * name - The name of the sensor.
 * type - The sensor type.
 * index - Indicates which value returned by getTemp/getVolt/getFanSp
 *   corresponds to this feature.
 */
typedef struct xmb_sensor_feature {
	char *name;
	xmb_sensor_type_t type;
	int index;
} xmb_sensor_feature;

/*
 * The list of sensor features for the system's main chip.
 */
extern xmb_sensor_feature xmb_sensor_features[];

#endif
