package java.util.prefs;

import java.lang.SecurityException;
import java.util.HashMap;
import java.util.Set;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import gnu.java.util.prefs.TransientFactory;
import gnu.java.util.prefs.NodeReader;

/**
 * Not yet a standard class, should be part of JDK1.5
 *
 * @author Pat Tullmann <tullmann@cs.utah.edu>
 */
public class TransientPreferences
	extends AbstractPreferences
{
	private boolean isUser = true;
	private HashMap keyValueMap = new HashMap();
	private HashMap childMap = new HashMap();

	private TransientPreferences(TransientPreferences parent, String name)
	{
		super(parent, name);
	}

	public TransientPreferences() {
		super(null, "");
	}

	public TransientPreferences(TransientPreferences base) {
		super(null, base.name() + "(clone)");
		this.merge(base);
	}

	public TransientPreferences(InputStream is) 
		throws IOException, InvalidPreferencesFormatException 
	{
		super(null, "From " +is.toString());

		PreferencesFactory factory = new TransientFactory(this);
		NodeReader reader = new NodeReader(is, factory);
	}

	public void merge(TransientPreferences source) {
		// Copy all the keys from the source
		String[] skeys = source.keys();
		for (int i = 0; i < skeys.length; i++)
		{
			this.putSpi(skeys[i], source.getSpi(skeys[i]));
		}
		
		// Merge all the children
		String[] schildren = source.childrenNames();
		for (int i = 0; i < schildren.length; i++)
		{
			TransientPreferences schild = (TransientPreferences)source.node(schildren[i]);
			TransientPreferences dchild = (TransientPreferences)this.node(schildren[i]);
			dchild.merge(schild);
		}
	}

	public String[] keys()
	{
		try
		{
			return super.keys();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public String[] childrenNames()
	{
		try
		{
			return super.childrenNames();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public void clear()
	{
		try
		{
			super.clear();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public boolean nodeExists(String n)
	{
		try
		{
			return super.nodeExists(n);
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public void removeNode()
	{
		try
		{
			super.removeNode();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public void flush()
	{
		try
		{
			super.flush();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	public void sync()
	{
		try
		{
			super.sync();
		}
		catch (BackingStoreException bse)
		{
			throw new InternalError(bse);
		}
	}

	protected  void putSpi(String key, String value) {
		this.keyValueMap.put(key, value);
	}

	protected String getSpi(String key) {
		return (String)this.keyValueMap.get(key);
	}

	protected void removeSpi(String key) {
		this.keyValueMap.remove(key);
	}

	protected void removeNodeSpi() {
		this.keyValueMap.clear();
	}

	protected String[] keysSpi() {
		Set k = this.keyValueMap.keySet();
		String[] s = new String[0];
		return (String[])k.toArray(s);
	}

	protected String[] childrenNamesSpi() {
		Set k = this.childMap.keySet();
		String[] s = new String[0];
		return (String[])k.toArray(s);
	}

	protected AbstractPreferences childSpi(String name) {
		if (this.childMap.containsKey(name))
			return (TransientPreferences)(this.childMap.get(name));
		
		// Otherwise, create it
		TransientPreferences child = new TransientPreferences(this, name);
		this.childMap.put(name, child);
		return child;
	}

	protected void flushSpi() {
	}

	protected void syncSpi() {
	}

	public boolean isUserNode()
	{
		return isUser;
	}
}
