/*
 * CPUSpec.java
 * Base resource specification for CPU resources.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Base resource specification for CPU resources.
 */
public class CPUSpec
    extends ResourceSpec
    implements Cloneable
{
    /**
     * Value used to indicate an infinite CPU usage limit.
     */
    public static final long INFINITE_LIMIT = 0;

    /**
     * Constructs the native part of the class.
     */
    private native void nativeConstructor(long limit);
    
    /**
     * Construct a CPU resource specification with the given values.
     *
     * @param limit The CPU usage limit.
     */
    public CPUSpec(long limit)
    {
	if( limit < 0 )
	{
	    throw new IllegalArgumentException("Negative limit value: "
					       + limit);
	}
	this.nativeConstructor(limit);
    }

    /**
     * Construct a CPU resource specification with the default values.
     */
    public CPUSpec()
    {
	this(INFINITE_LIMIT);
    }

    /**
     * @return The CPU usage limit.
     */
    public native long getLimit();

    /**
     * @param cl The subclass of CPUSpec to create.
     * @return A CPUSpec containing the current team's specification.
     */
    static native CPUSpec current(Class cl);

    public String toString()
    {
	return "CPUSpec[limit="
	    + this.getLimit()
	    + "]";
    }
}
