/*
 * DescriptorIOSpec.java
 * Resource specification for file descriptors.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Resource specification for file descriptors.
 */
public final class DescriptorIOSpec
    extends IOSpec
{
    /**
     * The default descriptor count.
     */
    public static final int DEFAULT_COUNT = 128;
    
    /**
     * Constructs the native part of the class.
     *
     * @param count The descriptor count.
     */
    private native void nativeConstructor(int count);
    
    /**
     * Construct a DescriptorIOSpec with the given value.
     *
     * @param count The limit for open file descriptors.
     */
    public DescriptorIOSpec(int count)
    {
	if( count < 0 )
	{
	    throw new IllegalArgumentException("Negative descriptor count: "
					       + count);
	}
	this.nativeConstructor(count);
    }

    /**
     * @return The limit for open file descriptors.
     */
    public native int getCount();

    public String toString()
    {
	return "DescriptorIOSpec["
	    + super.toString()
	    + "; count="
	    + this.getCount()
	    + "]";
    }

    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof DescriptorIOSpec )
	{
	    DescriptorIOSpec dios;

	    dios = (DescriptorIOSpec)obj;
	    retval = (this.getCount() == dios.getCount());
	}
	return retval;
    }

    public Object clone()
    {
	return new DescriptorIOSpec(this.getCount());
    }

    /**
     * @return A copy of the current DescriptorIOSpec.
     */
    public static native DescriptorIOSpec current();

    /**
     * @return A copy of the default DescriptorIOSpec.
     */
    public static final DescriptorIOSpec defaultSpec()
    {
	return new DescriptorIOSpec(DEFAULT_COUNT);
    }
}
