/*
 * LogSpec.java
 * Resource specification for the log file.
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Resource specification for the log file.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class LogSpec
    extends ResourceSpec
{
    /**
     * Construct the native part of the class.
     */
    private native void nativeConstructor(String fileName);

    /**
     * Finalize the native part of the class.
     */
    private native void nativeFinalizer();

    /**
     * Construct a LogSpec with the given file name.
     *
     * @param fileName The name of the file to use as output for any log data
     * or an empty string if no logging should be done.
     */
    public LogSpec(String fileName)
    {
	if( fileName == null )
	{
	    throw new IllegalArgumentException("fileName can't be null");
	}
	this.nativeConstructor(fileName);
    }

    /**
     * @return The name of the file to use as output for any log data or an
     * empty string if no logging should be done.
     */
    public native String getFileName();

    protected void finalize()
    {
	this.nativeFinalizer();
    }

    /**
     * @return The LogSpec used to create the current team or null if the
     * system default log is being used.
     */
    public static native LogSpec current();
}
