/*
 * MemoryResource.java
 * Provides java level access to a team's memory resource.
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Provides java level access to the current team's memory resource.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class MemoryResource
{
    /**
     * @return The memory usage, in bytes, for the current team.
     */
    public static native long getUsage();
    
    /**
     * @return The peak usage, in bytes, for the current team.
     */
    public static native long getPeakUsage();
    
    /**
     * @return The maximum allowed memory, in bytes, for this team.
     */
    public static native long getMax();

    /**
     * @param lm A LiableMemory object that this team should be charged for.
     */
    // public static native void charge(LiableMemory lm);

    /**
     * @param size The number of bytes to credit to this team.
     */
    // static native void credit(long size);

    /**
     * @param obj An object to get the size of.
     * @return The number of bytes occupied by the given object.
     */
    static native long sizeof(Object obj);
    
    /**
     * Register an object suspected to be a leak.
     */
    public static native void registerLeak(Object obj);

    /**
     * Unregister an object suspected to be a leak.
     */
    public static native void unregisterLeak(Object obj);

    /**
     * Its a static only class.
     */
    private MemoryResource()
    {
    }
}
