/*
 * NameSpaceSpec.java
 * Resource specification for the root name space.
 *
 * Copyright (c) 2000, 2001, 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Resource specification for the root name space of a team.  This allows you
 * to specify which classes should be imported from other teams, thus allowing
 * the teams to share objects.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class NameSpaceSpec
    extends ResourceSpec
{
    /**
     * Construct the native part of the class.
     */
    private native void nativeConstructor(NameSpaceGroup nsg[]);

    /**
     * Finalize the native part of the class.
     */
    private native void nativeFinalizer();

    /**
     * Construct a name space resource specification with the given class
     * imports.
     *
     * @param nsg An array of NameSpaceGroup's which describe which class
     * groups should be imported into the team.
     */
    public NameSpaceSpec(NameSpaceGroup nsg[])
    {
	if( nsg == null )
	{
	    throw new IllegalArgumentException("nsg can't be null");
	}
	if( nsg.length == 0 )
	{
	    throw new IllegalArgumentException("nsg can't be zero length");
	}
	this.nativeConstructor(nsg);
    }

    public native int getGroupCount();

    public native NameSpaceGroup getGroup(int index);

    public NameSpaceGroup[] getGroups()
    {
	NameSpaceGroup retval[];
	int lpc;
	
	retval = new NameSpaceGroup[this.getGroupCount()];
	for( lpc = 0; lpc < retval.length; lpc++ )
	{
	    retval[lpc] = this.getGroup(lpc);
	}
	return retval;
    }

    protected void finalize()
    {
	this.nativeFinalizer();
    }

    public native Object clone();

    /**
     * @return The NameSpaceSpec used to create the current team.
     */
    public static native NameSpaceSpec current();

    private native int nativeToString(char[] cbuf);

    public String toString()
    {
	char[] cbuf = new char[1024];
	int used = nativeToString(cbuf);
	return new String(cbuf, 0, used);
    }
}
