/*
 * TeamListHandler.java
 * An interface for TeamList call backs.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * An interface for EventTeamHandle call backs.  Interest in 
 * team events is registered by using an {@link EventTeamHandle}.  When
 * an event occurs in the team, the EventTeamHandles are
 * notified, and if the event matches their interest filter,
 * the associated {@link TeamEventHandler} is invoked.
 *
 * <p>The same TeamEventHandler may be registered with more than one
 * EventTeamHandle.
 *
 * @see TeamEvent
 * @see EventTeamHandle
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public interface TeamEventHandler
{
    /**
     * Callback for handling events related to teams.  
     *
     * <p><b>NOTE</b>: This is called in the context of the kernel so
     * you must use the provided TeamReservation if you want to do
     * anything in the team that caused the Event.  The TeamEvent
     * object is allocated in the kernel Team context.
     *
     * <p><b>NOTE</b>: Do not modify the TeamEvent that is passed.  You will
     * confuse later receivers of the (same) event object.  Do not
     * store the TeamEvent object, as it will be modified for
     * subsequent events.
     *
     * @param eth The handle that refers to the team that triggered the event.
     * @param tr A TeamReservation thats reserved the team that `this' is in.
     * @param ev The TeamEvent object that describes the event that occurred.
     */
    void event(EventTeamHandle eth, TeamReservation tr, TeamEvent ev);
}
