/*
 * IODupFactory.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.io;

/**
 * XXX this shouldn't be public.
 */
public class IODupFactory
{
	private static final IODupFactory globalFactory = new IODupFactory();

	public static IODupFactory getFactory(/* XXX trusted key */) 
	{
		return globalFactory;
	}
	
	private IODupFactory()
	{
	}
	
	public FileDescriptor dup(FileDescriptor fd)
		throws IOException
	{
		return new FileDescriptor(fd);
	}

	public FileInputStream dup(FileInputStream fis)
		throws IOException
	{
		return new FileInputStream(dup(fis.getFD()));
	}

	public FileOutputStream dup(FileOutputStream fos)
		throws IOException
	{
		return new FileOutputStream(dup(fos.getFD()));
	}
	
	public FileInputStream makeFileInputStream(int fd)
		throws IOException
	{
		return new FileInputStream(FileDescriptor.wrapExisting(fd));
	}
	
	public FileOutputStream makeFileOutputStream(int fd)
		throws IOException
	{
		return new FileOutputStream(FileDescriptor.wrapExisting(fd));
	}
	
	public int getFD(FileInputStream fis)
		throws IOException
	{
		return fis.getFD().getIntFD();
	}
	
	public int getFD(FileDescriptor fd)
		throws IOException
	{
		return fd.getIntFD();
	}
	
	public int getFD(FileOutputStream fos)
		throws IOException
	{
		return fos.getFD().getIntFD();
	}
}
