/*
 * ExitStatus.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang;

import edu.utah.janosvm.sys.TeamLocal;

public final class ExitStatus
{
	public static final class Type
	{
		public static final Type NONE = new Type();
		
		public static final Type EXIT = new Type();
		
		public static final Type HALT = new Type();
		
		private Type()
		{
		}
	}
	
	private static final TeamLocal STATUS = new TeamLocal() {
			protected Object initialValue()
			{
				return new ExitStatus();
			}
		};

	public static ExitStatus getCurrentStatus()
	{
		return (ExitStatus)STATUS.get();
	}

	private Type type = Type.NONE;
	private int code;

	/* XXX private temporary hack */ ExitStatus()
	{
	}
	
	public synchronized void setTypeAndCode(Type type, int code)
	{
		if( (this.type == Type.NONE) || (type == Type.HALT) )
		{
			this.type = type;
			this.code = code;
		}
	}

	public Type getType()
	{
		return this.type;
	}

	public int getCode()
	{
		return this.code;
	}

	public String toString()
	{
		return "ExitStatus[type="
			+ this.type
			+ "; code="
			+ this.code
			+ "]";
	}
}
