/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */

package java.lang;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileNotFoundException;

import java.util.StringTokenizer;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamLocal;
import edu.utah.janosvm.sys.VMInstance;

public class Runtime
{
/**
 *  XXX This is not part of the public interface.
 */
public static interface MemoryAdvice {

	// These should match those in kaffe.lang.MemoryAdvice
	int GREEN = 0;
	int YELLOW = 1;
	int ORANGE = 2;
	int RED = 3;
}

private static final TeamLocal RUNTIME = new TeamLocal() {
		protected Object initialValue()
		{
			return new Runtime();
		}
	};
	
private static kaffe.lang.MemoryAdvice advice
	= kaffe.lang.MemoryAdvice.getInstance();

	
/* private XXX temporary hack */Runtime () {
}

public Process exec(String command) throws IOException {
	return exec(command, null, null);
}

public Process exec(String command, String envp[]) throws IOException {
	return exec(command, envp, null);
}

public Process exec(String command, String envp[], File dir)
		throws IOException {
	StringTokenizer tokenizer=new StringTokenizer(command);

	int count=tokenizer.countTokens();
	String cmdarray[]=new String[count];

	for (int pos=0; pos<count; pos++) {
		cmdarray[pos]=tokenizer.nextToken();
	}

	return exec(cmdarray, envp, dir);
}

public Process exec(String cmdarray[]) throws IOException {
	return exec(cmdarray, null, null);
}

public Process exec(String cmdarray[], String envp[]) throws IOException {
	return exec(cmdarray, envp, null);
}

public Process exec(String[] cmdarray, String[] envp, File dir)
		throws IOException {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null)
		sm.checkExec(cmdarray[0]);
	return execInternal(cmdarray, envp, dir);
}

private native Process execInternal(String cmdary[], String envp[], File dir)
	throws IOException;

public void exit(int status) {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null)
		sm.checkExit(status);
	Thread.currentThread().warnDeath();

	ExitStatus.getCurrentStatus().setTypeAndCode(ExitStatus.Type.EXIT,
						     status);
	
	/* 
	 * Make exit() exit the current Team.  Unless the
	 * kernel team invokes it, in which case, really
	 * exit the VM.
	 */
	Team t = Team.current();
	if( !t.equals(t.getKernelTeam()) )
	{
		t.terminate();
	}
	else
	{
		VMInstance.shutdown();
	}

	/*
	 * In case the current thread is a visitor, it needs
	 * to be booted from the team.  Destroy has that
	 * side-effect.  Obviously, if the current thread
	 * isn't visiting, it won't get this far. 
	 */
	Thread.currentThread().destroy();
}

public void halt(int status) {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null)
		sm.checkExit(status);
	Thread.currentThread().warnDeath();
	
	ExitStatus.getCurrentStatus().setTypeAndCode(ExitStatus.Type.HALT,
						     status);
	
	/* 
	 * Make halt terminate the current Team.  Unless the kernel
	 * team invokes it, in which case, really halt the VM.
	 */
	Team t = Team.current();
	if( !t.equals(t.getKernelTeam()) )
	{
		t.terminate();
	}
	else
	{
		exitInternal(status);
	}

	/*
	 * In case the current thread is a visitor, it needs
	 * to be booted from the team.  Destroy has that
	 * side-effect.  Obviously, if the current thread
	 * isn't visiting, it won't get this far. 
	 */
	Thread.currentThread().destroy();
}

native private void exitInternal(int status);

native public long freeMemory();

native public void gc();

/**
 * @deprecated
 */
public InputStream getLocalizedInputStream(InputStream in) {
	return (in);
}

/**
 * @deprecated
 */
public OutputStream getLocalizedOutputStream(OutputStream out) {
	return (out);
}

public static Runtime getRuntime() {
	return (Runtime)RUNTIME.get();
}

public void loadLibrary(String libname) {
	loadLibrary(libname, Class.getStackClass(1).getClassLoader());
}

void loadLibrary(String libname, ClassLoader loader) {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null)
		sm.checkLink(libname);
	String errmsg = libname + ": not found";
	String filename;
	String[] names;

	if (loader != null && (filename = loader.findLibrary(libname)) != null)
		names = new String[] { filename };
	else
		names = NativeLibrary.getLibraryNames(libname);
	for (int i = 0; i < names.length; i++) {
		try {
			new NativeLibrary(names[i], loader);
			return;
		} catch (FileNotFoundException e) {
		} catch (UnsatisfiedLinkError e) {
			errmsg = e.getMessage();
		}
	}
	throw new UnsatisfiedLinkError(errmsg);
}

public void load(String filename) {
	load(filename, Class.getStackClass(1).getClassLoader());
}

void load(String filename, ClassLoader loader) {
	SecurityManager sm = System.getSecurityManager();
	if (sm != null)
		sm.checkLink(filename);
	try {
		new NativeLibrary(filename, loader);
	}
	catch (FileNotFoundException e) {
		throw new UnsatisfiedLinkError(filename + ": not found");
	}
}

int getMemoryAdvice() {
	return (advice.getColor());
}

int waitForMemoryAdvice(int level) throws InterruptedException {
	return (advice.waitForOtherColor(level));
}

public void addShutdownHook(Thread hook) {
	// XXX implement me
	throw new kaffe.util.NotImplemented(
	    getClass().getName() + ".addShutdownHook()");
}

public boolean removeShutdownHook(Thread hook) {
	// XXX implement me
	return false;
}

native public void runFinalization();

native public static void runFinalizersOnExit(boolean value);

native public long totalMemory();

native public void traceInstructions(boolean on);

native public void traceMethodCalls(boolean on);
}
