/*
 * NetDupFactory.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.net;

import java.io.IODupFactory;
import java.io.FileDescriptor;
import java.io.IOException;

/**
 * XXX this shouldn't be public.
 */
public class NetDupFactory
{
	private static final NetDupFactory globalFactory = new NetDupFactory();
	private static final IODupFactory ioDupFactory = IODupFactory.getFactory();

	public static NetDupFactory getFactory(/* XXX trusted key */) 
	{
		return globalFactory;
	}
	
	private NetDupFactory()
	{
	}

	public FileDescriptor dup(FileDescriptor fd)
		throws IOException
	{
		return ioDupFactory.dup(fd);
	}
	
	public Socket dup(Socket s)
		throws IOException, SocketException
	{
		PlainSocketImpl impl = (PlainSocketImpl)(s.impl); /* XXX catch type cast error */
		return new Socket(new PlainSocketImpl(impl));
	}

	public DatagramSocket dup(DatagramSocket ds)
		throws IOException, SocketException
	{
		PlainDatagramSocketImpl impl = (PlainDatagramSocketImpl)(ds.impl); /* XXX catch type cast error */
		return new DatagramSocket(new PlainDatagramSocketImpl(impl));
	}

	public ServerSocket dup(ServerSocket ss)
		throws IOException, SocketException
	{
		PlainSocketImpl impl = (PlainSocketImpl)(ss.impl()); /* XXX catch type cast error */
		return new ServerSocket(new PlainSocketImpl(impl));
	}

	public Socket makeSocket(int fd)
	{
		throw new Error("NetDupFactory.makeSocket(int fd) unimplemented.");
	}
	
	public DatagramSocket makeDatagramSocket(int fd)
	{
		throw new Error("NetDupFactory.makeSocket(int fd) unimplemented.");
	}
	
	public ServerSocket makeServerSocket(int fd)
	{
		throw new Error("NetDupFactory.makeSocket(int fd) unimplemented.");
	}
	
	public int getFD(Socket s)
		throws IOException
	{
		SocketImpl simpl = s.impl;
		return ioDupFactory.getFD(simpl.getFD());
	}

	public int getFD(DatagramSocket ds)
		throws IOException
	{
		DatagramSocketImpl dsimpl = ds.getImpl();
		return ioDupFactory.getFD(dsimpl.getFD());
	}

	public int getFD(ServerSocket s)
		throws IOException
	{
		SocketImpl simpl = s.impl();
		return ioDupFactory.getFD(simpl.getFD());
	}
}
