/*
 * ErrorHelper.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoserv;

import java.lang.isolate.Link;
import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;

import java.io.IOException;
import java.io.PrintStream;

import java.net.Socket;

/**
 * ErrorHelper is an Isolate that will handle replying to requests that have
 * exihibited errors.
 */
public class ErrorHelper
    implements HTTPConstants,
	       Runnable
{
    /**
     * Main function for the ErrorHelper isolate.  First, it will create an
     * ErrorHelper instance with the Link stored in the first element of the
     * isolate's start messages.  Then, it will call ErrorHelper.run().
     *
     * @param args Ignored.
     */
    public static void main(String args[])
    {
	IsolateMessage im[];

	System.out.println("ErrorHelper is alive.");
	
	/* Get the start messages from our creator, the main server isolate. */
	im = Isolate.currentIsolateStartMessages();
	if( im.length > 0 )
	{
	    ErrorHelper eh;
	    
	    eh = new ErrorHelper(im[0].getLink());
	    eh.run();
	    /* Must've lost the main server, time to die. */
	}
	else
	{
	    System.err.println("Isolate Usage:");
	    System.err.println("\tArguments: (none)");
	    System.err.println("\tMessages: <Request link>");
	}
    }

    /**
     * The link used by the parent to send requests to us.
     */
    protected Link inputLink;

    /**
     * Construct an ErrorHelper with the given value.
     *
     * @param inputLink The link used by the parent to send requests to us.
     */
    public ErrorHelper(Link inputLink)
    {
	this.inputLink = inputLink;
    }

    /**
     * @see java.lang.Runnable#run()
     */
    public void run()
    {
	try
	{
	    IsolateMessage im[] = new IsolateMessage[4];
	    HTTPHeader hh = new HTTPHeader();

	    /*
	     * Initialize the response header with any values that are
	     * constant across uses.
	     */
	    hh.put("Content-type", "text/html");
	    
	    while( true )
	    {
		String file, responseString;
		int method, response;
		PrintStream ps;
		Socket socket;

		/* Get the messages from the server. */
		im = this.inputLink.receive().getComposite();
		
		/* Convert IsolateMessages to regular primitives/objects */
		socket = im[0].getSocket();
		try
		{
		    method = Integer.parseInt(im[1].getString());
		    file = im[2].getString();
		    response = Integer.parseInt(im[3].getString());
		    
		    ps = new PrintStream(socket.getOutputStream());

		    /* Build up the response */
		    hh.setResponse(response);

		    responseString = hh.getResponseString();
		    
		    ps.print(hh.toString());
		    ps.println("<html>");
		    ps.println("<head>");
		    ps.println("<title>" + responseString + "</title>");
		    ps.println("</head>");
		    ps.println("<body><h1>" + responseString + "</h1></body>");
		    ps.println("</html>");
		}
		catch(Throwable th)
		{
		    System.err.println("Exception in ErrorHelper:");
		    th.printStackTrace(System.err);
		}
		finally
		{
		    /* We're done with them. */
		    socket.close();
		}
	    }
	}
	catch(IOException e)
	{
	    /* Lost connection to the server. */
	    e.printStackTrace(System.err);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "ErrorHelper["
	    + "]";
    }
}
