/*
 * HTTPConstants.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoserv;

/**
 * Constants used in HTTP.
 *
 * Based on the tutorial by David Brown at:
 *
 * developer.java.sun.com/developer/technicalArticles/Networking/Webserver/
 */
public interface HTTPConstants
{
    /** HTTP methods */
    int HTTP_METHOD_GET = 1;
    int HTTP_METHOD_HEAD = 2;
    int HTTP_METHOD_UNSUPPORTED = 3;

    String HTTP_METHOD_STRINGS[] = {
	"GET",
	"HEAD",
    };

    /** End of line used by HTTP. */
    String HTTP_EOL = "\r\n";

    /** Supported version of HTTP. */
    String HTTP_VERSION = "HTTP/1.0";
    String HTTP_ALTERNATE_VERSION = "HTTP/1.1";
    
    /** 2XX: generally "OK" */
    int HTTP_OK = 200;
    int HTTP_CREATED = 201;
    int HTTP_ACCEPTED = 202;
    int HTTP_NOT_AUTHORITATIVE = 203;
    int HTTP_NO_CONTENT = 204;
    int HTTP_RESET = 205;
    int HTTP_PARTIAL = 206;

    String HTTP_2XX_STRINGS[] = {
	"OK",
	"Created",
	"Accepted",
	"Not Authoritative",
	"No Content",
	"Reset",
	"Partial",
    };

    /** 3XX: relocation/redirect */
    int HTTP_MULT_CHOICE = 300;
    int HTTP_MOVED_PERM = 301;
    int HTTP_MOVED_TEMP = 302;
    int HTTP_SEE_OTHER = 303;
    int HTTP_NOT_MODIFIED = 304;
    int HTTP_USE_PROXY = 305;

    String HTTP_3XX_STRINGS[] = {
	"Multiple Choice",
	"Moved Permanently",
	"Moved Temporarily",
	"See Other",
	"Not Modified",
	"Use Proxy",
    };

    /** 4XX: client error */
    int HTTP_BAD_REQUEST = 400;
    int HTTP_UNAUTHORIZED = 401;
    int HTTP_PAYMENT_REQUIRED = 402;
    int HTTP_FORBIDDEN = 403;
    int HTTP_NOT_FOUND = 404;
    int HTTP_BAD_METHOD = 405;
    int HTTP_NOT_ACCEPTABLE = 406;
    int HTTP_PROXY_AUTH = 407;
    int HTTP_CLIENT_TIMEOUT = 408;
    int HTTP_CONFLICT = 409;
    int HTTP_GONE = 410;
    int HTTP_LENGTH_REQUIRED = 411;
    int HTTP_PRECON_FAILED = 412;
    int HTTP_ENTITY_TOO_LARGE = 413;
    int HTTP_REQ_TOO_LONG = 414;
    int HTTP_UNSUPPORTED_TYPE = 415;

    String HTTP_4XX_STRINGS[] = {
	"Bad Request",
	"Unauthorized",
	"Payment Required",
	"Forbidden",
	"Not Found",
	"Bad Method",
	"Not Acceptable",
	"Proxy Authorization",
	"Client Timeout",
	"Conflict",
	"Gone",
	"Length Required",
	"Precondition Failed",
	"Entity Too Large",
	"Request Too Long",
	"Unsupported Type",
    };

    /** 5XX: server error */
    int HTTP_SERVER_ERROR = 500;
    int HTTP_INTERNAL_ERROR = 501;
    int HTTP_BAD_GATEWAY = 502;
    int HTTP_UNAVAILABLE = 503;
    int HTTP_GATEWAY_TIMEOUT = 504;
    int HTTP_VERSION_NOT_SUPPORTED = 505;
    
    String HTTP_5XX_STRINGS[] = {
	"Server Error",
	"Internal Error",
	"Bad Gateway",
	"Service Unavailable",
	"Gateway Timeout",
	"Version Not Supported",
    };
}
