/*
 * MIMETypes.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoserv;

import java.util.Hashtable;

/**
 * MIMETypes maps file name extensions to MIME types.
 */
public class MIMETypes
    extends Hashtable
{
    /**
     * Type used for unknown extensions.
     */
    public static final String UNKNOWN_TYPE = "text/plain";

    /**
     * Construct a MIMETypes object with the default mappings.
     */
    public MIMETypes()
    {
	super.put("", UNKNOWN_TYPE);
        super.put(".uu", "application/octet-stream");
        super.put(".exe", "application/octet-stream");
        super.put(".ps", "application/postscript");
        super.put(".zip", "application/zip");
        super.put(".sh", "application/x-shar");
        super.put(".tar", "application/x-tar");
        super.put(".snd", "audio/basic");
        super.put(".au", "audio/basic");
        super.put(".wav", "audio/x-wav");
        super.put(".gif", "image/gif");
        super.put(".jpg", "image/jpeg");
        super.put(".jpeg", "image/jpeg");
        super.put(".png", "image/png");
        super.put(".htm", "text/html");
        super.put(".html", "text/html");
        super.put(".text", "text/plain");
        super.put(".as", "text/plain");
        super.put(".c", "text/plain");
        super.put(".cc", "text/plain");
        super.put(".c++", "text/plain");
        super.put(".h", "text/plain");
        super.put(".pl", "text/plain");
        super.put(".txt", "text/plain");
        super.put(".java", "text/plain");
        super.put(".properties", "text/plain");
    }

    /**
     * Map a file name to a MIME type.
     *
     * @param fileName The file name to map.
     * @return The MIME type that matches the file name's extension or
     * UNKNOWN_TYPE if a match could not be found.
     */
    public String mapFileName(String fileName)
    {
	String retval = null;
	int index;

	if( (index = fileName.lastIndexOf('.')) > 0 )
	{
	    String ext;

	    ext = fileName.substring(index);
	    retval = (String)super.get(ext);
	}
	if( retval == null )
	{
	    retval = UNKNOWN_TYPE;
	}
	return retval;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "MIMETypes["
	    + super.toString()
	    + "]";
    }
}
