/*
 * ServletHelper.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoserv;

import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.net.Socket;

public class ServletHelper
    extends ClassLoader
    implements HTTPConstants,
	       Runnable
{
    public static final String RESTRICTED_PACKAGES[] = new String[] {
	"java.lang.isolate",
    };
    
    public static void main(String args[])
	throws Throwable
    {
	IsolateMessage im[];
	ServletHelper sh;

	im = Isolate.currentIsolateStartMessages();
	sh = new ServletHelper(im[0].getSocket(), args[0]);
	try
	{
	    sh.run();
	}
	finally
	{
	    System.exit(0);
	}
    }

    private Socket socket;
    private InputStream is;
    private OutputStream os;
    private String className;

    public ServletHelper(Socket socket, String className)
	throws IOException
    {
	this.socket = socket;
	this.is = socket.getInputStream();
	this.os = socket.getOutputStream();
	this.className = className;
    }

    private byte[] loadFile(String fileName)
	throws FileNotFoundException,
	       IOException
    {
	FileInputStream fis;
	byte retval[];
	File file;

	file = new File(fileName);
	retval = new byte[(int)file.length()];
	fis = new FileInputStream(fileName);
	fis.read(retval);
	return retval;
    }

    protected Class loadClass(String className, boolean resolve)
	throws ClassNotFoundException
    {
	Class retval;
	int lpc;

	for( lpc = 0; lpc < RESTRICTED_PACKAGES.length; lpc++ )
	{
	    if( className.startsWith(RESTRICTED_PACKAGES[lpc]) )
	    {
		throw new ClassNotFoundException(className
						 + ": class is in restricted "
						 + "package.");
	    }
	}
	retval = super.loadClass(className, resolve);
	if( resolve )
	{
	    this.resolveClass(retval);
	}
	return retval;
    }

    protected Class findClass(String className)
	throws ClassNotFoundException
    {
	String fileName;
	Class retval;

	fileName = className.replace('.', '/').concat(".class");
	try
	{
	    byte bits[];

	    bits = this.loadFile(fileName);
	    retval = this.defineClass(className, bits, 0, bits.length);
	}
	catch(IOException e)
	{
	    throw new ClassNotFoundException(className);
	}
	return retval;
    }

    public void run()
    {
	try
	{
	    IsolatedServlet ilet;
	    Class cl;

	    cl = Class.forName(this.className);
	    ilet = (IsolatedServlet)cl.newInstance();
	    ilet.handle(this.is, this.os);
	}
	catch(ClassNotFoundException e)
	{
	    PrintStream ps;

	    ps = new PrintStream(this.os);
	    ps.println(HTTP_VERSION
		       + " "
		       + HTTP_NOT_FOUND);
	    ps.println("");
	    ps.println(e.getMessage());
	}
	catch(Throwable th)
	{
	    PrintStream ps;

	    ps = new PrintStream(this.os);
	    ps.println(HTTP_VERSION
		       + " "
		       + HTTP_SERVER_ERROR);
	    th.printStackTrace(ps);
	}
    }

    public String toString()
    {
	return "ServletHelper["
	    + "]";
    }
}
