/*
 * InternalCommand.java
 *
 * Copyright (c) 2001-2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoshell;

import java.util.HashMap;

/**
 * For shell-internal commands and actions.
 *
 * @author Patrick Tullmann, University of Utah &lt;<a href="mailto:tullmann@cs.utah.edu">tullmann@cs.utah.edu</a>&gt;
 */
class InternalCommandManager
{
    private final HashMap internalActions = new HashMap();

    abstract static class Action
    {
	abstract void doAction(ParsedCommand cmd);
    }
    
    void addCommand(String name, Action action)
    {
	internalActions.put(name, action);
    }

    boolean isInternalCommand(String name)
    {
	return internalActions.containsKey(name);
    }
    
    void doInternalCommand(ParsedCommand cmd)
    {
	Action ia = (Action)internalActions.get(cmd.name());
	ia.doAction(cmd);
    }
}
