/*
 * PackagePath.java
 *
 * Copyright (c) 2001-2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoshell;

import java.util.StringTokenizer;

/**
 * Utilty class for storing a list of packages to search 
 * for commands.  Singleton.
 *
 * @author Patrick Tullmann, University of Utah &lt;<a href="mailto:tullmann@cs.utah.edu">tullmann@cs.utah.edu</a>&gt;
 */
class PackagePath
{
    private final String[] pathElems;
    
    /** Initialize with the given path. */
    PackagePath(String path)
    {
	// XXX Use system.pathdelim
	StringTokenizer strtok = new StringTokenizer(path, ":", false);
	
	pathElems = new String[strtok.countTokens()];
	int i = 0;
	while (i < pathElems.length)
	{
	    pathElems[i] = strtok.nextToken().intern();
	    if (pathElems[i].equals("."))
		pathElems[i] = "";
	    Trace.t(Trace.CLOOKUP, "Added '" +pathElems[i]+ "' to lookup path.");
	    i++;
	}
    }
    
    String find(String className)
    {
	for (int i = 0; i < pathElems.length; i++)
	{
	    String n;
	    try
	    {
		if (pathElems[i].equals(""))
		    n = className;
		else
		    n = pathElems[i] + "." + className;
		Trace.t(Trace.CLOOKUP, "Looking for '" +n+ "'...");
		Class c = Class.forName(n);
		return n;
	    }
	    catch (ClassNotFoundException _)
	    {
		// ignore this, its expected
	    }
	    catch (Throwable th)
	    {
		Trace.t(Trace.CLOOKUP, "Warning: " +th.getMessage());
	    }
	}
	
	return null;
    }
}


