/*
 * Command.java
 *
 * Copyright (c) 2001-2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.apps.isoshell;

import java.util.LinkedList;

/**
 * A user's command line is turned into a Command object which
 * encapsulates the name of the class and the arguments to pass to
 * that class.
 *
 * @author Patrick Tullmann, University of Utah &lt;<a href="mailto:tullmann@cs.utah.edu">tullmann@cs.utah.edu</a>&gt;
 */
class ParsedCommand
{
    /** Name of the command (or class). */
    private String name;
    
    /** Can be null, or empty, or full. */
    private String[] args;
    
    private boolean background;
    
    static ParsedCommand newInstance(LinkedList args, boolean background)
    {
	String cmdName = (String)(args.removeFirst());
	String[] argArray = new String[args.size()];
	args.toArray(argArray);
	return new ParsedCommand(cmdName, argArray, background);
    }

    static ParsedCommand newSimpleInstance(String cmdName)
    {
	return new ParsedCommand(cmdName);
    }


    private ParsedCommand(String name)
    {
	this(name, null, false);
    }

    private ParsedCommand(String name, String[] args)
    {
	this(name, args, false);
    }
      	
    private ParsedCommand(String name, String[] args, boolean background)
    {
	this.name = name;
	this.args = args;
	this.background = background;
    }

    public String name()
    {
	return this.name;
    }
    
    public void setName(String name)
    {
	this.name = name;
    }
    
    public boolean background()
    {
	return this.background;
    }
    
    public int argCount()
    {
	if (this.args != null)
	    return this.args.length;
	return 0;
    }
    
    public String[] args()
    {
	return this.args;
    }
    
    public String args(int i)
    {
	return this.args[i];
    }
    
    public String toString()
    {
	if (args == null)
	    return name + " + <no args>";
	
	String rc = name + " + " + args.length + " arguments";
	if (args.length > 0)
	{
	    boolean space = false;
	    rc += " (";
	    for (int i = 0; i < args.length; i++)
	    {
		if (space)
		    rc += " ";
		rc += "'" + args[i] + "'";
		space = true;
	    }
	    rc += ")";
	}
	
	if (this.background)
	    rc += " &";
	
	return rc;
    }
}

