/*
 * IsolateEventHandler.java
 * A TeamEventHandler listener for Isolate objects.
 * 
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.isolate;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.EventTeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.TeamEventHandler;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.NoSuchExportException;

final class IsolateEventHandler
    implements TeamEventHandler
{
    /**
     * The Isolate object paired with this object.
     */
    private Isolate isolate;

    /**
     * Construct a listener for the given Isolate object.
     *
     * @param isolate The Isolate object to pair with this object.
     */
    public IsolateEventHandler(Isolate isolate)
    {
	this.isolate = isolate;
    }

    public synchronized void event(EventTeamHandle eth,
				   TeamReservation tr,
				   TeamEvent ev)
    {
	try
	{
	    tr.visit();
	    {
		switch( ev.getEvent() )
		{
		case TeamEvent.FINISHED:
		    this.isolate.setState(Isolate.STATE_FINISHED);
		    break;
		case TeamEvent.TERMINATED:
		    this.isolate.setState(Isolate.STATE_DYING);
		    break;
		case TeamEvent.BURIED:
		    this.isolate.setState(Isolate.STATE_DEAD);
		    break;
		}
	    }
	    tr.leave();
	}
	catch(TeamEjectionException e)
	{
	}
    }

    public String toString()
    {
	return "IsolateEventHandler["
	    + super.toString()
	    + "; isolate="
	    + this.isolate
	    + "]";
    }
}
