/*
 * IsolateMainThread.java
 * An implementation of the JSR-121 straw man classes.
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.isolate;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import edu.utah.janosvm.sys.Team;

/**
 * A Runnable class that is run in an Isolate's "main" thread.
 */
final class IsolateMainThread
    implements Runnable
{
    /**
     * The "main" method to be called.
     */
    private final Method meth;

    /**
     * The arguments for meth;
     */
    private final Object args[];
    
    /**
     * Construct an empty IsolateMainThread.
     */
    IsolateMainThread(Method meth, Object args[])
    {
	this.meth = meth;
	this.args = args;
    }

    public void run()
    {
	Isolate iso = Isolate.getCurrentIsolate();
	
	try
	{
	    this.meth.invoke(null, this.args);
	}
	catch(InvocationTargetException e)
	{
	    e.getTargetException().printStackTrace();
	}
	catch(Throwable e)
	{
	    e.printStackTrace();
	}
	/* Become a daemon and wait for any other non-daemons to finish. */
	Thread.currentThread().setDaemon(true);
	iso.waitForState(Isolate.STATE_FINISHED, 0, 0);
	/* We're dead, remove ourself from the global mapping table. */
	iso.removeMapping();
	Team.current().terminate();
    }
    
    public String toString()
    {
	return "IsolateMainThread[meth="
	    + this.meth
	    + "; params="
	    + this.args
	    + "]";
    }
}
