/*
 * ServerResources.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.loader;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

import edu.utah.janosvm.resources.LogSpec;
import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.NativeSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.ClassPathSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.ResourceSpecException;
import edu.utah.janosvm.resources.InvalidClassGroupException;
import edu.utah.janosvm.resources.ViewGroupNotFoundException;
import edu.utah.janosvm.resources.ClassGroupNotFoundException;

public class ServerResources
{
    private static NameSpaceSpec getKernelNameSpaceSpec()
    {
	NameSpaceSpec retval = null;
	
	try
	{
	    TeamReservation tr;
	    TeamHandle kernel;
	    NameSpaceSpec nss;
	    Team curr;
	    
	    curr = Team.current();
	    kernel = curr.getKernelTeam();
	    tr = kernel.pushReservation();
	    tr.visit();
	    nss = NameSpaceSpec.current();
	    tr.leave();
	    retval = (NameSpaceSpec)nss.clone();
	    tr.pop();
	}
	catch(TeamEjectionException e)
	{
	}
	catch(DeadTeamException e)
	{
	}
	return retval;
    }
    
    protected ResourceSpec specs[];
    
    ServerResources(ResourceSpec rs[])
    {
	this.specs = rs;
    }

    public ServerResources()
    {
	this(new ResourceSpec[] {
	    new StdioSpec("builtin://stdin",
			  "builtin://stdout",
			  "builtin://stderr"),
	    new MemorySpec(2 * 1024 * 1024),
	    ClassPathSpec.current(),
	    NativeSpec.current(),
	    getKernelNameSpaceSpec(),
	});
    }

    ResourceSpec[] getResourceSpecs()
    {
	return this.specs;
    }

    public String toString()
    {
	return "ServerResources[specs="
	    + this.specs
	    + "]";
    }
}
