/*
 * CPUMonitor.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.resources.CPUResource;

/**
 * A simple CPU ResourceMonitor application.  It will print out the difference
 * in CPU usage for each team between polls.
 */
public final class CPUMonitor
    extends ResourceMonitor
{
    /**
     * A CPU ResourceSampler.
     */
    static final class CPUSampler
	extends ResourceSampler
    {
	/**
	 * The last value sampled.
	 */
	private long last;
	
	/**
	 * The difference between the last value sampled and the current.
	 */
	private long diff;

	/**
	 * Construct an empty CPUSampler.
	 */
	public CPUSampler()
	{
	}
	
	public void update()
	{
	    long curr;

	    curr = CPUResource.getUsage();
	    this.diff = curr - this.last;
	    this.last = curr;
	}

	public void empty()
	{
	    this.diff = 0;
	}

	public void output()
	{
	    System.out.print(" " + this.diff);
	}

	public String toString()
	{
	    return "CPUSampler["
		+ super.toString()
		+ "; last="
		+ this.last
		+ "; diff="
		+ this.diff
		+ "]";
	}
    }

    /**
     * Construct a CPUMonitor that uses the CPUSampler.
     */
    public CPUMonitor()
    {
	super(CPUSampler.class);
    }

    public void endSampling()
    {
	System.out.print("\n");
    }

    public String toString()
    {
	return "CPUMonitor["
	    + super.toString()
	    + "]";
    }
    
    public static void main(String args[])
	throws Throwable
    {
	/*
	 * Create the monitor, add the requested teams, and start the
	 * thread.
	 */
	new Thread(new CPUMonitor().addTeams(args)).start();
    }
}
