/*
 * MemoryMonitor.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.resources.MemoryResource;

/**
 * A simple Memory ResourceMonitor application.  It will print out the
 * difference in Memory usage for each team between polls.
 */
public final class MemoryMonitor
    extends ResourceMonitor
{
    /**
     * A Memory ResourceSampler.
     */
    static final class MemorySampler
	extends ResourceSampler
    {
	/**
	 * The last value sampled.
	 */
	private long last;
	
	/**
	 * Construct an empty MemorySampler.
	 */
	public MemorySampler()
	{
	}
	
	public void update()
	{
	    this.last = MemoryResource.getUsage();
	}

	public void empty()
	{
	    this.last = 0;
	}

	public void output()
	{
	    System.out.print(" " + this.last);
	}

	public String toString()
	{
	    return "MemorySampler["
		+ super.toString()
		+ "; last="
		+ this.last
		+ "]";
	}
    }

    /**
     * Construct a MemoryMonitor that uses the MemorySampler.
     */
    public MemoryMonitor()
    {
	super(MemorySampler.class);
    }

    public void endSampling()
    {
	System.out.print("\n");
    }

    public String toString()
    {
	return "MemoryMonitor["
	    + super.toString()
	    + "]";
    }
    
    public static void main(String args[])
	throws Throwable
    {
	/*
	 * Create the monitor, add the requested teams, and start the
	 * thread.
	 */
	new Thread(new MemoryMonitor().addTeams(args)).start();
    }
}
