/*
 * ResourceSampler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.sys.TeamHandle;

/**
 * Base class for objects that will be probing the team's resource state.
 */
public abstract class ResourceSampler
{
    /**
     * The team being sampled.
     */
    private TeamHandle th;

    /**
     * Construct an empty ResourceSampler.
     */
    public ResourceSampler()
    {
    }

    /**
     * @param th The team to sample.
     */
    public void setTeam(TeamHandle th)
    {
	this.th = th;
    }

    /**
     * @return The team to sample.
     */
    public TeamHandle getTeam()
    {
	return this.th;
    }

    /**
     * Update the sampler's state.  Note:  This is called while in a visit, so
     * nothing should be printed/manipulated/etc...
     */
    public abstract void update();

    /**
     * Update the sampler's state for a dead/unreachable team.
     */
    public abstract void empty();

    /**
     * Output the sampler state.  Note: Unlike update, this is called within
     * the monitor's team.
     */
    public abstract void output();

    public String toString()
    {
	return "ResourceSample[th="
	    + this.th
	    + "]";
    }
}
