/*
 * TeamEventMessage.java
 * A message class that stores the values from a TeamEvent.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.EventTeamHandle;

import edu.utah.janosvm.kit.comm.amiga.Message;

/**
 * A message class that stores the values from a TeamEvent.
 */
public class TeamEventMessage
    extends Message
{
    private EventTeamHandle eth;
    
    /**
     * The TeamEvent event number.
     */
    private int event;

    /**
     * Construct a TeamEventMessage with the same values as the given
     * TeamEvent.
     *
     * @param ev The TeamEvent object to copy values from.
     */
    TeamEventMessage(EventTeamHandle eth, TeamEvent ev)
    {
	this.eth = eth;
	this.event = ev.getEvent();
    }

    public EventTeamHandle getHandle()
    {
	return this.eth;
    }

    /**
     * @return The TeamEvent event number.
     */
    public int getEvent()
    {
	return this.event;
    }

    public String toString()
    {
	return "TeamEventMessage[eth="
	    + this.eth
	    + "; event="
	    + this.event
	    + "]";
    }
}
