/*
 * TeamEventMessenger.java
 * A TeamEventHandler implementation that sends TeamEventMessages to a MsgPort.
 *
 * Copyright (c) 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.kit.management.team;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.EventTeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.TeamEventHandler;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.NoSuchExportException;

import edu.utah.janosvm.kit.comm.amiga.MsgPort;

/**
 * A TeamEventHandler implementation that sends TeamEventMessages to a MsgPort.
 * Since the `event' call back happens on kernel time we don't want to waste
 * time so we just visit the other team, construct a message and send it off
 * to someone who can handle it better.
 */
public class TeamEventMessenger
    implements TeamEventHandler
{
    /**
     * The port we should send TeamEventMessage's too.
     */
    private MsgPort mp;

    /**
     * Construct a messenger that will send TeamEventMessage's to the given
     * MsgPort.
     *
     * @param mp A MsgPort that is willing to receive and process
     * TeamEventMessage's
     */
    public TeamEventMessenger(MsgPort mp)
    {
	this.mp = mp;
    }
    
    public void event(EventTeamHandle eth, TeamReservation tr, TeamEvent ev)
    {
	try
	{
	    /*
	     * We're in the kernel so visit our team to create and send the
	     * message.
	     */
	    tr.visit();
	    {
		TeamEventMessage tem = new TeamEventMessage(eth, ev);

		this.mp.putMessage(tem);
	    }
	    tr.leave();
	}
	catch(TeamEjectionException e)
	{
	}
    }

    public String toString()
    {
	return "TeamEventMessenger[mp"
	    + this.mp
	    + "]";
    }
}
