/*
 * IsolateGUID.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.lang.isolate;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;


/**
 * Globally unique 64-bit ID manager.  64-bit ids shouldn't ever
 * wrap.
 *
 * XXX rename to 'GUIDManager'
 */
/*package*/ class IsolateGUID
{
  private final Object lock = new Object();
  private long currentGUID;
  
  /*package*/ IsolateGUID(int initialValue)
  {
    currentGUID = initialValue;
  }

  /*package*/ long nextGUID()
  {
    TeamReservation kres;
    long guid;
    try
    {
      // ENTER KERNEL
      kres = Team.current().getKernelTeam().pushReservation();
      kres.visit();
      synchronized(lock)
      {
	currentGUID++;
	guid = currentGUID;
      }
      // EXIT KERNEL
      kres.leave();
      kres.pop();
    }
    catch (TeamEjectionException tee)
    {
      throw new InternalError("Fatal error", tee);
    }
    catch (DeadTeamException dte)
    {
      throw new InternalError("Fatal error", dte);
    }
    return guid;
  }
}
