/*
 * IsolateMessage.java
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.lang.isolate;

import edu.utah.janosvm.resources.*; // XXX
import edu.utah.janosvm.sys.*; // XXX
import edu.utah.janosvm.sys.TeamHandle;
//import java.nio.Buffer;
//import java.nio.ByteBuffer;
import java.net.Socket;
import java.net.DatagramSocket;
import java.net.ServerSocket;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

public abstract class IsolateMessage
{
  /**
   * Really a global value.  Immutable, and eternally referenceable, so
   * giving everyone in the system a pointer to this is okay by me.
   */
  public static final IsolateMessage EMPTY_MESSAGE = new IsolateMessage_Empty();

  /*package*/ IsolateMessage()
  {
  }

  // -- CONSTRUCTORS

  public static IsolateMessage newIsolateMessage(Isolate i)
    throws IllegalArgumentException
  {
    return new IsolateMessage_Isolate(i);
  }
  
  public static IsolateMessage newLinkMessage(Link l)
  {
    return new IsolateMessage_Link(l);
  }
  
  public static IsolateMessage newIsolateEventMessage(IsolateEvent e)
  {
    return new IsolateMessage_Event(e);
  }
  
  public static IsolateMessage newCompositeMessage(IsolateMessage[] m)
  {
    return new IsolateMessage_Composite(m);
  }
  
  // XXX no NIO
  //  public static IsolateMessage newByteBufferMessage(ByteBuffer b)
  //  {
  //    return null;
  //  }

  public static IsolateMessage newByteArrayMessage(byte[] b)
  {
    return new IsolateMessage_ByteArray(b);
  }
  
  public static IsolateMessage newStringMessage(String s)
  {
    return new IsolateMessage_String(s);
  }

  public static IsolateMessage newDatagramSocketMessage(DatagramSocket s)
  {
    return new IsolateMessage_DatagramSocket(s);
  }

  public static IsolateMessage newSocketMessage(Socket s)
  {
    return new IsolateMessage_Socket(s);
  }

  public static IsolateMessage newServerSocketMessage(ServerSocket s)
  {
    return new IsolateMessage_ServerSocket(s);
  }

  public static IsolateMessage newFileInputStreamMessage(FileInputStream f)
  {
    return new IsolateMessage_FileInputStream(f);
  }

  public static IsolateMessage newFileOutputStreamMessage(FileOutputStream f)
  {
    return new IsolateMessage_FileOutputStream(f);
  }

  public static IsolateMessage newSerializableMessage(Object o)
  {
    return new IsolateMessage_Serializable(o);
  }
  

  // -- ACCESSORS

  public Isolate getIsolate()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not an Isolate message");
  }
  
  public IsolateMessage[] getComposite()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not a composite message");
  }
  
  public Link getLink()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not a Link message");
  }
  
  public IsolateEvent getIsolateEvent()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not an IsolateEvent message: " + this);
  }

  public FileInputStream getFileInputStream()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not an FileInputStream message");
  }

  public FileOutputStream getFileOutputStream()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not an FileOutputStream message");
  }

  //   public ByteBuffer getByteBuffer()
  //     throws IllegalStateException
  //   {
  //     return null;
  //   }
  
  public byte[] getByteArray()
    throws IllegalStateException
  {
    throw new IllegalStateException("Not a byte[] message");
  }
  
  public String getString()
    throws IllegalStateException
  {
    throw new IllegalStateException();
  }
  
  public DatagramSocket getDatagramSocket()
    throws IllegalStateException
  {
    throw new IllegalStateException();
  }

  public Socket getSocket()
    throws IllegalStateException
  {
    throw new IllegalStateException();
  }

  public ServerSocket getServerSocket()
    throws IllegalStateException
  {
    throw new IllegalStateException();
  }

  public Object getSerializable()
    throws IllegalStateException, LinkSerializationException
  {
    throw new IllegalStateException();
  }

  public abstract void acceptVisitor(IsolateMessageVisitor lmv)
    throws LinkSerializationException;

  /*package*/ String typeName()
  {
    return "IsolateMessage";
  }

  public String toString()
  {
    return typeName()
      + "@0x"
      + Integer.toHexString(System.identityHashCode(this));
  }

  // -- Internal accessors

  /** 
   * Return true if this message contains an IO object suitable for
   * stdio redirection.
   */
  /*package*/ abstract boolean isIOMessage();

  /**
   * Set up a message for translation.  (Only used by serializables to
   * make sure they're serialized before sending).  Used by composite, too.
   */
  void pretranslate()
    throws LinkSerializationException
  {
  }

  /** 
   * INTERNAL translation of IsolateMessage
   *
   * XXX thrown exception is odd?  (IOException for fd duping errors)
   */
  /*package*/ abstract IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException;
}

