/*
 * IsolateMessageVisitor.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.lang.isolate;

import java.net.Socket;
import java.net.ServerSocket;

public abstract class IsolateMessageVisitor
{
  public void visitIsolateMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept Isolate messages");
  }

  public void visitLinkMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept Link messages");
  }

  public void visitCompositeMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept Composite messages");
  }

  public void visitEmptyMessage() {
    throw new IllegalStateException(this + " does not accept the empty message");
  }


//  public void visitByteBufferMessage(IsolateMessage lm) {
//    throw new IllegalStateException(this + " does not accept ByteBuffer messages");
//  }

  public void visitByteArrayMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept byte[] messages");
  }

  public void visitStringMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept String messages");
  }

  public void visitDatagramSocketMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept DatagramSocket messages");
  }

  public void visitSocketMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept Socket messages");
  }

  public void visitServerSocketMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept ServerSocket messages");
  }

  public void visitFileInputStreamMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept FileInputStream messages");
  }

  public void visitFileOutputStreamMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept FileOutputStream messages");
  }

  public void visitSerializableMessage(IsolateMessage lm) 
    throws LinkSerializationException 
  {
    throw new IllegalStateException(this + " does not accept Serializable messages");
  }

//  public void visitPipeSourceMessage(IsolateMessage lm) {
//    throw new IllegalStateException(this + " does not accept Pipe.Source messages");
//  }
//
//  public void visitPipeSinkMessage(IsolateMessage lm) {
//    throw new IllegalStateException(this + " does not accept Pipe.Sink messages");
//  }
//
  public void visitIsolateEventMessage(IsolateMessage lm) {
    throw new IllegalStateException(this + " does not accept IsolateEvent messages");
  }
}
