/*
 * IsolateMessage_ByteArray.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_ByteArray
  extends IsolateMessage_DataMessage
{
  /*protected*/ final byte[] b;

  /*protected*/ IsolateMessage_ByteArray(byte[] b)
  {
    if( b == null )
      throw new NullPointerException("Null message value not allowed");
    this.b = b;
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitByteArrayMessage(this);
  }

  public byte[] getByteArray()
  {
    return this.b;
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_ByteArray)
    {
      IsolateMessage_ByteArray other = (IsolateMessage_ByteArray)o;
      return this.b == other.b;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    byte[] rc = new byte[this.b.length];
    System.arraycopy(this.b, 0, rc, 0, this.b.length);
    return new IsolateMessage_ByteArray(rc);
  }
}
