/*
 * IsolateMessage_Composite.java
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import java.io.IOException;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_Composite
  extends IsolateMessage_DataMessage
{
  /*protected*/ final IsolateMessage[] m;

  /*protected*/ IsolateMessage_Composite(IsolateMessage[] m)
  {
    if( m == null )
      throw new NullPointerException("Null message value not allowed");
    // Make a deep copy of the array so its immutable
    int arraylen = m.length;
    this.m = new IsolateMessage[arraylen];
    for (int i = 0; i < arraylen; i++)
    {
      if (m[i] == null)
	throw new IllegalArgumentException("null elements of composite message initializer not allowed.  Index=" + i);
      this.m[i] = m[i];
      
    }
  }

  /*package*/ String typeName()
  {
    return "IsolateMessage<Composite>";
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitCompositeMessage(this);
  }

  public IsolateMessage[] getComposite()
  {
    // Make a deep copy of the array so users can't #$@! with mine.
    return (IsolateMessage[])this.m.clone();
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_Composite)
    {
      IsolateMessage_Composite other = (IsolateMessage_Composite)o;
      if (other.m.length == this.m.length)
      {
	for (int i = 0; i < this.m.length; i++)
	  if (! this.m[i].equals(other.m[i]))
	    return false;
	return true;
      }
    }
    return false;
  }

  void pretranslate()
    throws LinkSerializationException
  {
    int lpc;

    for( lpc = 0; lpc < this.m.length; lpc++ )
    {
      this.m[lpc].pretranslate();
    }
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    IsolateMessage[] clonedArray = new IsolateMessage[this.m.length];
    for (int i = 0; i < this.m.length; i++)
    {
      // assert this.m[i] != null
      
      clonedArray[i] = this.m[i].translate(sourceTeam, currentTeam);
    }

    return new IsolateMessage_Composite(clonedArray);
  }
}
