/*
 * IsolateMessage_DatagramSocket.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import java.net.DatagramSocket;
import java.net.NetDupFactory;
import java.io.IOException;

class IsolateMessage_DatagramSocket
  extends IsolateMessage_IOMessage
{
  private static final NetDupFactory netDupFactory = NetDupFactory.getFactory();

  /*protected*/ final DatagramSocket s;

  /*protected*/ IsolateMessage_DatagramSocket(DatagramSocket s)
  {
    // XXX make sure its the right kind of DatagramSocket

    if( s == null )
      throw new NullPointerException("Null message value not allowed");
    this.s = s;
  }

  /*package*/ int getFD()
    throws IOException
  {
    return netDupFactory.getFD(this.s);
  }

  /*package*/ IsolateMessage_IOMessage cloneTypeAroundFD(int fd)
  {
    return new IsolateMessage_DatagramSocket(netDupFactory.makeDatagramSocket(fd));
  }

  public DatagramSocket getDatagramSocket()
  {
    return s;
  }
  
  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitDatagramSocketMessage(this);
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_DatagramSocket)
    {
      IsolateMessage_DatagramSocket other = (IsolateMessage_DatagramSocket)o;
      return this.s == other.s;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return IsolateMessage.newDatagramSocketMessage(netDupFactory.dup(this.s));
  }
}
