/*
 * IsolateMessage_Event.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_Event
  extends IsolateMessage_DataMessage
{
  /*protected*/ final IsolateEvent e;

  /*protected*/ IsolateMessage_Event(IsolateEvent e)
  {
    if( e == null )
      throw new NullPointerException("Null message value not allowed");
    this.e = e;
  }

  /*package*/ String typeName()
  {
    return "IsolateMessage<IsolateEvent>";
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitIsolateEventMessage(this);
  }

  public IsolateEvent getIsolateEvent()
  {
    return this.e;
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_Event)
    {
      IsolateMessage_Event other = (IsolateMessage_Event)o;
      return this.e == other.e;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return new IsolateMessage_Event(this.e.clone(sourceTeam, currentTeam));
  }
}
