/*
 * IsolateMessage_FileOutputStream.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import java.io.FileOutputStream;
import java.io.IODupFactory;
import java.io.IOException;

class IsolateMessage_FileOutputStream
  extends IsolateMessage_IOMessage
{
  private static final IODupFactory ioDupFactory = IODupFactory.getFactory();

  /*protected*/ final FileOutputStream s;

  /*protected*/ IsolateMessage_FileOutputStream(FileOutputStream s)
  {
    // XXX make sure its the right kind of FileOutputStream

    if( s == null )
      throw new NullPointerException("Null message value not allowed");
    this.s = s;
  }

  public FileOutputStream getFileOutputStream()
  {
    return s;
  }
  
  /*package*/ int getFD()
    throws IOException
  {
    return ioDupFactory.getFD(this.s);
  }
  
  /*package*/ boolean isWritable()
  {
    return true;
  }

  /*package*/ IsolateMessage_IOMessage cloneTypeAroundFD(int fd)
    throws IOException
  {
    return new IsolateMessage_FileOutputStream(ioDupFactory.makeFileOutputStream(fd));
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitFileOutputStreamMessage(this);
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_FileOutputStream)
    {
      IsolateMessage_FileOutputStream other = (IsolateMessage_FileOutputStream)o;
      return this.s == other.s;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return IsolateMessage.newFileOutputStreamMessage(ioDupFactory.dup(this.s));
  }
}
