/*
 * IsolateMessage_IOMessage.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */
package java.lang.isolate;

import java.io.IOException;

/**
 * All IsolateMessages that wrap IO objects extend this class.  The
 * extra interfaces are used by the stdio initialization code in
 * the Isolate constructors.
 */
public abstract class IsolateMessage_IOMessage
	extends IsolateMessage
{
  /*package*/ IsolateMessage_IOMessage()
  {
  }

  /*package*/ boolean isIOMessage()
  {
    return true;
  }

  /**
   * To be suitable for an input stream, an IO message type must
   * override this to return true.  (Defaults to false).
   */
  /*package*/ boolean isReadable()
  {
    return false;
  }

  /**
   * To be suitable for an output stream, an IO message type must
   * override this to return true.  (Defaults to false).
   */
  /*package*/ boolean isWritable()
  {
    return false;
  }

  /**
   * All IOMessages must implement this method.  They must return
   * a new instance of themselves but wrapping the given fd.  
   */
  /*package*/ abstract IsolateMessage_IOMessage cloneTypeAroundFD(int fd)
    throws IOException;

  /** 
   * IO-type messages must implement this for JanosVM stdio
   * redirection, see Isolate.makeStdioSpec().
   */
  /*package*/ abstract int getFD()
    throws IOException;
}
