/*
 * IsolateMessage_Isolate.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_Isolate
  extends IsolateMessage_DataMessage
{
  /*protected*/ final Isolate i;

  /*protected*/ IsolateMessage_Isolate(Isolate i)
  {
    if( i == null )
      throw new NullPointerException("Null message value not allowed");
    if (! i.hasStarted())
      throw new IllegalStateException("Illegal to put non-started isolate in a message");

    this.i = i;
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitIsolateMessage(this);
  }

  public Isolate getIsolate()
  {
    return this.i;
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_Isolate)
    {
      IsolateMessage_Isolate other = (IsolateMessage_Isolate)o;
      return this.i == other.i;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return new IsolateMessage_Isolate(this.i.clone(sourceTeam, currentTeam));
  }
}
