/*
 * IsolateMessage_Link.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_Link
  extends IsolateMessage_DataMessage
{
  /*protected*/ final Link l;

  /*protected*/ IsolateMessage_Link(Link l)
  {
    if( l == null )
      throw new NullPointerException("Null message value not allowed");
    this.l = l;
  }

  /*package*/ String typeName()
  {
    return "IsolateMessage<Link>";
  }

  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitLinkMessage(this);
  }

  public Link getLink()
  {
    return this.l;
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_Link)
    {
      IsolateMessage_Link other = (IsolateMessage_Link)o;
      return this.l == other.l;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return new IsolateMessage_Link(this.l.clone(sourceTeam, currentTeam));
  }
}
