/*
 * IsolateMessage_ServerSocket.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import java.net.ServerSocket;
import java.net.NetDupFactory;
import java.io.IOException;

// XXX this probably shouldn't be an IOMessage?
class IsolateMessage_ServerSocket
  extends IsolateMessage_IOMessage
{
  private static final NetDupFactory netDupFactory = NetDupFactory.getFactory();

  /*protected*/ final ServerSocket s;

  /*protected*/ IsolateMessage_ServerSocket(ServerSocket s)
  {
    // XXX check that s is appropriate type of socket

    if( s == null )
      throw new NullPointerException("Null message value not allowed");
    this.s = s;
  }

  /*package*/ int getFD()
    throws IOException
  {
    return netDupFactory.getFD(this.s);
  }

  /*package*/ IsolateMessage_IOMessage cloneTypeAroundFD(int fd)
  {
    return new IsolateMessage_ServerSocket(netDupFactory.makeServerSocket(fd));
  }

  public ServerSocket getServerSocket()
  {
    return s;
  }
  
  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitServerSocketMessage(this);
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_ServerSocket)
    {
      IsolateMessage_ServerSocket other = (IsolateMessage_ServerSocket)o;
      return this.s == other.s;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return IsolateMessage.newServerSocketMessage(netDupFactory.dup(this.s));
  }
}
