/*
 * IsolateMessage_Socket.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;
import java.net.Socket;
import java.net.NetDupFactory;
import java.io.IOException;

class IsolateMessage_Socket
  extends IsolateMessage_IOMessage
{
  private static final NetDupFactory netDupFactory = NetDupFactory.getFactory();

  /*protected*/ final Socket s;

  /*protected*/ IsolateMessage_Socket(Socket s)
  {
    // XXX make sure its the right kind of Socket

    if( s == null )
      throw new NullPointerException("Null message value not allowed");
    this.s = s;
  }

  /*package*/ int getFD()
    throws IOException
  {
    return netDupFactory.getFD(this.s);
  }

  /*package*/ boolean isReadable()
  {
    return true;
  }

  /*package*/ boolean isWritable()
  {
    return true;
  }

  /*package*/ IsolateMessage_IOMessage cloneTypeAroundFD(int fd)
  {
    return new IsolateMessage_Socket(netDupFactory.makeSocket(fd));
  }

  public Socket getSocket()
  {
    return s;
  }
  
  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitSocketMessage(this);
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_Socket)
    {
      IsolateMessage_Socket other = (IsolateMessage_Socket)o;
      return this.s == other.s;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException, IOException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return IsolateMessage.newSocketMessage(netDupFactory.dup(this.s));
  }
}
