/*
 * IsolateMessage_String.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.Team;

class IsolateMessage_String
  extends IsolateMessage_DataMessage
{
  /*protected*/ final String s;

  /*protected*/ IsolateMessage_String(String s)
  {
    if( s == null )
      throw new NullPointerException("Null message value not allowed");
    this.s = s;
  }

  String s()
  {
    return this.s;
  }

  public String getString()
  {
    return s;
  }
  
  public void acceptVisitor(IsolateMessageVisitor lmv)
  {
    lmv.visitStringMessage(this);
  }

  public boolean equals(Object o)
  {
    if (o instanceof IsolateMessage_String)
    {
      IsolateMessage_String other = (IsolateMessage_String)o;
      return this.s == other.s;
    }
    return false;
  }

  /** 
   * INTERNAL translation of IsolateMessage
   */
  IsolateMessage translate(TeamHandle sourceTeam, TeamHandle currentTeam)
    throws ClosedLinkException
  {
    Assert.ensure(sourceTeam != null);
    Assert.ensure(currentTeam != null);
    Assert.ensure(currentTeam.equals(Team.current()));

    return new IsolateMessage_String(Isolate.cloneString(this.s));
  }
}
