/*
 * IsolateServer.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.NoSuchExportException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.LogSpec;
import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.NativeSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.ClassPathSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.NameSpaceGroup;
import edu.utah.janosvm.resources.NameSpaceResource;
import edu.utah.janosvm.resources.ResourceSpecException;
import edu.utah.janosvm.resources.InvalidClassGroupException;
import edu.utah.janosvm.resources.ViewGroupNotFoundException;
import edu.utah.janosvm.resources.ClassGroupNotFoundException;

/**
 * An IsolateServer is a TeamHandle that refers to the team containing
 * any isolate global data (e.g. classes, status objects).  The team
 * that contains the global data must be created before any isolate
 * code executes, this is usually done by a
 * <code>edu.utah.janosvm.kit.loader.Loader</code>.
 *
 * <p>XXX rename to IsolateServerHandle
 */
/*package*/ class IsolateServer
  extends TeamHandle
{
  /**
   * Cached referenced to the this class' Class object.  Used to identify which
   * team exported the isolate classes.
   */
  private static Class THIS_CLASS = IsolateServer.class;

  /**
   * @return An IsolateServer object that refers to the isolate server team.
   */
  /*package*/ static IsolateServer findServer()
  {
    IsolateServer retval = null;
    Team curr;

    curr = Team.current();
    if( curr.owns(THIS_CLASS) )
    {
      /* The current team loaded the classes?  not good... */
      if( false )
      {
	retval = new IsolateServer(curr);
      }
      else
      {
	throw new InternalError("Don't forget to use the loader.");
      }
    }
    else
    {
      NameSpaceGroup nsg[];
      NameSpaceSpec nss;
      int lpc;

      /*
       * Search the list of imported class groups for the team containing the
       * isolate classes.
       */
      nss = NameSpaceSpec.current();
      nsg = nss.getGroups();
      for( lpc = 0; (lpc < nsg.length) && (retval == null); lpc++ )
      {
	TeamHandle th;

	th = nsg[lpc].getTeam();
	if( th.owns(THIS_CLASS) )
	{
	  /* Found the exporter. */
	  retval = new IsolateServer(th);
	}
      }
    }
    return retval;
  }

  /**
   * Construct an IsolateServer object that refers to the isolate server team.
   *
   * @param th A handle on the isolate server team.
   */
  IsolateServer(TeamHandle th)
  {
    super(th);
  }

  public String toString()
  {
    return "IsolateServer["
      + super.toString()
      + "]";
  }
}
