/*
 * IsolateStartupException.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import java.io.PrintStream;
import java.io.PrintWriter;

public class IsolateStartupException extends Exception 
{
  private final String remoteName;
  private final String remoteMessage;
  private final Object[] trace;

  /*package*/ IsolateStartupException(String detail, Exception other)
  {
    this(detail, other.getClass().getName(), other.getMessage(), null);
  }

  public IsolateStartupException()
  {
    this(null, null, null, null);
  }

  public IsolateStartupException(String detail)
  {
    this(detail, null, null, null);
  }

  public IsolateStartupException(String detail, String remoteName, 
				 String remoteMessage, Object[] trace) 
  {
    super(detail);
    this.remoteName = remoteName;
    this.remoteMessage = remoteMessage;
    this.trace = trace;
  }

  public String getRemoteMessage()
  {
    return this.remoteMessage;
  }

  public String getRemoteName() {
    return this.remoteName;
  }

  public Object[] getRemoteStackTrace() {
    return this.trace;
  }
  
  /**
   * Prints the remote exception name, message, and stack trace, like
   * {@link java.lang.Throwable#printStackTrace()} does for the local
   * exception.  If no exception information is available, nothing
   * will be printed.
   */
  public void printRemoteStackTrace() {
    printRemoteStackTrace(System.out);
  }
  
  /**
   * Prints the remote exception like {@link #printRemoteStackTrace()}
   * to the given <code>PrintStream</code>.
   * If no exception information is available, nothing
   * will be printed.
   */
  public void printRemoteStackTrace(PrintStream ps) {
    printRemoteStackTrace(new PrintWriter(ps));
  }

  /**
   * Prints the remote exception like {@link #printRemoteStackTrace()}
   * to the given <code>PrintWriter</code>.
   * If no exception information is available, nothing
   * will be printed.
   */
  public void printRemoteStackTrace(PrintWriter pw) {
    if (this.remoteName != null)
    {
      if (this.remoteMessage != null)
	pw.println(this.remoteName + ": " + this.remoteMessage);
      else
	pw.println(this.remoteName + ":");
    }
  }
}
