/*
 * IsolateState.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;


/**
 * State of an Isolate.  Kept in the IsolateBackEnd where it is the
 * "true" state of the Isolate.  Also stored in Handles where it may
 * be not always be correct, but it always is a "minimum" state for
 * the real Isolate.  (I.e., a handle may say "RUNNING", but the
 * actual Isolate could be DYING or DEAD or gone completely, but not
 * INVALID or NEWBORN.)
 */
/*package*/ class IsolateState
{
  private final String name; // XXX debugging only
  private final int id;
  private IsolateState(int id, String name)
  {
    this.id = id;
    this.name = name;
  }

  /*package*/ static final IsolateState INVALID  = new IsolateState(0, "INVALID"); // just barely created
  /*package*/ static final IsolateState NEWBORN  = new IsolateState(1, "NEWBORN"); // constructor completed
  /*package*/ static final IsolateState STARTING = new IsolateState(2, "STARTING"); // start invoked
  /*package*/ static final IsolateState RUNNING  = new IsolateState(3, "RUNNING"); // main thread running
  /*package*/ static final IsolateState DYING    = new IsolateState(4, "DYING"); // started exit process
  /*package*/ static final IsolateState DEAD     = new IsolateState(5, "DEAD"); // dead and buried, 

  boolean atLeast(IsolateState other)
  {
    return this.id >= other.id;
  }
  

  boolean comesAfter(IsolateState other)
  {
    return this.id > other.id;
  }

  boolean comesBefore(IsolateState other)
  {
    return this.id < other.id;
  }

  public String toString()
  {
    return "IsolateState[" +name+ "]";
  }
}
