/*
 * MainThread.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package java.lang.isolate;

import edu.utah.janosvm.resources.*; // XXX
import edu.utah.janosvm.sys.*; // XXX
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import java.util.prefs.TransientPreferences;

/**
 * The main thread for a new Isolate.
 */
/*package*/ class MainThread
	extends Thread
{
  private static final int defaultExitStatus = 0;
  private final String classname;
  private final String[] args;

  /*package*/ MainThread(String classname,
			 String[] mainClassArgs)
  {
    this.classname = classname;
    this.args = mainClassArgs;
  }

  private static void printError(Throwable th)
  {
    th.printStackTrace(System.err);

    // Assume we'll die with this uncaught exception.  Will get
    // overridden by something else if not...
    IsolateBackEnd.current().setExitInfo(defaultExitStatus,
					 IsolateEvent.ExitReason.UNCAUGHT_EXCEPTION);
  }

  public void run()
  {
    IsolateBackEnd.current().mainThreadRunning();

    // XXX install threadgroup uncaught exception handler???

    // XXX explain what happens to exceptions

    Assert.ensure(this.classname != null);
    Assert.ensure(this.args != null);
    Assert.ensure(Team.current().owns(this.classname));
    Assert.ensure(Team.current().owns(this.args));

    try
    {
      Class mainClass;
      mainClass = NameSpaceResource.loadTeamClass(this.classname);
      // XXX catch
      

      Class[] mainSig = { this.args.getClass() };
      Method m;
      
      m = mainClass.getMethod("main", mainSig);
      // XXX catch
      
      if (!Modifier.isStatic(m.getModifiers()))
	throw new NoSuchMethodException("static main");
      
      // System.out.println("mainClass= " +mainClass);
      // System.out.println("method = " +m);

      m.invoke(null, new Object[] { this.args } );
    }
    catch(ClassNotFoundException e)
    {
      printError(e);
    }
    catch(NoSuchMethodException e)
    {
      printError(e);
    }
    catch(ClassFormatError e)
    {
      printError(e);
    }
    catch(NoClassDefFoundError e)
    {
      printError(e);
    }
    catch(InvocationTargetException e)
    {
      printError(e.getTargetException());
    }
    catch(Throwable e)
    {
      printError(e);
    }
  }
}
