#
# Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

from janosvm import *

class Machine:
    def __init__(self, jvm):
        self.jvm = jvm
        self.jvm.msg.pushSpecifier("machine")

    def getName(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("name")
        reply = self.jvm.send()
        return reply.find("result")

    def getCPU(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("cpu")
        reply = self.jvm.send()
        return (long(reply.find("used")), long(reply.find("idle")))

    def getUptime(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("uptime")
        reply = self.jvm.send()
        return long(reply.find("result"))

    def getMemory(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("memory")
        reply = self.jvm.send()
        return (long(reply.find("used")), long(reply.find("max")))

    def getNICs(self):
        self.jvm.msg.setWhat(JSI_LIST_PROPERTY)
        self.jvm.msg.pushSpecifier("nic")
        reply = self.jvm.send()
        retval = {}
        lpc = 0
        count = reply.identifierCount("name")
        while lpc < count:
            name = reply.find("name", lpc)
            if retval.has_key(name):
                addrs = retval[name]
            else:
                addrs = []
                retval[name] = addrs
            addrs.append((reply.find("type", lpc),
                          reply.find("address")))
            lpc = lpc + 1
        return retval

    def getIO(self):
        self.jvm.msg.setWhat(JSI_GET_PROPERTY)
        self.jvm.msg.pushSpecifier("io")
        reply = self.jvm.send()
        return {
            "ipc" : (long(reply.find("ipc.in")),
                     long(reply.find("ipc.out"))),
            "disk" : (long(reply.find("disk.in")),
                      long(reply.find("disk.out"))),
            "network" : (long(reply.find("network.in")),
                         long(reply.find("network.out"))),
            }
