#! /usr/local/bin/python
#
# Copyright (c) 2003 The University of Utah and the Flux Group.
# All rights reserved.
# 
# This file is part of the JanosVM documentation.
# 
# Permission to use, copy, modify, distribute, and sell this
# documentation for any purpose is hereby granted without fee,
# provided that the above copyright notice(s) appear in all copies.
#
# Author: Tim Stack
#

import re
import sys
import time
import getopt

from socket import *

from jsi.channel import *
from jsi.message import *
from jsi.messageError import *
from janosvm.janosvm import *

default_target = "tcp:localhost/8192"
default_port = 19150

target = default_target
port = default_port

def usage():
    print "Usage: " + sys.argv[0] + " [options]"
    print
    print "Adapt JanosVM JSI properties to gkrellm server protocol."
    print
    print "Options:"
    print "\t-h, --help\tPrint this help."
    print "\t-t, --target\tThe target JanosVM. (default: " + default_target + ")"
    print "\t-p, --port\tThe server port to listen on. (default: " + `default_port` + ")"

try:
    opts, args = getopt.getopt(sys.argv[1:],
                               "ht:p:",
                               [ "help",
                                 "target=",
                                 "port=",
                                 ])
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        if o in ("-t", "--target"):
            target = a
        elif o in ("-p", "--port"):
            try:
                port = int(a)
            except ValueError:
                print "Error: Bad port value: " + a
                usage()
                sys.exit()
except getopt.GetoptError:
    usage()
    sys.exit(2)

jvm = JanosVM(jsiConnectChannelTo(target))

sock = socket(AF_INET, SOCK_STREAM)
sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
sock.bind(("0.0.0.0", port))
sock.listen(1)

client = sock.accept()

clisock = client[0]

done = 0
success = 0
line = ""
while not done:
    data = clisock.recv(1024)
    line = line + data
    if line.find('\n') != -1:
        done = 1;
        if re.match('gkrellm 2\.\d\.\d', line):
            success = 1

preamble = """<gkrellmd_setup>
<decimal_point>
.

<cpu_setup>
n_cpus 1
<hostname>
%(hostname)s
<sysname>
JanosVM
<time>
%(time)s
<monitors>
cpu
proc
mem
disk
net
uptime
</gkrellmd_setup>
"""

cpu_format = """<cpu>
0 %(cpu_user)d %(cpu_nice)d %(cpu_sys)d %(cpu_idle)d
"""

proc_format = """<proc>
%(proc_num)d %(proc_num)d 0 0.0 1
"""

mem_format = """<mem>
%(mem_total)d %(mem_used)d %(mem_free)d 0 0 0
"""

disk_format = """<disk>
all %(disk_in)d %(disk_out)d
"""

net_format = """<net>
net_all %(net_in)d %(net_out)d
"""

uptime_format = """<uptime>
%(uptime)f
"""

initial_update = """<initial_update>
%(cpu)s%(disk)s%(net)s
<net_routed>
net_all 1
<time>
%(time)s
</initial_update>
"""

def gktime():
    tm = time.localtime()
    return (str(tm[5])
            + " "
            + str(tm[4])
            + " "
            + str(tm[3])
            + " "
            + str(tm[2])
            + " "
            + str(tm[1])
            + " "
            + str(tm[0] - 1900)
            + " "
            + str(tm[6])
            + " "
            + str(tm[7])
            + " "
            + str(tm[8]))

if success:
    config = {}
    config["hostname"] = jvm.machine().getName()
    config["time"] = gktime()
    config["cpu_user"] = 0
    config["cpu_nice"] = 0
    config["cpu_sys"] = 0
    config["cpu_idle"] = 0
    config["cpu"] = cpu_format % config
    config["disk_in"] = 0
    config["disk_out"] = 0
    config["disk"] = disk_format % config
    config["net_in"] = 0
    config["net_out"] = 0
    config["net"] = net_format % config
    clisock.sendall(preamble % config)
    clisock.sendall(initial_update % config)
    while 1:
        #config["time"] = gktime()
        cpu = jvm.machine().getCPU()
        config["cpu_user"] = cpu[0]
        config["cpu_idle"] = cpu[1]
        try:
            config["cpu_sys"] = long(jvm.team(1).getResource("cpu").
                                     find("cpu.used"))
            config["proc_num"] = len(jvm.teams().list())
        except:
            config["cpu_sys"] = 0
            config["proc_num"] = 1

        mem = jvm.machine().getMemory()
        config["mem_used"] = mem[0]
        config["mem_total"] = mem[1]
        config["mem_free"] = mem[1] - mem[0]

        uptime = jvm.machine().getUptime()
        config["uptime"] = long(float(uptime) / (60.0 * 1000.0 * 1000.0))

        io = jvm.machine().getIO()
        config["disk_in"] = io["disk"][0]
        config["disk_out"] = io["disk"][1]
        config["net_in"] = io["network"][0]
        config["net_out"] = io["network"][1]
        
        clisock.sendall(cpu_format % config)
        clisock.sendall(disk_format % config)
        clisock.sendall(net_format % config)
        clisock.sendall(proc_format % config)
        clisock.sendall(mem_format % config)
        clisock.sendall(uptime_format % config)
        time.sleep(0.5)
