
#include "config.h"

#if defined(TRANSLATOR)

#include <stdio.h>

#include <jni.h>
#include <kaffe/version.h>

#include "../../libltdl/ltdl.h"
#include "debug.h"
#include "itypes.h"
#include "module.h"
#include "moduleList.h"
#include "classMethod.h"
#include "jthread.h"

#include <xprof/debugFile.h>

#include <misc/stringParsing.h>

JavaVMInitArgs vmargs;
JNIEnv* env;
int noExit = 0;

extern JavaVMInitArgs Kaffe_JavaVMArgs[1]; // XXX

extern int internal_test(parsedString *ps);

moduleInitializer_t jitModuleInitList[] = {
	{ &classPathModule, 0 },
	{ &debugModule, 0 },
	{ &lockPoolModule, 0 },
#if defined(INIT_MD)
	{ &machineDependentModule, 0 },
#endif /* defined(INIT_MD) */
	{ &sectionFileModule, 0 },
	{ &dirFileTypeModule, 0 },
	{ &jarFileTypeModule, 0 },
	{ &jteamModule, 0 },
	{ &resourceModule, 0 },
	{ &memResourceModule, 0 },
#if defined(UNIX_JTHREADS2)
	{ &jskedResourceModule, 0 },
#endif /* defined(UNIX_JTHREADS2) */
	{ &gcModule, 0 },
#if defined(UNIX_JTHREADS2)
	{ &fsResourceModule, 0 },
#endif /* defined(UNIX_JTHREADS2) */
	{ &logResourceModule, 0 },
	{ &stdioResourceModule, 0 },
	{ &gcModule, 1 },
	{ &threadModule, 0 },
	{ &gcModule, 2 },
	{ &utfModule, 0 },
#if defined(KAFFE_XDEBUGGING)
	{ &xdebugModule, 0 },
#endif /* defined(KAFFE_XDEBUGGING) */
#if defined(UNIX_JTHREADS2)
	{ &jskedResourceModule, 1 },
#endif /* defined(UNIX_JTHREADS2) */
	{ &nativeResourceModule, 0 },
	{ &classMethodModule, 0 },
#if defined(UNIX_JTHREADS2)
	{ &fsResourceModule, 1 },
#endif /* defined(UNIX_JTHREADS2) */
	{ &classPathModule, 1 },
	{ &classGroupModule, 0 },
	{ &nameSpaceModule, 0 },
	{ 0, 0 },
};

/* If not otherwise specified, assume at least 1MB for main thread */
#ifndef MAINSTACKSIZE
#define MAINSTACKSIZE (1024*1024)
#endif

#define	CLASSPATH1	"KAFFECLASSPATH"
#define	CLASSPATH2	"CLASSPATH"
#define TEST_CLASSES	"TEST_CLASSES"

extern Hjava_lang_Class* ObjectClass;
extern Hjava_lang_Class* javaLangThrowable;
extern Hjava_lang_Class* javaLangArrayIndexOutOfBoundsException;

int main(int argc, char *argv[])
{
	int retval = 1;
	char *cp;

	dbgSetMaskStr("MOREJIT,JIT");
	vmargs.version = JAVA_VERSION_HEX;
	JNI_GetDefaultJavaVMInitArgs(&vmargs);
	
#if defined(KAFFE_XDEBUGGING)
	machine_debug_filename = "xdb.as";
#endif

	/* set up libtool/libltdl dlopen emulation */
	LTDL_SET_PRELOADED_SYMBOLS();

	cp = getenv(CLASSPATH1);
	if (cp == 0) {
		cp = getenv(CLASSPATH2);
#if defined(DEFAULT_CLASSPATH)
		if (cp == 0) {
			cp = DEFAULT_CLASSPATH;
		}
#endif
	}
	vmargs.classpath = cp;
	
	Kaffe_JavaVMArgs[0] = vmargs;
	if( initModules(jitModuleInitList) )
	{
		Hjava_lang_Thread mainThread;
		parsedString testName;
		char *tests;
		
		initTypes();
		loadStaticClass(&ObjectClass, "java/lang/Object");
		loadStaticClass(&javaLangThrowable, "java/lang/Throwable");
		loadStaticClass(&javaLangArrayIndexOutOfBoundsException, "java/lang/ArrayIndexOutOfBoundsException");
		memset(&mainThread, 0, sizeof(mainThread));
		jthread_createfirst(MAINSTACKSIZE,
				    java_lang_Thread_NORM_PRIORITY,
				    &mainThread);

		if( (tests = getenv(TEST_CLASSES)) )
		{
			retval = parseString(
				tests,
				SPO_NotEmpty,
				  SPO_Do,
				    SPO_NonEmptyString, &testName,
				    SPO_Expect, ".class",
				    SPO_WhileSpace,
				    SPO_Handle, internal_test, &testName,
				    SPO_End,
				  SPO_End,
				SPO_End);
			retval = !retval;
		}
		else
		{
			int lpc;

			for( lpc = 1; lpc < argc; lpc++ )
			{
				testName.data = argv[lpc];
				testName.len = strlen(argv[lpc]);
				internal_test(&testName);
			}
		}
	}
	else
	{
		fprintf(stderr, "Modules failed to initialize\n");
	}
	return( retval );
}

#else

int main(int argc, char *argv[])
{
	return( 0 );
}

#endif
