
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamEvent;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.EventTeamHandle;
import edu.utah.janosvm.sys.TeamReservation;
import edu.utah.janosvm.sys.TeamEventHandler;
import edu.utah.janosvm.sys.TeamEjectionException;

import edu.utah.janosvm.resources.CPUSpec;
import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.CPUResource;
import edu.utah.janosvm.resources.ResourceSpec;

class CPULimit
{
    /**
     * Silly infinite loop class that will suck up CPU resources.
     */
    static class CPUMuncher
	implements Runnable
    {
	public void run()
	{
	    for( ;; )
	    {
		Thread.yield();
	    }
	}
    }

    static class ChildState
    {
	int state = TeamEvent.NONE;
    }
    
    public static void main(String args[])
	throws Throwable
    {
	final ChildState cs = new ChildState();
	final Team parent = Team.current();
	EventTeamHandle child;
	
	/* The basic resources for our child team. */
	ResourceSpec rs[] = {
	    new CPUSpec(1000),
	    new MemorySpec(1 * 1024 * 1024),
	    StdioSpec.currentFD(),
	};

	child = new EventTeamHandle(TeamEvent.BURIED, new TeamEventHandler() {
		public void event(EventTeamHandle eth,
				  TeamReservation tr,
				  TeamEvent ev)
		{
		    try
		    {
			tr.visit();
			{
			    synchronized( cs )
			    {
				cs.state = TeamEvent.BURIED;
				cs.notifyAll();
			    }
			}
			tr.leave();
		    }
		    catch(TeamEjectionException e)
		    {
		    }
		}
	    });
	Team.create(child, rs);
	child.switchTo();
	{
	    Team curr = Team.current();

	    curr.addDependency(parent);
	    curr.setEventHandler(new TeamEventHandler() {
		    private int count = 5;
		    
		    public void event(EventTeamHandle eth,
				      TeamReservation tr,
				      TeamEvent ev)
		    {
			switch( ev.getEvent() )
			{
			case TeamEvent.CPU_OVERDRAW:
			    this.count -= 1;
			    if( this.count >= 0 )
			    {
				try
				{
				    tr.visit();
				    {
					System.out.print(".");
				    }
				    tr.leave();
				}
				catch(TeamEjectionException e)
				{
				}
			    }
			    else
			    {
				eth.terminate();
			    }
			    break;
			default:
			    break;
			}
		    }
		});
	    new Thread(new CPUMuncher()).start();
	}
	child.returnFrom();
	synchronized( cs )
	{
	    int lpc;
	    
	    for( lpc = 0; (lpc < 3) && (cs.state != TeamEvent.BURIED); lpc++ )
	    {
		cs.wait(1 * 1000);
	    }
	}
	System.out.print("\n");
	child.terminate();
    }
}

/* Expected Output:
.....
*/
