
import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.MemorySpec;
import edu.utah.janosvm.resources.CPUResource;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.StrideCPUSpec;

class CPUTest
{
    /**
     * How tolerant we should be with differences in measured vs. specified
     * CPU usage.
     */
    static final double TOLERANCE = 0.2;

    /**
     * The basic number of tickets for teams.
     */
    static final int TICKET_BASE = 100;

    /**
     * The multiplier for the low ticket teams.  So their total tickets will
     * be: TICKET_BASE * LO_MULTIPLIER
     */
    static final int LO_MULTIPLIER = 1;
    
    /**
     * The multiplier for the high ticket team.  So its total tickets will be:
     * TICKET_BASE * HI_MULTIPLIER
     */
    static final int HI_MULTIPLIER = 5;

    /**
     * The number of seconds to let the CPUMunchers run before killing the
     * teams.
     */
    static final int DELAY_SECONDS = 1;
    
    /**
     * Silly infinite loop class that will suck up CPU resources.
     */
    static class CPUMuncher
	implements Runnable
    {
	public void run()
	{
	    for( ;; );
	}
    }
    
    public static void main(String args[])
	throws Throwable
    {
	try
	{
	    /* The basic resources for our child teams. */
	    ResourceSpec rs[] = {
		null,
		new MemorySpec(1 * 1024 * 1024),
		StdioSpec.currentFD(),
	    };

	    Team parent = Team.current();
	    TeamHandle thHi, thLo1, thLo2;
	    long usageHi, usageLo1, usageLo2;
	    float ratio, ratio1, ratio2;
	    StrideCPUSpec scs;

	    try
	    {
		/* First, a basic test of parameter checking. */
		new StrideCPUSpec(-1);
		System.out.println("Able to make CPU spec with -1 tickets?");
	    }
	    catch(IllegalArgumentException e)
	    {
	    }

	    try
	    {
		/* First, a basic test of parameter checking. */
		new StrideCPUSpec(0);
		System.out.println("Able to make CPU spec with 0 tickets?");
	    }
	    catch(IllegalArgumentException e)
	    {
	    }

	    try
	    {
		CPUResource.change(null);
		System.out.println("Null parameter to change worked?");
	    }
	    catch(NullPointerException e)
	    {
	    }

	    scs = new StrideCPUSpec(1234);
	    if( scs.getTickets() != 1234 )
	    {
		System.out.println("Bad value returned from getTickets?");
	    }

	    /* Create the three teams. */
	    rs[0] = new StrideCPUSpec(HI_MULTIPLIER * TICKET_BASE);
	    thHi = Team.create(rs);
	    rs[0] = new StrideCPUSpec(LO_MULTIPLIER * TICKET_BASE);
	    thLo1 = Team.create(rs);
	    rs[0] = new StrideCPUSpec(LO_MULTIPLIER * TICKET_BASE);
	    thLo2 = Team.create(rs);

	    /* Switch to the team and start a thread. */
	    thLo2.switchTo();
	    {
		/*
		 * Note: We need to make each child dependent on the parent
		 * team so they can all share the CPUMuncher class.
		 */
		Team.current().addDependency(parent);
		new Thread(new CPUMuncher()).start();
	    }
	    thLo2.returnFrom();
	    
	    thLo1.switchTo();
	    {
		Team.current().addDependency(parent);
		new Thread(new CPUMuncher()).start();
	    }
	    thLo1.returnFrom();
	    
	    thHi.switchTo();
	    {
		Team.current().addDependency(parent);
		new Thread(new CPUMuncher()).start();
	    }
	    thHi.returnFrom();

	    /* Let them suck up the CPU for awhile. */
	    Thread.sleep(DELAY_SECONDS * 1000);

	    /* Switch back and get their usage values. */
	    thLo2.switchTo();
	    {
		if( StrideCPUSpec.current().getTickets() !=
		    (LO_MULTIPLIER * TICKET_BASE) )
		{
		    System.out.println("Current spec doesn't match "
				       + "specified: "
				       + StrideCPUSpec.current());
		}
		usageLo2 = CPUResource.getUsage();
	    }
	    thLo2.returnFrom();
	    
	    thLo1.switchTo();
	    {
		if( StrideCPUSpec.current().getTickets() !=
		    (LO_MULTIPLIER * TICKET_BASE) )
		{
		    System.out.println("Current spec doesn't match "
				       + "specified!");
		}
		usageLo1 = CPUResource.getUsage();
	    }
	    thLo1.returnFrom();
	    
	    thHi.switchTo();
	    {
		if( StrideCPUSpec.current().getTickets() !=
		    (HI_MULTIPLIER * TICKET_BASE) )
		{
		    System.out.println("Current spec doesn't match "
				       + "specified!");
		}
		usageHi = CPUResource.getUsage();
	    }
	    thHi.returnFrom();

	    /* Terminate them... */
	    thLo2.terminate();
	    thLo1.terminate();
	    thHi.terminate();

	    /* Make sure it all worked. */
	    ratio = LO_MULTIPLIER / HI_MULTIPLIER;
	    ratio1 = usageLo1 / usageHi;
	    ratio2 = usageLo2 / usageHi;
	    if( (Math.abs(ratio1 - ratio) < TOLERANCE) &&
		(Math.abs(ratio2 - ratio) < TOLERANCE) )
	    {
		System.out.println("okie");
	    }
	    else
	    {
		System.out.println("usageHi = " + usageHi);
		System.out.println("usageLo1 = " + usageLo1);
		System.out.println("usageLo2 = " + usageLo2);
	    }
	}
	catch(UnsatisfiedLinkError e)
	{
	    /* Schedulers not linked in, ignore it. */
	    System.out.println("okie");
	}
    }
}

/* Expected Output:
okie
*/
