
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.FileSystemSpec;
import edu.utah.janosvm.resources.FileSystemResource;
import edu.utah.janosvm.resources.ResourceSpecException;

/* XXX Needs more File related tests (e.g. stat/mkdir/rmdir) */

class FSTest
{
    private static final String BOGUS_ROOT = "/non-existant-path";
    
    public static void main(String args[])
    {
	FileSystemSpec fss = null;
	Team lt = Team.current();
	TeamHandle child = null;

	try
	{
	    new FileSystemSpec(null);
	    System.out.println("Able to create spec with null root?");
	}
	catch(IllegalArgumentException e)
	{
	}
	
	/*
	 * Test a FileSystemSpec with an empty root.  The team should not be
	 * able to access anything.
	 */
	try
	{
	    ResourceSpec rs[] = {
		fss = new FileSystemSpec(""),
		StdioSpec.currentFD(),
		NameSpaceSpec.current(),
	    };
	    
	    if( !fss.getRoot().equals("") )
	    {
		System.out.println("Spec doesn't have empty root path?");
	    }
	    child = Team.create("Child 1 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				rs);
	    child.switchTo();
	    {
		if( new File((String)"FSTest.class".clone()).length() > 0 )
		{
		    System.out.println(
			"Able to stat a file with an empty root?");
		}
		try
		{
		    new FileInputStream((String)"FSTest.class".clone());
		    System.out.println(
			"Able to open file in team with empty root?");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open file in team with empty root.");
		}
		try
		{
		    new FileOutputStream((String)"foo".clone());
		    System.out.println(
			"Able to create file in team with empty root?");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to create file in team with empty root.");
		}
		try
		{
		    FileSystemResource.change(null);
		    System.out.println("Null parameter to change worked?");
		}
		catch(NullPointerException e)
		{
		}
		try
		{
		    FileSystemResource.change(new FileSystemSpec(BOGUS_ROOT));
		    System.out.println("Bogus parameter to change worked?");
		}
		catch(ResourceSpecException e)
		{
		    System.out.println(e);
		}
		FileSystemResource.change(new FileSystemSpec("/"));
		if( new File((String)"FSTest.class".clone()).length() <= 0 )
		{
		    System.out.println(
			"Unable to stat a file with a changed root?");
		}
		try
		{
		    new FileInputStream((String)"FSTest.class".clone());
		    System.out.println(
			"Able to open file in team with a changed root.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open file in team with a changed root?");
		}
		try
		{
		    new FileOutputStream((String)"fake-file.out".clone());
		    System.out.println(
			"Able to create file in team with a changed root.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to create file in team with a changed root?");
		}
	    }
	    child.returnFrom();
	    child.terminate();
	}
	catch(TeamEjectionException e)
	{
	    e.printStackTrace();
	}
	catch(DuplicateExportException e)
	{
	    e.printStackTrace();
	}
	catch(ResourceSpecException e)
	{
	    e.printStackTrace();
	}
	catch(DeadTeamException e)
	{
	    e.printStackTrace();
	}

	/**
	 * Test a FileSystemSpec with a bogus root, creation should fail.
	 */
	try
	{
	    ResourceSpec rs[] = {
		fss = new FileSystemSpec(BOGUS_ROOT),
	    };

	    if( !fss.getRoot().equals(BOGUS_ROOT) )
	    {
		System.out.println("Spec doesn't have correct root path?");
	    }
	    child = Team.create("Child 2 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				rs);
	    child.terminate();
	    System.out.println("Able to create team with bogus fs root?");
	}
	catch(DeadTeamException e)
	{
	    e.printStackTrace();
	}
	catch(DuplicateExportException e)
	{
	    e.printStackTrace();
	}
	catch(ResourceSpecException e)
	{
	    if( e.getUnsatisfiedSpec() == fss )
	    {
		System.out.println(
		    "Unable to create team with bogus fs root.");
	    }
	    else
	    {
		e.printStackTrace();
	    }
	}

	/*
	 * Test a FileSystemSpec that points to the current directory.  The
	 * team should be able to access this directory and its children, but
	 * nothing else.
	 */
	try
	{
	    ResourceSpec rs[] = {
		new FileSystemSpec(System.getProperty("user.dir")),
		StdioSpec.currentFD(),
	    };

	    child = Team.create("Child 3 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				rs);
	    child.switchTo();
	    {
		try
		{
		    new FileInputStream((String)"/bin/sh".clone());
		    System.out.println(
			"Able to open file in original root directory?");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open file in original root directory.");
		}
		try
		{
		    new FileInputStream((String)"/../regression/Makefile".clone());
		    System.out.println(
			"Able to open file in parent directory?");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open file in parent directory.");
		}
		try
		{
		    new FileOutputStream((String)"/../foo-dash-bar".clone());
		    System.out.println(
			"Able to create file in parent directory.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to create file in parent directory?");
		}
		try
		{
		    new FileInputStream((String)"/foo-dash-bar".clone());
		    System.out.println(
			"Able to open created file in parent directory.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open created file in parent directory? "
			+ e);
		}
		try
		{
		    new FileInputStream((String)"/FSTest.class".clone());
		    System.out.println(
			"Able to open file in new root directory.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to open file in new root directory? " + e);
		}
		try
		{
		    new FileOutputStream((String)"/fake-file2.out".clone());
		    System.out.println(
			"Able to create file in new root directory.");
		}
		catch(FileNotFoundException e)
		{
		    System.out.println(
			"Unable to create file in new root directory? " + e);
		}
	    }
	    child.returnFrom();
	    child.terminate();
	}
	catch(DuplicateExportException e)
	{
	    e.printStackTrace();
	}
	catch(TeamEjectionException e)
	{
	    e.printStackTrace();
	}
	catch(DeadTeamException e)
	{
	    e.printStackTrace();
	}
	catch(ResourceSpecException e)
	{
	    e.printStackTrace();
	}
    }
}

/* Expected Output:
Unable to open file in team with empty root.
Unable to create file in team with empty root.
edu.utah.janosvm.resources.ResourceSpecException: Invalid root directory: /non-existant-path(unsatisfiedSpec=null)
Able to open file in team with a changed root.
Able to create file in team with a changed root.
Unable to create team with bogus fs root.
Unable to open file in original root directory.
Unable to open file in parent directory.
Able to create file in parent directory.
Able to open created file in parent directory.
Able to open file in new root directory.
Able to create file in new root directory.
*/
