
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.DatagramSocket;

import edu.utah.janosvm.sys.Team;
import edu.utah.janosvm.sys.TeamHandle;
import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;
import edu.utah.janosvm.sys.DuplicateExportException;

import edu.utah.janosvm.resources.StdioSpec;
import edu.utah.janosvm.resources.IOResource;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.NameSpaceSpec;
import edu.utah.janosvm.resources.DescriptorIOSpec;
import edu.utah.janosvm.resources.ResourceSpecException;

class IOTest
{
    private static final String ADDRESS = "127.0.0.1";
    private static final int PACKET_SIZE = 128;

    private static void printStats()
    {
	System.out.println("Input:");
	System.out.println("  Bytes = "
			   + IOResource.getIn(IOResource.ACCOUNT_NETWORK));
	System.out.println("  Packets = "
			   + IOResource.getIn(IOResource.
					      ACCOUNT_NETWORK_PACKET));
	System.out.println("Output:");
	System.out.println("  Bytes = "
			   + IOResource.getOut(IOResource.ACCOUNT_NETWORK));
	System.out.println("  Packets = "
			   + IOResource.getOut(IOResource.
					       ACCOUNT_NETWORK_PACKET));
    }
	
    public static void main(String args[])
	throws Throwable
    {
	Team lt = Team.current();
	TeamHandle child = null;
	
	DatagramSocket ds_in, ds_out;
	DatagramPacket dp;
	InetAddress ia;

	ia = InetAddress.getByName(ADDRESS);
	ds_in = new DatagramSocket();
	ds_out = new DatagramSocket();
	
	dp = new DatagramPacket(new byte[PACKET_SIZE],
				PACKET_SIZE,
				ia,
				ds_in.getLocalPort());

	printStats();
	
	ds_out.send(dp);

	printStats();
	
	ds_in.receive(dp);
	
	printStats();

	DescriptorIOSpec dios;
	
	try
	{
	    new DescriptorIOSpec(-1);
	    System.out.println(
		"Able to create DescriptorIOSpec with negative value?");
	}
	catch(IllegalArgumentException e)
	{
	    System.out.println(e);
	}
	try
	{
	    ResourceSpec rs[] = {
		dios = new DescriptorIOSpec(1), // stdout, stderr
		StdioSpec.currentFD(),
		NameSpaceSpec.current(),
	    };

	    if( dios.getCount() != 1 )
	    {
		System.out.println("Bad value returned from getCount?");
	    }

	    child = Team.create("Child 1 of "
				+ lt.getName()
				+ "("
				+ lt.getID()
				+ ")",
				rs);
	    child.switchTo();
	    try
	    {
		new FileInputStream((String)"IOTest.class".clone());
		System.out.println(
		    "Able to open file without descriptors?");
	    }
	    catch(FileNotFoundException e)
	    {
		System.out.println(
		    "Unable to open file without descriptors.");
	    }
	    try
	    {
		new FileOutputStream((String)"foo.out".clone());
		System.out.println(
		    "Able to create file without descriptors?");
	    }
	    catch(FileNotFoundException e)
	    {
		System.out.println(
		    "Unable to create file without descriptors.");
	    }
	    child.returnFrom();
	    child.terminate();
	}
	catch(TeamEjectionException e)
	{
	    e.printStackTrace();
	}
	catch(DuplicateExportException e)
	{
	    e.printStackTrace();
	}
	catch(ResourceSpecException e)
	{
	    e.printStackTrace();
	}
	catch(DeadTeamException e)
	{
	    e.printStackTrace();
	}
    }
}

/* Expected Output:
Input:
  Bytes = 0
  Packets = 0
Output:
  Bytes = 0
  Packets = 0
Input:
  Bytes = 0
  Packets = 0
Output:
  Bytes = 128
  Packets = 1
Input:
  Bytes = 128
  Packets = 1
Output:
  Bytes = 128
  Packets = 1
java.lang.IllegalArgumentException: Negative descriptor count: -1
Unable to open file without descriptors.
Unable to create file without descriptors.
*/
