/*
 * InterruptedLinkReceive.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this file for
 * any purpose is hereby granted without fee, provided that the above
 * copyright notice is maintained in any derived source code.
 */

import java.lang.isolate.Link;
import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateEvent;
import java.lang.isolate.IsolateMessage;
import java.lang.isolate.ClosedLinkException;

public class InterruptedLinkReceive
{
  static boolean alive = false;
  
  public static void main(String args[])
    throws Throwable
  {
    if( args.length > 0 )
    {
      final Object lock = new Object();
      IsolateMessage startIM[];
      final Link link;
      Thread thread;

      startIM = Isolate.currentIsolateStartMessages();
      if( startIM.length > 0 )
      {
	link = startIM[0].getLink();
      }
      else
      {
	link = Link.newLink(Isolate.currentIsolate(),
			    Isolate.currentIsolate());
      }
      thread = new Thread(new Runnable() {
	  public void run()
	  {
	    synchronized( lock )
	    {
	      alive = true;
	      lock.notifyAll();
	    }
	    try
	    {
	      link.receive();
	    }
	    catch (ClosedLinkException cle)
	    {
	      System.out.println("th = " + cle.getClass().getName());
	    }
	    catch(Throwable th)
	    {
	      th.printStackTrace(System.err);
	    }
	    System.out.println("isOpen() = " + link.isOpen());
	    synchronized( lock )
	    {
	      alive = false;
	      lock.notifyAll();
	    }
	  }
	});
      thread.start();
      
      synchronized( lock )
      {
	while( !alive )
	{
	  lock.wait();
	}
      }
      thread.interrupt();
      synchronized( lock )
      {
	while( alive )
	{
	  lock.wait();
	}
      }
    }
    else
    {
      boolean done = false;
      Link link, elink;
      Isolate iso;

      /* Check local links. */
      main(new String[] { "foo" });

      System.out.println("Testing cross isolate link interruption.");
      
      iso = new Isolate("InterruptedLinkReceive", new String[] { "foo" });
      link = Link.newLink(Isolate.currentIsolate(), iso);
      elink = iso.newEventLink();
      iso.start(new IsolateMessage[] {
	IsolateMessage.newLinkMessage(link),
      });
      while( !done )
      {
	IsolateMessage im;
	IsolateEvent ie;
	
	im = elink.receive();
	ie = im.getIsolateEvent();
	if( ie.getType() == IsolateEvent.Type.TERMINATED )
	{
	  done = true;
	}
      }
      System.out.println("isOpen() = " + link.isOpen());
    }
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
th = java.lang.isolate.ClosedLinkException
isOpen() = false
Testing cross isolate link interruption.
th = java.lang.isolate.ClosedLinkException
isOpen() = false
isOpen() = false
*/
