/*
 * IsolateConstructor.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this file for
 * any purpose is hereby granted without fee, provided that the above
 * copyright notice is maintained in any derived source code.
 */

import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;
import java.lang.isolate.IsolatePermission;

import java.security.Permission;

import java.io.FileInputStream;
import java.io.FileOutputStream;

public class IsolateConstructor
{
  public static class HarshSecurityManager
    extends SecurityManager
  {
    public void checkPermission(Permission perm)
    {
      if( perm instanceof RuntimePermission )
      {
      }
      else if( perm instanceof IsolatePermission )
      {
	if( perm.getName().equals("create") )
	{
	  throw new SecurityException();
	}
      }
    }
  }
  
  public static void main(String args[])
    throws Throwable
  {
    FileOutputStream fos;
    FileInputStream fis;

    fos = new FileOutputStream("ic.out");
    fis = new FileInputStream("IsolateConstructor.class");
    
    try
    {
      Isolate iso = new Isolate(null, null);

      System.out.println("Able to make Isolate with null mainClass?");
      iso.start(null);
      iso.halt(0);
    }
    catch(NullPointerException e)
    {
      System.out.println("Unable to make Isolate with null mainClass.");
    }
    try
    {
      Isolate iso = new Isolate("IsolateConstructor", new String[] { null });

      System.out.println("Able to make Isolate with null arguments?");
      iso.start(null);
      iso.halt(0);
    }
    catch(IllegalArgumentException e)
    {
      System.out.println("Unable to make Isolate with null arguments.");
    }
    try
    {
      Isolate iso = new Isolate("IsolateConstructor",
				null,
				null,
				IsolateMessage.newFileOutputStreamMessage(fos),
				null,
				null);

      System.out.println("Able to make Isolate with bogus stdin?");
      iso.start(null);
      iso.halt(0);
    }
    catch(IllegalArgumentException e)
    {
      System.out.println("Unable to make Isolate with bogus stdin.");
    }
    try
    {
      Isolate iso = new Isolate("IsolateConstructor",
				null,
				null,
				null,
				IsolateMessage.newFileInputStreamMessage(fis),
				null);

      System.out.println("Able to make Isolate with bogus stdout?");
      iso.start(null);
      iso.halt(0);
    }
    catch(IllegalArgumentException e)
    {
      System.out.println("Unable to make Isolate with bogus stdout.");
    }
    try
    {
      Isolate iso = new Isolate("IsolateConstructor",
				null,
				null,
				null,
				null,
				IsolateMessage.newFileInputStreamMessage(fis));

      System.out.println("Able to make Isolate with bogus stderr?");
      iso.start(null);
      iso.halt(0);
    }
    catch(IllegalArgumentException e)
    {
      System.out.println("Unable to make Isolate with bogus stderr.");
    }

    System.setSecurityManager(new HarshSecurityManager());
    try
    {
      Isolate iso = new Isolate("IsolateConstructor",
				null,
				null,
				null,
				null,
				IsolateMessage.newFileInputStreamMessage(fis));

      System.out.println("Able to make Isolate with harsh security manager?");
      iso.start(null);
      iso.halt(0);
    }
    catch(SecurityException e)
    {
      System.out.println("Unable to make Isolate with harsh security manager.");
    }
    System.setSecurityManager(null);
    
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
Unable to make Isolate with null mainClass.
Unable to make Isolate with null arguments.
Unable to make Isolate with bogus stdin.
Unable to make Isolate with bogus stdout.
Unable to make Isolate with bogus stderr.
Unable to make Isolate with harsh security manager.
*/
