/*
 * IsolateContext.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this file for
 * any purpose is hereby granted without fee, provided that the above
 * copyright notice is maintained in any derived source code.
 */

import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;
import java.lang.isolate.IsolatePermission;

import java.security.Permission;

public class IsolateContext
{
  public static class NoContextSecurityManager
    extends SecurityManager
  {
    public void checkPermission(Permission perm)
    {
      if( perm instanceof RuntimePermission )
      {
      }
      else if( perm instanceof IsolatePermission )
      {
	if( perm.getName().equals("context") )
	{
	  throw new SecurityException();
	}
      }
    }
  }
  
  public static void main(String args[])
    throws Throwable
  {
    if( args.length > 0 )
    {
      IsolateMessage im[];

      im = Isolate.currentIsolateStartMessages();
      im[0] = null;
      im = Isolate.currentIsolateStartMessages();
      if( im[0] == null )
      {
	System.out.println("Start messages aren't a copy?");
      }
      
      im = Isolate.currentIsolateIOBindings();
      im[1] = null;
      im = Isolate.currentIsolateIOBindings();
      if( im[1] == null )
      {
	System.out.println("IO bindings aren't a copy?");
      }
    }
    else
    {
      Isolate iso;

      iso = new Isolate("IsolateContext", new String[] { "foo" });
      iso.start(new IsolateMessage[] {
	IsolateMessage.newStringMessage("dummy"),
      });
      
      System.setSecurityManager(new NoContextSecurityManager());

      try
      {
	Isolate.currentIsolateIOBindings();
	System.out.println("Able to get current IO bindings with harsh "
			   + "security manager?");
      }
      catch(SecurityException e)
      {
	System.out.println("Unable to get current IO bindings with harsh "
			   + "security manager.");
      }
      try
      {
	Isolate.currentIsolateStartMessages();
	System.out.println("Able to get current start messages with harsh "
			   + "security manager?");
      }
      catch(SecurityException e)
      {
	System.out.println("Unable to get current start messages with harsh "
			   + "security manager.");
      }
      try
      {
	Isolate.currentIsolateContext();
	System.out.println("Able to get current context with harsh "
			   + "security manager?");
      }
      catch(SecurityException e)
      {
	System.out.println("Unable to get current context with harsh "
			   + "security manager.");
      }
    }
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
Unable to get current IO bindings with harsh security manager.
Unable to get current start messages with harsh security manager.
Unable to get current context with harsh security manager.
*/
