
import java.lang.isolate.Link;
import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;
import java.lang.isolate.IsolateMessageVisitor;
import java.lang.isolate.LinkSerializationException;

import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.net.Socket;
import java.net.ServerSocket;
import java.net.DatagramSocket;

public class IsolateMessageConstructor
{
  static final int KIND_BYTE_ARRAY = 0;
  static final int KIND_COMPOSITE = 1;
  static final int KIND_DATAGRAM_SOCKET = 2;
  static final int KIND_FILE_INPUT = 3;
  static final int KIND_FILE_OUTPUT = 4;
  static final int KIND_ISOLATE_EVENT = 5;
  static final int KIND_ISOLATE = 6;
  static final int KIND_LINK = 7;
  static final int KIND_SERIALIZABLE = 8;
  static final int KIND_SERVER_SOCKET = 9;
  static final int KIND_SOCKET = 10;
  static final int KIND_STRING = 11;
  static final int KIND_EMPTY = 12;

  private static boolean gotExpected;
  
  private static void getFail(int kind, int expect)
  {
    if( kind == expect )
      gotExpected = true;
    else
      throw new Error();
  }
  
  private static void checkGetters(IsolateMessage im, int kind)
    throws LinkSerializationException
  {
    gotExpected = false;
    try { im.getByteArray(); getFail(KIND_BYTE_ARRAY, kind); }
    catch(IllegalStateException e) { }
    try { im.getComposite(); getFail(KIND_COMPOSITE, kind); }
    catch(IllegalStateException e) { }
    try { im.getDatagramSocket(); getFail(KIND_DATAGRAM_SOCKET, kind); }
    catch(IllegalStateException e) { }
    try { im.getFileInputStream(); getFail(KIND_FILE_INPUT, kind); }
    catch(IllegalStateException e) { }
    try { im.getFileOutputStream(); getFail(KIND_FILE_OUTPUT, kind); }
    catch(IllegalStateException e) { }
    try { im.getIsolate(); getFail(KIND_ISOLATE, kind); }
    catch(IllegalStateException e) { }
    try { im.getIsolateEvent(); getFail(KIND_ISOLATE_EVENT, kind); }
    catch(IllegalStateException e) { }
    try { im.getLink(); getFail(KIND_LINK, kind); }
    catch(IllegalStateException e) { }
    try { im.getSerializable(); getFail(KIND_SERIALIZABLE, kind); }
    catch(IllegalStateException e) { }
    try { im.getServerSocket(); getFail(KIND_SERVER_SOCKET, kind); }
    catch(IllegalStateException e) { }
    try { im.getSocket(); getFail(KIND_SOCKET, kind); }
    catch(IllegalStateException e) { }
    try { im.getString(); getFail(KIND_STRING, kind); }
    catch(IllegalStateException e) { }
    if( (kind != KIND_EMPTY) && !gotExpected )
      throw new Error();
  }
  
  private static void checkVisitor(IsolateMessage im, int kind)
    throws LinkSerializationException
  {
    gotExpected = false;
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitByteArrayMessage(IsolateMessage im2) { }
	});
      getFail(KIND_BYTE_ARRAY, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitCompositeMessage(IsolateMessage im2) { }
	});
      getFail(KIND_COMPOSITE, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitDatagramSocketMessage(IsolateMessage im2) { }
	});
      getFail(KIND_DATAGRAM_SOCKET, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitFileInputStreamMessage(IsolateMessage im2) { }
	});
      getFail(KIND_FILE_INPUT, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitFileOutputStreamMessage(IsolateMessage im2) { }
	});
      getFail(KIND_FILE_OUTPUT, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitIsolateEventMessage(IsolateMessage im2) { }
	});
      getFail(KIND_ISOLATE_EVENT, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitIsolateMessage(IsolateMessage im2) { }
	});
      getFail(KIND_ISOLATE, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitLinkMessage(IsolateMessage im2) { }
	});
      getFail(KIND_LINK, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitSerializableMessage(IsolateMessage im2) { }
	});
      getFail(KIND_SERIALIZABLE, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitServerSocketMessage(IsolateMessage im2) { }
	});
      getFail(KIND_SERVER_SOCKET, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitSocketMessage(IsolateMessage im2) { }
	});
      getFail(KIND_SOCKET, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitStringMessage(IsolateMessage im2) { }
	});
      getFail(KIND_STRING, kind);
    }
    catch(IllegalStateException e) { }
    try
    {
      im.acceptVisitor(new IsolateMessageVisitor() {
	  public void visitEmptyMessage(IsolateMessage im2) { }
	});
      getFail(KIND_EMPTY, kind);
    }
    catch(IllegalStateException e) { }
    if( (kind != KIND_EMPTY) && !gotExpected )
      throw new Error();
  }
  
  public static void main(String args[])
    throws Throwable
  {
    if( args.length > 0 )
    {
    }
    else
    {
      IsolateMessage im;
      int lpc;

      /** BEGIN Empty **/
      checkGetters(IsolateMessage.EMPTY_MESSAGE, KIND_EMPTY);
      checkVisitor(IsolateMessage.EMPTY_MESSAGE, KIND_EMPTY);

      /** BEGIN Isolate **/
      Isolate iso1, iso2;
      
      iso1 = new Isolate("IsolateMessageConstructor", new String[] { "foo" });
      iso2 = new Isolate("IsolateMessageConstructor", new String[] { "foo" });
      
      try
      {
	IsolateMessage.newIsolateMessage(null);
	System.out.println("Able to make isolate message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      try
      {
	IsolateMessage.newIsolateMessage(iso1);
	System.out.println("Able to make isolate message with unstarted arg?");
      }
      catch(IllegalStateException e)
      {
      }

      /** BEGIN Link **/
      Link link = Link.newLink(iso1, iso2);
      
      try
      {
	IsolateMessage.newLinkMessage(null);
	System.out.println("Able to make link message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newLinkMessage(link);
      checkGetters(im, KIND_LINK);
      checkVisitor(im, KIND_LINK);

      /** BEGIN ByteArray **/
      byte ba[] = new byte[32];
      
      try
      {
	IsolateMessage.newByteArrayMessage(null);
	System.out.println("Able to make byte array message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      
      im = IsolateMessage.newByteArrayMessage(ba);
      if( ba != im.getByteArray() )
	System.out.println("Byte array message made a copy?");
      checkGetters(im, KIND_BYTE_ARRAY);
      checkVisitor(im, KIND_BYTE_ARRAY);

      /** BEGIN String **/
      try
      {
	IsolateMessage.newStringMessage(null);
	System.out.println("Able to make string message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newStringMessage("foo");
      checkGetters(im, KIND_STRING);
      checkVisitor(im, KIND_STRING);

      /** BEGIN ServerSocket **/
      ServerSocket ss = new ServerSocket(0);

      try
      {
	IsolateMessage.newServerSocketMessage(null);
	System.out.println("Able to make string message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newServerSocketMessage(ss);
      checkGetters(im, KIND_SERVER_SOCKET);
      checkVisitor(im, KIND_SERVER_SOCKET);

      /** BEGIN Socket **/
      Socket socket = new Socket("localhost", ss.getLocalPort());

      try
      {
	IsolateMessage.newSocketMessage(null);
	System.out.println("Able to make socket message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newSocketMessage(socket);
      checkGetters(im, KIND_SOCKET);
      checkVisitor(im, KIND_SOCKET);

      /** BEGIN DatagramSocket **/
      DatagramSocket ds = new DatagramSocket();

      try
      {
	IsolateMessage.newDatagramSocketMessage(null);
	System.out.println("Able to make datagram socket message with "
			   + "null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newDatagramSocketMessage(ds);
      checkGetters(im, KIND_DATAGRAM_SOCKET);
      checkVisitor(im, KIND_DATAGRAM_SOCKET);

      /** BEGIN FileInputStream **/
      FileInputStream fis;

      fis = new FileInputStream("IsolateMessageConstructor.class");
      try
      {
	IsolateMessage.newFileInputStreamMessage(null);
	System.out.println("Able to make file input stream message with "
			   + "null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newFileInputStreamMessage(fis);
      checkGetters(im, KIND_FILE_INPUT);
      checkVisitor(im, KIND_FILE_INPUT);
      
      /** BEGIN FileOutputStream **/
      FileOutputStream fos;

      fos = new FileOutputStream("foo");
      try
      {
	IsolateMessage.newFileOutputStreamMessage(null);
	System.out.println("Able to make file output stream message with "
			   + "null arg?");
      }
      catch(NullPointerException e)
      {
      }
      im = IsolateMessage.newFileOutputStreamMessage(fos);
      checkGetters(im, KIND_FILE_OUTPUT);
      checkVisitor(im, KIND_FILE_OUTPUT);
      
      /** BEGIN Composite **/
      try
      {
	IsolateMessage.newCompositeMessage(null);
	System.out.println("Able to make composite message with null arg?");
      }
      catch(NullPointerException e)
      {
      }
      try
      {
	IsolateMessage.newCompositeMessage(new IsolateMessage[] { null });
	System.out.println("Able to make composite message with null "
			   + "element?");
      }
      catch(IllegalArgumentException e)
      {
      }
      try
      {
	IsolateMessage.newCompositeMessage(new IsolateMessage[0]);
      }
      catch(Throwable e)
      {
	System.out.println("Unable to make composite message with zero "
			   + "elements? " + e);
      }

      IsolateMessage ims[] = new IsolateMessage[] {
	IsolateMessage.newStringMessage("foo"),
	IsolateMessage.newStringMessage("bar"),
      };

      im = IsolateMessage.newCompositeMessage(ims);

      if( im.getComposite() == ims )
	System.out.println("Composite array isn't a copy?");
      checkGetters(im, KIND_COMPOSITE);
      checkVisitor(im, KIND_COMPOSITE);
      for( lpc = 0; lpc < ims.length; lpc++ )
      {
	if( im.getComposite()[lpc] != ims[lpc] )
	{
	  System.out.println("Composite element at index "
			     + lpc
			     + " is incorrect?");
	}
      }
      im.getComposite()[0] = null;
      if( im.getComposite()[0] == null )
	System.out.println("getComposite() doesn't return a copy?");

      System.out.println("done");
    }
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
done
*/
