/*
 * IsolateStart.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this file for
 * any purpose is hereby granted without fee, provided that the above
 * copyright notice is maintained in any derived source code.
 */

import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;

public class IsolateStart
{
  private static final void delay(int millis)
  {
    try
    {
      Thread.sleep(millis);
    }
    catch (InterruptedException ie)
    {
    }
  }

  public static void main(String args[])
    throws Throwable
  {
    if( args.length > 0 )
    {
      Thread.currentThread().setName("IsolateStart child " +args[0]);
    }
    else
    {
      Thread.currentThread().setName("IsolateStart Main Thread");

      Isolate iso = new Isolate("IsolateStart", new String[] { "foo1" });

      try
      {
	iso.start(new IsolateMessage[] { null });
	System.out.println("Able to start isolate with null message?");
      }
      catch(IllegalArgumentException e)
      {
	System.out.println("Unable to start isolate with null message.");
      }

      if( iso.hasStarted() )
	System.out.println("hasStarted() is wrong?");
      if( iso.hasExited() )
	System.out.println("hasExited() is wrong?");
      if( iso.hasTerminated() )
	System.out.println("hasTerminated() is wrong?");
      iso.start(null);
      if( !iso.hasStarted() )
	System.out.println("hasStarted() is wrong?");
      try
      {
	iso.start(null);
	System.out.println("Able to restart isolate?");
      }
      catch(IllegalStateException e)
      {
	System.out.println("Unable to restart isolate: " + e.getClass().getName());
      }
      iso.halt(0);
      delay(500);// halt is not synchronous, so wait for it to happen... should be quick
      if( !iso.hasTerminated() )
	System.out.println("hasTerminated() is wrong?");
      try
      {
	iso.start(null);
	System.out.println("Able to start isolate after termination?");
      }
      catch(IllegalStateException e)
      {
	System.out.println("Unable to start isolate after termination: "
			   + e.getClass().getName());
      }
    }
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
Unable to start isolate with null message.
Unable to restart isolate: java.lang.IllegalStateException
Unable to start isolate after termination: java.lang.IllegalStateException
*/
