/*
 * LinkEqualityCheck.java
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this file for
 * any purpose is hereby granted without fee, provided that the above
 * copyright notice is maintained in any derived source code.
 */

import java.lang.isolate.Link;
import java.lang.isolate.Isolate;
import java.lang.isolate.IsolateMessage;
import java.lang.isolate.ClosedLinkException;

public class LinkEqualityCheck
{
  static final boolean verbose = false;
  static final Object ioLock = new Object();
  
  public static void main(String[] args)
    throws Exception
  {
    if (args.length > 0)
      reflector();
    else
    {
      Isolate me = Isolate.currentIsolate();
      Isolate reflector = new Isolate(LinkEqualityCheck.class.getName(), new String[] { "doreflector" });
      Link me2ref = Link.newLink(me, reflector);
      Link ref2me = Link.newLink(reflector, me);
      reflector.start(new IsolateMessage[] { 
	IsolateMessage.newLinkMessage(me2ref), 
	  IsolateMessage.newLinkMessage(ref2me) });
	
      generator(reflector, me2ref, ref2me);

      reflector.exit(0);
      System.exit(0);
    }
  }

  // Just reflect everything received back on send link
  static void reflector()
    throws Exception
  {
    IsolateMessage[] m = Isolate.currentIsolateStartMessages();
    Link recvLink = m[0].getLink();
    Link sendLink = m[1].getLink();
    
    try
    {
      while (true)
      {
	sendLink.send(recvLink.receive());
      }
    }
    catch (ClosedLinkException cle)
    {
      return;
    }
  }
  
  // generator
  static void generator(Isolate reflector, Link sendLink, Link recvLink)
    throws Exception
  {
    System.out.println("sendLink.equal(recvLink)=" +sendLink.equals(recvLink)+ ".");

    sendLink.send(IsolateMessage.newLinkMessage(sendLink));
    Link sendClone = recvLink.receive().getLink();
    System.out.println("sendLink comes back equal? " +sendClone.equals(sendLink)+ ".");
    
    sendLink.send(IsolateMessage.newLinkMessage(recvLink));
    Link recvClone = recvLink.receive().getLink();
    System.out.println("recvLink comes back equal? " +recvClone.equals(recvLink)+ ".");

    Link evLink = reflector.newEventLink();
    sendLink.send(IsolateMessage.newLinkMessage(evLink));
    Link evClone = recvLink.receive().getLink();
    System.out.println("evLink comes back equal? " +evClone.equals(evLink)+ ".");
    
  }
}

// Loader: edu.utah.janosvm.kit.loader.Loader

/* Expected Output:
*/
