/*
 * TransientPrefsTest.java
 *
 * Test for java.util.prefs.TransientPreferences.
 */

import java.io.IOException;
import java.util.prefs.Preferences;
import java.util.prefs.TransientPreferences;
import java.util.prefs.BackingStoreException;

/*
 * TODO:
 *   + InputStream constructor
 *
 * Prefs constructor
 * Merge
 */

public class TransientPrefsTest
{
	private static void printSubtree(String name, Preferences p)
		throws Exception
	{
		System.out.println();
		System.out.println(" === " +name+ " === ");

		// p.exportSubtree(System.out);
		printSubtreeIndent(name, p, " ");
	}

	private static void printSubtreeIndent(String name, Preferences p, String indent)
		throws Exception
	{
		System.out.println(indent+ "'" +name+"' ->");

		String[] keys = p.keys();
		for (int i = 0; i < keys.length; i++)
		{
			String val = p.get(keys[i], null);
			if (val != null)
				System.out.println(indent+ "\"" +keys[i]+ "\" : \"" +val+ "\"");
			else 
				System.out.println(indent+ "\"" +keys[i]+ "\" <missing?>");
		}
		String newIndent = indent+" ";
		String[] children = p.childrenNames();
		for (int i = 0; i < children.length; i++)
		{
			printSubtreeIndent(children[i], p.node(children[i]), newIndent);
		}
	}

	public static void main(String[] args)
		throws Exception
	{
		TransientPreferences tp = new TransientPreferences();
		tp.put("string", "string value");
		tp.putBoolean("bool", false);
		tp.putInt("int", 11);
		tp.putLong("long", 34);
		
		Preferences p = tp.node("this/is/a/fairly/deep/node/in/the/transient/prefs/tree");
		p.put("string", "string value");
		
		// Print whole tree
		printSubtree("Whole Tree", tp);

		// Print sub-tree
		printSubtree("Sub Tree", tp.node("this/is/a/fairly/deep/node"));


		// Merge tests
		TransientPreferences tp2 = new TransientPreferences();
		tp2.put("string2", "string2 value");
		tp2.put("string", "OVERRIDDEN string value");
		tp2.putInt("int", 42);

		tp.merge(tp2);

		printSubtree("Tree with merge", tp);
		printSubtree("Merged bits", tp2);
	}
}

/* Expected Output:

 === Whole Tree === 
 'Whole Tree' ->
 "long" : "34"
 "string" : "string value"
 "int" : "11"
 "bool" : "false"
  'this' ->
   'is' ->
    'a' ->
     'fairly' ->
      'deep' ->
       'node' ->
        'in' ->
         'the' ->
          'transient' ->
           'prefs' ->
            'tree' ->
            "string" : "string value"

 === Sub Tree === 
 'Sub Tree' ->
  'in' ->
   'the' ->
    'transient' ->
     'prefs' ->
      'tree' ->
      "string" : "string value"

 === Tree with merge === 
 'Tree with merge' ->
 "string2" : "string2 value"
 "long" : "34"
 "string" : "OVERRIDDEN string value"
 "int" : "42"
 "bool" : "false"
  'this' ->
   'is' ->
    'a' ->
     'fairly' ->
      'deep' ->
       'node' ->
        'in' ->
         'the' ->
          'transient' ->
           'prefs' ->
            'tree' ->
            "string" : "string value"

 === Merged bits === 
 'Merged bits' ->
 "string2" : "string2 value"
 "string" : "OVERRIDDEN string value"
 "int" : "42"
*/
